

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>

extern "C" {
#include "callbacks.h"
#include "interface.h"
#include "support.h"
}
#include "ControlCenterClientWrapper.h"

int MAX_PKT_RATE=112; //7*16

gboolean
on_window_main_destroy           (GtkWidget       *widget,
                                        gpointer         user_data)
{
  gtk_exit(0);
  return FALSE;
}

void
on_button_user_require_clicked         (GtkButton       *button,
                                        gpointer         user_data)
{
  server->assignFilter(FILTER);
  printf(" [on_button_user_require_clicked(): (%x) applied.\n",FILTER);
}


void
on_checkbutton_gzip_toggled            (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
  if (gtk_toggle_button_get_active(togglebutton)) {
    SET_FILTER(GZIP);
  }
  else {
    UNSET_FILTER(GZIP);
  }
}


void
on_checkbutton_shrink_toggled          (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
  if (gtk_toggle_button_get_active(togglebutton)) {
    SET_FILTER(SHRINK);
  }
  else {
    UNSET_FILTER(SHRINK);
  }
}


void
on_checkbutton_grey_toggled            (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
  if (gtk_toggle_button_get_active(togglebutton)) {
    SET_FILTER(GREY);
  }
  else {
    UNSET_FILTER(GREY);
  }
}


void
on_checkbutton_jpeg_toggled            (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
  if (gtk_toggle_button_get_active(togglebutton)) {
    SET_FILTER(JPEG1);
  }
  else {
    UNSET_FILTER(JPEG1);
  }
}


void
on_checkbutton_encrypt_toggled         (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
  if (gtk_toggle_button_get_active(togglebutton)) {
    SET_FILTER(ENCRYPT);
  }
  else {
    UNSET_FILTER(ENCRYPT);
  }
}


void
on_checkbutton_flowCtrl_toggled         (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
  if (gtk_toggle_button_get_active(togglebutton)) {
    FILTER |= 0x80000000;
  }
  else {
    FILTER &= 0x7FFFFFFF;
  }
}


void
on_button_filter_clicked               (GtkButton       *button,
                                        gpointer         user_data)
{
  server->assignFilter(FILTER);
  printf(" [on_button_filter_clicked(): (%x) applied.\n",FILTER);
}



void
on_hscale_fps_value_changed     (GtkRange        *range,
                                        gpointer         user_data)
{
  guint fps;
  fps = gtk_range_get_value(range);

  if (fps < 0 ) fps = 1;
  else if (fps > MAX_PKT_RATE) fps = MAX_PKT_RATE;
  //The fps uses the 2nd byte in FILTER
  FILTER &= 0xFF00FFFF;
  FILTER |= fps<<16;
  printf(" [on_hscale_fps_value_changed(): (%d)\n",fps);
}


guint on_measurement_update(gpointer data)
{
  char line[256] ;
  char filter_str[9];
  static int wc=0;
  static unsigned int last_last_seq_nr = 0;
  int nr;
  GtkTextIter    iter1, iter2;

  float bw; 
  int jitter;
  int pkt_rate;
  float miss_rate;
  unsigned int last_seq_nr;
  float fps;

  GTimeVal t;

  server->getMeasurements(bw, jitter, pkt_rate, miss_rate, last_seq_nr, fps);

  if (last_seq_nr == last_last_seq_nr) 
    return TRUE;
  else
    last_last_seq_nr = last_seq_nr;

  //Visualize filter 
  if (FILTER & 1) filter_str[7] = 'X'; else filter_str[7]='o';
  if (FILTER & 2) filter_str[6] = 'X'; else filter_str[6]='o';
  if (FILTER & 4) filter_str[5] = 'X'; else filter_str[5]='o';
  if (FILTER & 8) filter_str[4] = 'X'; else filter_str[4]='o';
  if (FILTER & 16) filter_str[3] = 'X'; else filter_str[3]='o';
  if (FILTER & 32) filter_str[2] = 'X'; else filter_str[2]='o';
  if (FILTER & 64) filter_str[1] = 'X'; else filter_str[1]='o';
  if (FILTER & 128) filter_str[0] = 'X'; else filter_str[0]='o';
  filter_str[8]='\0';

  g_get_current_time (&t);
  GtkTextBuffer* gtb = gtk_text_view_get_buffer(GTK_TEXT_VIEW(textview_log));

  //Get the current iterator offset
  gtk_text_buffer_get_iter_at_offset(gtb, &iter1, wc);

  nr = sprintf(line, " [%d:%d] last_seq_nr: %d, bw: %6.2f Mbps, jitter: %d ms, fps: %6.2f, missing_rate: %5.2f.  [%s]\n\n", t.tv_sec, t.tv_usec, last_seq_nr, bw, jitter/1000, fps, miss_rate, filter_str);
	  
  gtk_text_buffer_insert (gtb, &iter1, line, nr);
  gtk_text_buffer_get_end_iter(gtb, &iter1);
  gtk_text_view_scroll_to_iter(GTK_TEXT_VIEW(textview_log), &iter1,
			       0,TRUE,0,0);

  wc+=nr;
  if (wc>= 3000) {
    gtk_text_buffer_get_start_iter(gtb, &iter1);
    gtk_text_buffer_get_end_iter(gtb, &iter2);
    gtk_text_buffer_delete(gtb, &iter1, &iter2);
    wc = 0;
  }

  return TRUE;
}




void
on_optionmenu_size_changed           (GtkOptionMenu   *optionmenu,
				      gpointer         user_data)
{
  gint i; 
  i = gtk_option_menu_get_history(optionmenu);
  if (0==i) 
    UNSET_FILTER(SHRINK);
  else if (1==i)
    SET_FILTER(SHRINK);
}

void
on_optionmenu_color_changed           (GtkOptionMenu   *optionmenu,
				      gpointer         user_data)
{
  gint i; 
  i = gtk_option_menu_get_history(optionmenu);
  if (0==i) 
    UNSET_FILTER(GREY);
  else if (1==i)
    SET_FILTER(GREY);
}


void
on_optionmenu_compress_changed           (GtkOptionMenu   *optionmenu,
				      gpointer         user_data)
{
  gint i; 
  i = gtk_option_menu_get_history(optionmenu);
  if (0==i) 
    SET_FILTER(GZIP);
  else if (1==i)
    UNSET_FILTER(GZIP);
}

void
on_optionmenu_lossy_changed           (GtkOptionMenu   *optionmenu,
				      gpointer         user_data)
{
  gint i; 
  i = gtk_option_menu_get_history(optionmenu);
  if (0==i) 
    SET_FILTER(JPEG1);
  else if (1==i)
    UNSET_FILTER(JPEG1);
}


void
on_optionmenu_encrypt_changed           (GtkOptionMenu   *optionmenu,
				      gpointer         user_data)
{
  gint i; 
  i = gtk_option_menu_get_history(optionmenu);
  if (0==i) 
    SET_FILTER(ENCRYPT);
  else if (1==i)
    UNSET_FILTER(ENCRYPT);
}

