/*
 * $Id: cfg_info_parse.c,v 1.1.1.1 2003/06/11 19:05:38 wyan Exp $
 * $Log: cfg_info_parse.c,v $
 * Revision 1.1.1.1  2003/06/11 19:05:38  wyan
 * UAV2 includes AVStreams and C versions
 *
 * Revision 1.2  2002/11/22 00:34:10  wyan
 * *** empty log message ***
 *
 * $Revision: 1.1.1.1 $
 */
#include <sys/types.h>
#include <sys/stat.h>

#include <fcntl.h>
#include <glib.h>
#include <stdio.h>
#include <stdlib.h>
#include "cfg_info_parse.h"

struct profile {
  char* name;
  GList* filters;
};
  
static struct profile* curr_profile;

GScannerConfig gsconfig =
{
  (
   " \t\r\n"
   )                    /* cset_skip_characters */,
  (
   G_CSET_a_2_z
   "_"
   G_CSET_A_2_Z
   )                    /* cset_identifier_first */,
  (
   G_CSET_a_2_z
   "_-0123456789"
   G_CSET_A_2_Z
   )                    /* cset_identifier_nth */,
  ( "#\n" )             /* cpair_comment_single */,
  
  TRUE                  /* case_sensitive */,
  
  TRUE                  /* skip_comment_multi */,
  TRUE                  /* skip_comment_single */,
  TRUE                  /* scan_comment_multi */,
  TRUE                  /* scan_identifier */,
  FALSE                 /* scan_identifier_1char */,
  FALSE                 /* scan_identifier_NULL */,
  TRUE                  /* scan_symbols */,
  TRUE                  /* scan_binary */,
  TRUE                  /* scan_octal */,
  TRUE                  /* scan_float */,
  TRUE                  /* scan_hex */,
  TRUE                  /* scan_hex_dollar */,
  TRUE                  /* scan_string_sq */,
  TRUE                  /* scan_string_dq */,
  TRUE                  /* numbers_2_int */,
  FALSE                 /* int_2_float */,
  FALSE                 /* identifier_2_string */,
  TRUE                  /* char_2_token */,
  TRUE                  /* symbol_2_token */,
  FALSE                 /* scope_0_fallback */,
};


typedef enum {
  ISG_TOKEN_INVALID = G_TOKEN_LAST,
  ISG_TOKEN_fps_profile,
  ISG_TOKEN_resolution_profile,
  ISG_TOKEN_filter,
  ISG_TOKEN_LAST
} ISG_token_type;

struct
{
  gchar *name;
  guint token;
} symbols[] = {
  { "fps_profile",        ISG_TOKEN_fps_profile },
  { "resolution_profile", ISG_TOKEN_resolution_profile },
  { "filter",             ISG_TOKEN_filter},
};



void parse_error(guint token)
{
  int i;
  for (i=0; i < G_N_ELEMENTS (symbols) ; i++) {
    if (token == symbols[i].token) {
      printf(" Error: token %s expected!\n",symbols[i].name);
      return;
    }
  }
 
  printf(" Error: unexpected token!\n");
}


/* 
 * Append the found file names to various string file name list
 * Used in parsing each demo cfg file like 'UAV.conf'.
 * demo: the name of the current demo
 * root: the returned root dir stored here.
 * profile_type: which file name list should be appended.
 */
guint parse_filter_names ( GScanner* scanner,  guint profile_type)
{
  guint i, token;
  struct profile* pf = NULL;
  gchar* str;

  for (i=0; i< g_list_length(ALL_PROFILE); i++) {
    pf = (struct profile*)g_list_nth_data(ALL_PROFILE, i);
    if (NULL != pf) {
      if (profile_type == ISG_TOKEN_fps_profile) {
	if ( 0 == g_ascii_strcasecmp(pf->name, "fps_profile") )
	  break;
      }
      else if (profile_type == ISG_TOKEN_resolution_profile) {
	if ( 0 == g_ascii_strcasecmp(pf->name, "resolution_profile"))
	  break;
      }
    }
  }

  if (NULL == pf) {
    printf(" [parse_filter_names(): Name doesn't match!\n");
    exit(-1);
  }

  token = g_scanner_get_next_token(scanner);//eat ISG_TOKEN_filter_name
  token = g_scanner_peek_next_token(scanner);

  if (token != '=') {
    parse_error('=');
  }

  token = g_scanner_get_next_token(scanner);//eat '='
  token = g_scanner_peek_next_token(scanner);
  if (token != G_TOKEN_STRING) {
    parse_error(G_TOKEN_STRING);
  }

  str = g_strdup(scanner->next_value.v_string);
 
  switch (profile_type) 
    {
    case ISG_TOKEN_fps_profile:
      pf->filters = g_list_append(pf->filters, g_strdup(str));
      break;
    case ISG_TOKEN_resolution_profile:
      pf->filters = g_list_append(pf->filters, g_strdup(str));
      break;
    default:
      printf(" [parse_filter_names(): Error profile_type");
    }

  g_free(str);

  return G_TOKEN_STRING;  
}



/* 
 *  Start parsing.
 * demo: the name of the current demo
 * root: the returned root dir stored here.
 */
guint parse_conf_statement ( GScanner *scanner) 
{
  guint token;
  static guint profile_type;
  token = g_scanner_peek_next_token(scanner);

  switch (token) 
    {
    case ISG_TOKEN_fps_profile:
      profile_type = token;
      token = g_scanner_get_next_token(scanner);
      curr_profile = (struct profile*)g_malloc(sizeof(struct profile));
      curr_profile->name = g_strdup("fps_profile");
      curr_profile->filters = NULL;
      ALL_PROFILE = g_list_append (ALL_PROFILE, curr_profile); 
      return token;

    case ISG_TOKEN_resolution_profile:
      //printf(" case ISG_TOKEN_resolution_profile\n");
      profile_type = token;
      token = g_scanner_get_next_token(scanner);
      curr_profile = (struct profile*)g_malloc(sizeof(struct profile));
      curr_profile->name = g_strdup("resolution_profile");
      curr_profile->filters = NULL;
      ALL_PROFILE = g_list_append (ALL_PROFILE, curr_profile); 
      return token;

    case ISG_TOKEN_filter:
      //printf(" case ISG_TOKEN_filer\n");
      parse_filter_names (scanner, profile_type);
      token = g_scanner_get_next_token(scanner);
      return ISG_TOKEN_filter;

    case '{':
      //printf(" case {\n");
      token = g_scanner_get_next_token(scanner);
      return token;

    case '}':
      //printf(" case }\n");
      token = g_scanner_get_next_token(scanner);
      return token;

    default:
      token = g_scanner_get_next_token(scanner);
      return G_TOKEN_NONE;
    }
}


/* 
 * Used in parsing filter profile like 'profile.conf'.
 */
void parse_conf_file( GScanner* scanner)
{
  gint i;
  gboolean done;

  printf(" [parse_conf_file()\n");

  //if (!scanner) 
    for (i = 0; i < G_N_ELEMENTS (symbols); i++) {
      g_scanner_scope_add_symbol (scanner, 0, symbols[i].name, GINT_TO_POINTER (symbols[i].token));
      printf(" >%s< added\n", symbols[i].name);
    }

  done = FALSE;

  while (!done)
    {
      if (g_scanner_peek_next_token (scanner) == G_TOKEN_EOF)
        done=TRUE;
      else
        {
          guint expected_token;
          
          expected_token = parse_conf_statement (scanner);

          if (expected_token != G_TOKEN_NONE)
            {
              gchar *symbol_name;
              gchar *msg;
              
              msg = NULL;
              symbol_name = NULL;
            }
	  else {
	    printf("G_TOKEN_NONE\n");
	  }
	}
    }

}
  
