<xsl:stylesheet version="1.0"                                                
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform">

<xsl:template name="Sender_Node_Header">
<xsl:param name="filters"/>
<file><name>sender/Node000.h</name>

<content>

/**********************
 * Node000.h -- the header file for this node
 * Generated by ISG
 * 
 * !(modify)
 **********************/
#ifndef __NODE000_H
#define __NODE000_H 1

// Include the data definitions
#include "datatypes.h"
#include &lt;sys/types.h>
#include &lt;sys/socket.h>
#include &lt;sys/stat.h>
#include &lt;sys/param.h>
#include &lt;sys/wait.h>
#include &lt;sys/time.h>
#include &lt;netinet/in.h>
#include &lt;arpa/inet.h>
#include &lt;netdb.h>
#include &lt;errno.h>
#include &lt;pthread.h>
#include &lt;string.h>
#include &lt;sched.h>
#include "auth.h"

//&lt;block name="pipe_GZIP">
<xsl:if test="contains($filters,'GZIP')">
#include "gzip.h"
#define GZIP 5 
//&lt;/block>

</xsl:if>

//&lt;block name="pipe_LZO">
<xsl:if test="contains($filters, 'LZO')">
#include "lzo.h"
#define LZO 6 
</xsl:if>
//&lt;/block>

//&lt;block name="pipe_ENCRYPT">
<xsl:if test="contains($filters, 'ENCRYPT')">
#include "encrypt.h"
#define ENCRYPT	0x10
</xsl:if>
//&lt;/block>

//&lt;block name="pipe_JPEG">
<xsl:if test="contains($filters, 'JPEG')">
#include "jpgcodec.h"
#define JPG 7
</xsl:if>
//&lt;/block>

//&lt;block name="pipe_SHRINK">
<xsl:if test="contains($filters, 'SHRINK')">
#define SHRINK 0x21
</xsl:if>
//&lt;/block>

<xsl:if test="contains($filters, 'GREY')">
#define GRAYSCALE 0x22

	<xsl:if test="contains($filters, 'SHRINK')">
#define SHRINKGRAYSCALE 0x23 // aka SHRINK | GRAYSCALE
	</xsl:if>

</xsl:if>


void sender_node_Node000Submit();

int processQoSSender(void *uHdr, int *size);
int receiveDataPacket(int receiveSocket, void *uHdr);
int processQoSReceiver ( QoSHdrPtr qosHdr, void *uHdr);
int sendData(int sockfd, void *dstruct, int size, int flags);
int receiveData(int sockfd, void *dstruct, int size, int flags);
void *senderNodeControl(void* args);


// stats
/*********** Adaptation Tables ****************/

typedef struct adapt_table_t {
	// Avg. Size Reduction factor
	double comp_ratio; 
	
	// reduction in size ?
	// 0 => No reduction
	// 1 => Halve image
	char reqScale;

	// make it a gray scale image?
	// 0 => 3 bit color
	// 1 => grayscale , 1 bit per pixel
	char reqGrayscale;

	// Lossless Data Compression
	// 0 => No compression
	// 5, GZIP => GZIP
	// 6, LZO => LZO Compression
	char reqCompress;

	// Lossy Image compression
	// 0 => No compression
	// 7. JPG => JPG Compression
	char reqLossy;
}adaptation_table;

#endif

</content>
</file>
</xsl:template>


<xsl:template name="Sender_Node">
<xsl:param name="filters"/>

<file><name>sender/Node000.c</name>

<content>
/******************************
 * Supposed to be generated
 * Node000.c -- the implementation file for this node
 *
 *
 ******************************/

#include "Node000.h"
#include "localvars.h"

inline long getTS()
{
	struct timeval now;
	
	gettimeofday(&amp;now, NULL);

	return((now.tv_sec-baseTime.tv_sec)*1e6 + now.tv_usec);
}

/******************
 * Processing
 * 
 * Send Latency Information
 *****************/

void endProcessing()
{
	//qosHdr.latency = latency;
}


/******************
 * 
 * SEND DATA
 * Take care of fragmentation
 *
 * Issues: Since it is a TCP connection,
 * send all the data in one big group, and let TCP
 * take care of fragmentation/reassembly
 * 
 * First send the header, - which is a global struct
 * then use the user code to send the dataheader and datapacket
 *
 * must use sendData function 
 * 
 * INPUTS: user's datapacket(processed) +
 * datapacketsize (Total size of the datapacket in bytes 
 * 								- must include EVERYTHING) +
 * Hidden Inputs
 * 	the global header
 * 	the QoS Header - filled by the processing function
 * 	
 *****************/
int sendDataPacket(int submitSocket, void *uHdr, int size)
{
		
	int noError = TRUE, pad;
	//PPMDataHdrPtr ppmPtr;
	UserDataHdrPtr aPtr;
	long start, end;
	static int overflow = 0;	
	static int wind_timestamp = 0;
	int sleep_st;

	if(wind_timestamp  == 0)
		wind_timestamp = getTS();

	if(!authenticated) return 1;
	//////////////// GENERATED PORTION /////////////////////
	// Fill In the header
	dpktHdr.seqNumber= seq_no++;
	dpktHdr.flags = 0;
	dpktHdr.type = 0;
	dpktHdr.time = getTS();
	
	
	aPtr = (UserDataHdrPtr) uHdr;

	data_size_hist = (header_overhead + size + 9*data_size_hist) / 10;
	// Process the QoS options: 

	start = getTS();
	processQoSSender(uHdr, &amp;size);
	end = getTS();
	qos_time += (end - start);
	
	// At present in bytes. Modify to words(4 bytes) ??
	dpktHdr.length = header_overhead +size; 
	
	pkt_size_hist = (dpktHdr.length + 9 * pkt_size_hist)/10;
	
	// Send all the headers
	
	if(sendData(submitSocket, &amp;dpktHdr, dPktHdrLength, 0) 
									!= dPktHdrLength)
	{
		printf("Error:sending data packet header\n");
		noError = FALSE;
	}
	if(sendData(submitSocket, &amp;qosHdr, qosHdrLength, 0) 
									!= qosHdrLength)
	{
		printf("Error:sending QoS header\n");
		noError = FALSE;
	}
	
	// USER DATA PORTION
	
	if(userHdrLength != 4) // Header is not just a pointer
	{
		// First Send Header, then the buffer
		if(sendData(submitSocket, aPtr, userHdrLength - 4, 0)
										!= (userHdrLength -4))
		{
			printf("Error: Sending User Data Header\n");
		}
	}
	// Now send the Data
	if(sendData(submitSocket, aPtr->buffer, size, 0) 
									!= size)
	{
		printf("Error: Sending User Data Header\n");
		noError = FALSE;
	}
	pkts_sent++;


	//printf("Sending Done\n");
	return noError;
}


/************************
 * QoS on the Sender Side
 * 
 * At present NULL function
 ***********************/

int processQoSSender(void *uHdr, int *size)
{
	int status = 0;
	char *out;
	UserDataHdrPtr aptr = (UserDataHdrPtr)uHdr;
	qosHdr.type = (encrypt_type &amp; 0xF0) 
								| (compress_type &amp; 0x0F);
	out = temp_buf;

	printf("Ctype:%d, Etype:%d SType:%d\n", compress_type, encrypt_type, scale_type);
	// Scaling Options
	switch(scale_type)
	{
//&lt;block name="filter_SHRINK">
<xsl:if test="contains($filters, 'SHRINK')">
		case SHRINK:
			shrinkFilter(1, aptr, temp_buf);
			aptr->buffer= temp_buf;
			*size = aptr ->size;
			break;
</xsl:if>
//&lt;/block>

//&lt;block name="filter_GREY">
<xsl:if test="contains($filters, 'GREY')">
		case GRAYSCALE:
			greyFilter(1, aptr, temp_buf);
			aptr->buffer= temp_buf;
			*size = aptr ->size;
			break;
	
	<xsl:if test="contains($filters, 'SHRINK')">
		case SHRINKGRAYSCALE:
			shrinkFilter(1, aptr, aptr->buffer);
			//memset((void*)out_buf, 1, inImage->size); 	
			greyFilter(1, aptr, temp_buf);
			aptr->buffer = temp_buf;
			*size = aptr ->size;
			break;	
	</xsl:if>
//&lt;/block>
	
</xsl:if>
		default:
			break;
	}
	

	// Compression Options
	switch(compress_type)
	{

//&lt;block name="filter_LZO">
<xsl:if test="contains($filters, 'LZO')">
		case LZO:
			status = lzo_compress(qosHdr.options, aptr->buffer, aptr->size, out, size);
			aptr->buffer = out;
			break;
</xsl:if>
//&lt;/block>

//&lt;block name="filter_GZIP">
<xsl:if test="contains($filters, 'GZIP')">
		case GZIP:
    	status = gzip_compress(1, aptr->buffer, aptr->size, out, size);
    	aptr->buffer = out;
			break;
</xsl:if>
//&lt;/block>

//&lt;block name="filter_JPEG">
<xsl:if test="contains($filters, 'JPEG')">
		case JPG:
			init_jpeg_compress();
			ppm_to_jpeg(aptr->buffer,aptr->height, aptr->width, out, JPEGQFactor, size);     
			end_jpeg_compress();
			aptr->buffer = out;
			break;
</xsl:if>
//&lt;/block>

		default:
			break;
	}

//&lt;block name="filter_ENCRYPT">
<xsl:if test="contains($filters, 'ENCRYPT')">
	if(encrypt_type == ENCRYPT)
	{

		int t=*size;
		qosHdr.pad = (*size) % encryptParam.keySize;
		if(qosHdr.pad != 0)
		{
			t= *size + encryptParam.keySize - qosHdr.pad;
		}

		status = Encrypt((void *) &amp;(encryptParam.cryptContext), aptr->buffer, t, size);
		if(status &lt; 0)
			printf("Error Encrypting\n");

	}
</xsl:if>
//&lt;/block>

	return status;
}

/*****************
 * RAISE EVENT 
 * 
 *****************/
void raiseEvent(char *event)
{
	//printf("start event : %s \n", event);


	if (event == "send") {
		
		memmove(img_buf, Node000OutgoingData.buffer, Node000OutgoingData.size);
		free(Node000OutgoingData.buffer);
		Node000OutgoingData.buffer = img_buf;
		sendDataPacket(sender_node_Node000SubmitSocket,
		    &amp;Node000OutgoingData, Node000OutgoingData.size);
		//printf(" event : %s \n", event);
	}

	if (event == "endProcessing") {
		endProcessing();
		//printf(" event : %s \n", event);
	}

}

/**********************
 * SUBMIT - RECEIVE
 *
 **********************/

void sender_node_Node000Submit()
{
	//raiseEvent("endProcessing");
	raiseEvent("send");
}


/***********************
 * CLOSE SOCKET 
 *
 **********************/
void closeSocket(int sock)
{
	close(sock);
	printf("Close connection \n");
}

/*********************
 * INIT SOCKET 
 *
 *********************/
#define TCP_NODELAY 1
int sender_node_Node000initSubmitSocket()
{
	int sock, tcpnodelay_flag = 0;
	int listenSock;
	struct sockaddr_in sin;
	unsigned int alen;


	// Set connection parameters 
	listenSock = socket(PF_INET, SOCK_STREAM, 0);
	sin.sin_family = AF_INET;
	sin.sin_addr.s_addr = INADDR_ANY;
	sin.sin_port = htons(port);
	
	// Bind - Listen - Accept 
	bind(listenSock, (struct sockaddr *) &amp;sin, sizeof(sin));
	listen(listenSock, 1);
	alen = sizeof(sin);
	
	// Accept a new connection
	sock = accept(listenSock, (struct sockaddr *) &amp;sin, &amp;alen);

//	if (setsockopt(sock, IPPROTO_TCP, TCP_NODELAY, (char *)&amp;tcpnodelay_flag, sizeof(int)) == -1)
//	{
//		perror("error: setsockopt");
//		return 0;
//	}

	close(listenSock);

	printf("Listenning port: %d \n", sin.sin_port);
	printf(" PID: %d\n", getpid());

	return sock;
}

/******************
 * INFOPIPE START UP
 * 
 * Add Code to init Hdrs
 * Fill up length fields
 ******************/
pthread_t cThread;
void infopipeStartup()
{
	init_local_vars();
	// Init Control Thread
	if (pthread_create ( &amp;cThread, NULL, senderNodeControl, NULL) !=0 )
  { 
    perror("Unable to create threads");
    return;
  }

	// Init Data Socket
  sender_node_Node000SubmitSocket =
  sender_node_Node000initSubmitSocket();
	printf("Submit socket initialized \n");
}

/* 
 * All local variables are assigned initial values
 *
 */

int init_local_vars()
{
	char algo[30];
	int keySize=24;

	// Init Header Lengths
	dPktHdrLength = sizeof(dataPacketHdr);
	qosHdrLength = sizeof(QoSHdr);
	ppmDataHdrLength = sizeof(PPMDataHdr);	
	arrayDataLength = sizeof(ArrayData);
	ctrlHdrLength = sizeof(CtrlHdr);
	authHdrLength = sizeof(AuthHdr);
	sessionHdrLength = sizeof(SessionHdr);
	
	// Init DataPacketHeader Fields
	dpktHdr.seqNumber = 1;
	dpktHdr.sourceID=1001;
	

	// Init QoS Hdr fields
	qosHdr.type=0;
	qosHdr.options=(char)0;
	qosHdr.region='d';
	qosHdr.tag='e';
	qosHdr.pad=0;

	// INitialize available filter configurations

	// Code for LZO Filter Initialization
//&lt;block name="filter_LZO">
<xsl:choose>
<xsl:when test="contains($filters, 'LZO')">
	haveLZO = 1;
	lzo_filter_init();
</xsl:when>
<xsl:otherwise>
	haveLZO = 0;
</xsl:otherwise>
</xsl:choose>
//&lt;/block>

//&lt;block name="filter_JPEG">
<xsl:choose>
<xsl:when test="contains($filters, 'JPEG')">
	haveLossy = 1;
	JPEGQFactor = 100;;
</xsl:when>
<xsl:otherwise>
	haveLossy = 0;
</xsl:otherwise>
</xsl:choose>
//&lt;/block>


//&lt;block name="filter_GZIP">
<xsl:choose>
<xsl:when test="contains($filters, 'GZIP')">
	haveGZIP = 1;
</xsl:when>
<xsl:otherwise>
	haveGZIP = 0;
</xsl:otherwise>
</xsl:choose>
//&lt;/block>

//&lt;block name="filter_ENCRYPT">
<xsl:choose>
<xsl:when test="contains($filters, 'ENCRYPT')">
	haveEncode = 1;
	encryptParam.keySize = keySize;
	encryptParam.ivSize = keySize;

	encryptParam.key = (char *) malloc(encryptParam.keySize * sizeof(char));
	encryptParam.iv = (char *) malloc(encryptParam.ivSize * sizeof(char));

 	strncpy(encryptParam.key, "abcdefghijklmnopqrstuvwxyz123456", keySize);	
 	strncpy(encryptParam.iv, "abcdefghijklmnopqrstuvwxyz123456", keySize);	

	printf("Initialized Encryption mod with keysize:%d\n", keySize);

	if(encryptParam.keySize != 0)
	{
		
		sprintf(algo,"%s-%d","rijndael", (encryptParam.keySize*8));
	
		printf("Setting algo:%s\n", algo);
		encryptParam.cryptContext = CreateContext(algo, "ecb");
		SetParams((void *) &amp;(encryptParam.cryptContext), encryptParam.key, encryptParam.iv);

	}
</xsl:when>
<xsl:otherwise>
	haveEncode = 0;
</xsl:otherwise>
</xsl:choose>
//&lt;/block>

//&lt;block name="filter_GREY">
<xsl:choose>
<xsl:when test="contains($filters, 'GREY')">
	haveGrayscale = 1;
</xsl:when>
<xsl:otherwise>
	haveGrayscale = 0;
</xsl:otherwise>
</xsl:choose>
//&lt;/block>

//&lt;block name="filter_SHRINK">
<xsl:choose>
<xsl:when test="contains($filters, 'SHRINK')">
	haveScale = 1;
</xsl:when>
<xsl:otherwise>
	haveScale = 0;
</xsl:otherwise>
</xsl:choose>
//&lt;/block>

	// Initialize filter usage params
	
	compress_type = 0;
	encrypt_type = 0;
	scale_type = 0;

	temp_buf = (char*)malloc(260000);
	img_buf = (char*)malloc(260000);

	// FEEDBACK COMPONENTS INIT
	seq_no = 1;
	pkt_rate = 2000;
	header_overhead = dPktHdrLength + qosHdrLength + userHdrLength - 4; 
	
	pkt_size_hist = 250000 + header_overhead;
	data_size_hist = 250000 + header_overhead;
	qos_time_hist = 0;

	WINDOWSIZE=10;


	// Init Authentication Mode
	auth_mode = DUAL;
	control_thread = 0;

	// Get the starting Time
	gettimeofday(&amp;baseTime, NULL);	

	return 1;

}
/****************
 * INFOPIPE SHUT DOWN
 * 
 ***************/
void infopipeShutdown()
{
	/* Close channel */
	closeSocket(sender_node_Node000SubmitSocket);
	closeSocket(ctrlSock);
}

//SEND A FIN_REQ MSG
int end_conn()
{
	CtrlHdr chdr;
	dataPacketHdr dhdr;
	
	chdr.flags = FIN_REQ;
	chdr.pkt_rate = pkts_sent;

	sendData(ctrlSock, &amp;chdr, sizeof(chdr), 0);
	
	dhdr.flags = FIN_REQ;
	sendData(sender_node_Node000SubmitSocket, &amp;dhdr, sizeof(dhdr), 0);
	
	printf("Ending conn:%d\n", pkts_sent);
	return 1;
}

/***************
 * Control Thread for the Sender Node
 *
 * 1. Init socket and wait for connection
 * 2. Once connected, carry out authentication process
 * 		2.1 If needed setup session key and session params
 * 3. Recv control msgs from RCVR about
 *    -latency
 *    -bandwidth
 *    -security
 * 4. based on the msgs and requirements of the rcvr, modify qosHdr
 * 5. Carry out steps 3,4 in a while(1) loop
 *
 ***************/


#define SUPERWIND 12 

void *senderNodeControl(void* args)
{
	int sock_fd;
	int decrease;
	struct sockaddr_in server_addr, client_addr;
	unsigned int alen;
	char fname[30];	
	char comp_str[10];
	int ack_pkts, i;
	unsigned long curr_time;	
	CtrlHdr cHdr;
	double lat, flat, factor;
	static int last_super_wind_pkt_rate=0, last_pkts_sent = 0;
	static int last_double_super_wind_pkt_rate=0;
	int temp, temp1;
	FILE *fp;
	
	static int old_sleep_time = 1000;
	static int initcount = 0;
	
	bzero((void*)&amp;server_addr, sizeof(struct sockaddr));
	bzero((void*)&amp;client_addr, sizeof(struct sockaddr));
	sock_fd = socket (AF_INET, SOCK_STREAM, 0);
	if (sock_fd &lt; 0)
	{
		fprintf(stderr, "Unable to create socket. Exiting Server........\n");
		return 0;
	}
	
	server_addr.sin_family = AF_INET;
	server_addr.sin_port = htons (port + 5);
	server_addr.sin_addr.s_addr = htonl(INADDR_ANY);  	
	
	if ( bind (sock_fd, (struct sockaddr *)&amp;server_addr, sizeof(struct sockaddr)) &lt; 0)
	{
		fprintf(stderr, "Server bind error\n");
		return 0;
	}
	
	if(listen(sock_fd, 5) &lt; 0)
	{
		perror("Listen Error\n");
	}
	
	ctrlSock = accept(sock_fd, (struct sockaddr *) &amp;client_addr, &amp;alen);
	close(sock_fd);

	// ********************* Authentication ********************* //
	setKeysPath(keys_path);
	strcpy(pgmName, "server");
	i = strlen(pgmName);
	strcpy(fname, pgmName);
	strcat(&amp;fname[i], ".pub");
	fname[i+4] = '\0';
	readPubKey(fname, &amp;mypubKey);
	
	strcpy(&amp;fname[i], ".prv");
	fname[i+4] = '\0';
	readPrivKey(fname, &amp;privKey);
	
	authenticate(ctrlSock);

	authenticated = 1;
	i=0;
	ack_pkts = 0;
	
	while(1)
	{
		// recv pkts
		cHdr.flags = 0;
	
		if(receiveData(ctrlSock, &amp;cHdr, ctrlHdrLength, MSG_WAITALL) != ctrlHdrLength)
		{
			perror("Control Msg Error\n");
		}

		if(cHdr.flags == FIN_REP)
		{
			control_thread = 1;
			return 0;
		}
		
#ifdef TIMING_STATS		
		//TIMING
		curr_time = getTS();
		rcv_ts[rcv_idx++] = curr_time;	
#endif

		if(cHdr.flags == SETPARAMS)
		{
			target_fps = cHdr.pkt_rate;
			ifCompress = cHdr.srcID[0] &amp; haveCompress; 
			ifScale = cHdr.srcID[1] &amp; haveScale;
			ifLossy = cHdr.srcID[2] &amp; haveLossy;
			ifEncode = cHdr.srcID[3] &amp; haveEncode;
			ifGrayscale = cHdr.srcID[4] &amp; haveGrayscale;

			//printf("New params:ifEncode:.....................................%c\n", ifCompress+ '0');
			// Now force the adaptation
			adapt();

			continue;
		}
		if(cHdr.flags == FEEDBACK )
		{
			current_fps = cHdr.pkt_rate;
			//printf("cHdr.pkt_Rate:%d\n",cHdr.pkt_rate);
			adapt();
		}
	}
	
	return 0;
}

//FOR TCP Change all send/recv to
//sendData and recevieData

int authenticate(int sock_fd)
{
	CtrlHdr cHdr;
	AuthHdr aHdr;
	SessionHdr sHdr;
	
	char msg[2048];
	char dec_msg[2048];
	int t1, t2; // temp

	R_RSA_PUBLIC_KEY pubKey;
	recv(sock_fd, &amp;cHdr, ctrlHdrLength, MSG_WAITALL);
	switch((int)cHdr.enc_type)
	{
		case PLAINTXT:
			break;
		case PKENC:
			break;
		case DBLPKENC:
			printf("Must recv Length:%d\n", cHdr.length);
			recv(sock_fd, msg, cHdr.length, MSG_WAITALL);
			strcpy(dec_msg, cHdr.srcID);
			strcat(dec_msg, ".pub");
			readPubKey(dec_msg, &amp;pubKey);

			RSAPrivateDecryptBlock(dec_msg, &amp;t1, msg, cHdr.length, &amp;privKey);
			RSAPublicDecryptBlock(msg, &amp;t2, dec_msg, t1, &amp;pubKey);
			
			if(t2 != authHdrLength)
			{
				printf("Wrong Msg decrypted\n");
			}
			memcpy((void*)&amp;aHdr, msg, t2);
			
			if(strncmp(aHdr.magic, MAGIC, 16) == 0)
			{
				printf("Client \"%s\" authenticated\n", cHdr.srcID);
				
				cHdr.length = 0;
				// Check for request for Session Key
				if(aHdr.flags == (char)SESSKEYREQ)
				{
					recv(sock_fd, &amp;sHdr, sessionHdrLength, MSG_WAITALL); 
					// Process Session Options and send reply
					// SET REPLYFLAG
					// AND LENGTH
				}
				//Increment last Char of random by 1 and send
				aHdr.random[15]++;
				memcpy(msg, (void*)&amp;aHdr, authHdrLength);
				RSAPrivateEncryptBlock(dec_msg, &amp;t1, msg, authHdrLength, &amp;privKey);
				RSAPublicEncryptBlock(msg,  &amp;t2, dec_msg, t1, &amp;pubKey);
				printf("Encrypted Msg length:%d\n", t2);

				// Send the encrypted Data
				strcpy(cHdr.srcID, "server");
				cHdr.length += t2;

				send(sock_fd, &amp;cHdr, ctrlHdrLength, 0); 
				send(sock_fd, msg, t2, 0);
				
				if(aHdr.flags == (char)SESSKEYREP)
				{
					
				}
			}else {
				printf("Wrong auth Message\n");
			}
			break;
		default:
			return -1;
	}
}



/*
 * Adaptation Function - Both for static and Dynamic
 */

int adapt()
{
	double curr_comp_ratio, tgt_comp_ratio; // Current compression ratio
	double eff_data_rate; // Effective data rate in fps
	int index, i;


	// Steps
	// a) find current compression ratio
	// b) Using current fps, find effective data rate, 
	// 		i.e. data rate w.o any compression
	// c) Use this to find out the target ratio
	// d) traverse down the tree to find the correct algorithm
	
	//printf("Sizes %d  %d\n", pkt_size_hist, data_size_hist);
	curr_comp_ratio = (double)pkt_size_hist/data_size_hist;
	eff_data_rate = (double)current_fps * curr_comp_ratio;
	
	tgt_comp_ratio = eff_data_rate/ target_fps;
	// Disable all Compression
	if(tgt_comp_ratio > 0.90)
	{
		compress_type = 0;		
		scale_type =0;
		printf("Change: Removing all filters..\n");
	}else {
		int last_match = -1;	
	
	//	printf("Adapting: Curr fps:%d tgt_ratio:%.3f \n", current_fps, tgt_comp_ratio);
	//	printf("Scales : %d %d %d %d\n", ifScale, ifGrayscale, ifCompress, ifLossy);
	
		for(i=0;i&lt;A_LEVELS;i++)
		{
			if((aTable[i].reqScale != 0) &amp;&amp; (ifScale == 0))
				continue;

			if((aTable[i].reqGrayscale != 0) &amp;&amp; (ifGrayscale == 0))
				continue;
				
			if((aTable[i].reqCompress != 0) &amp;&amp; ((ifCompress) == 0))
				continue;
				
			if((aTable[i].reqLossy != 0) &amp;&amp; (ifLossy == 0))
				continue;

			// A matching filter has been found.
			last_match = i;
			if(tgt_comp_ratio >= aTable[i].comp_ratio)
			{
				compress_type = aTable[i].reqCompress | aTable[i].reqLossy;
				scale_type = 0;
				if(aTable[i].reqScale)
					scale_type = SHRINK;
				if(aTable[i].reqGrayscale)
					scale_type |= GRAYSCALE;
				
				break;
				printf("Selecting Compress type:---------------------------------------- :%d\n",i);
			}
		}
		
		// Cannot Satisfy Requirements for current compression ratio
		// 
		// So we choose the last possible match
		// if last_match = -1 then we disable all compression
		if(i== A_LEVELS)
		{
			if(last_match == -1)
			{
				scale_type = 0;
				compress_type = 0;
			}
			else 
			{
				compress_type = aTable[last_match].reqCompress | aTable[last_match].reqLossy;
				scale_type = 0;
				if(aTable[last_match].reqScale)
					scale_type = SHRINK;
				if(aTable[last_match].reqGrayscale)
					scale_type |= GRAYSCALE;
				printf("Filter Type:---------------------------------LAST MATCH :%d:%d\n",last_match, aTable[last_match].reqCompress);
			}
		}
	}
	//Enable Encryption
	if(ifEncode != 0)
	{
		encrypt_type = ENCRYPT; 
	}else{
		encrypt_type = 0;
	}
	
}

/***********************
 * 
 * SEND function.
 * Can be used by the user while writing his send fn.
 * Provides Transparency from underlying protocols
 * 
 * RETURN: number of bytes sent
 ***********************/

// Currently only TCP. so easily just a send
// FLAGS: MSG_OOB - outofband data
int sendData(int sockfd, void *dstruct, int size, int flags)
{
	return send(sockfd, dstruct, size, 0);
}


/***********************
 * 
 * RECEIVE function.
 * Can be used by the user while writing his recv fn.
 * Provides Transparency from underlying protocols
 * 
 * RETURN: number of bytes received
 ***********************/

// Currently only TCP.
// FLAGS : MSG_OOB - for out of band Data
// MSG_PEEK - to check w.o exactly recving
// MSG_WAITALL - fill the struct b4 returning

int receiveData(int sockfd, void *dstruct, int size, int flags)
{
	return recv(sockfd, dstruct, size, flags);
}

</content>
</file>
</xsl:template>
</xsl:stylesheet>
