# a class to build a tree to hold the parsed
# data from the SIP file so it can be converted
# into XIP, or whatever else

#root of a tree - that describes a SIP/XIP document
#has 4 sections
#types - points to list of types
#filters - points to list of filters
#aspects - points to list of aspects

indent = 2
useBlock = 'no'

class docBase:
  def Find(self, whattype, name):
    print 'Find: Not implemented on the object'
  def Sip(self, depth=0):
    return 'Sip: Not implemented on this object'
  def Xip(self, depth=0):
    print 'Xip: Not implemented on this object'
  def AddBlock(self, prefix, retString, indentString):
    if useBlock == 'yes':
      retString = indentString + '<block name="' + prefix + '_' + self.name + '">\n' + retString
      retString += indentString + '</block>\n'
    return retString


class xipDoc(docBase): 
  'Parent node for a XIP doc'
  blockNum = 0
  def __init__(self, datatypes, filters, pipes, aspects):
    self.nodetype = 'xipDoc'
    self.datatypes = datatypes
    self.filters = filters
    self.pipes = pipes
    self.aspects = aspects
    self.root = self  # we are always the root of the tree
  def Find( self, whattype, name ):
    if whattype == 'pipe':
      return self.pipes.Find( whattype, name )
    else:
      return None

    return retString

  def Sip(self, depth=0):
    retString = ''
    if self.datatypes: 
      retString += self.datatypes.Sip(depth)
    if self.filters: 
      retString += self.filters.Sip(depth)
    if self.pipes: 
      retString += self.pipes.Sip(depth)
    if self.composedPipe: 
      retString += self.composedPipe.Sip(depth+indent)

    if useBlock == 'yes':
      retString = '<blockSip>\n' + retString + '</blockSip>\n'

    return retString

  def Xip(self, depth=0):
    # our header for an XML file
    retString = ''
    indentString = ' '*depth
    retString += indentString + '<?xml version="1.0" ?>\n'
    #retString += '<!DOCTYPE InfopipeSpec SYSTEM "InfopipeSpec.dtd">\n'
    retString += indentString + '<xip name="sip2xip">\n'

    if self.datatypes:
      retString += indentString + ' '*indent + '<datatypes>\n'
      retString += self.datatypes.Xip(depth+indent)
      retString += indentString + ' '*indent + '</datatypes>\n'
    if self.filters:
      retString += indentString + ' '*indent + '<filters>\n'
      retString += self.filters.Xip(depth+indent)
      retString += indentString + ' '*indent + '</filters>\n'
    if self.pipes:
      retString += indentString + ' '*indent + '<pipes>\n'
      retString += self.pipes.Xip(depth+indent)
      retString += indentString + ' '*indent + '</pipes>\n'
    if self.composedPipe:
      retString += self.composedPipe.Xip(depth+indent)
    retString += indentString + '</xip>\n'
    return retString

class objectList(docBase):
  def __init__(self, root, objectType, node, moreNodes):
    self.root = root
    self.objectType = objectType
    self.node = node
    self.moreNodes = moreNodes
  def Sip(self, depth=0):
    retString = ''
    if self.node:
      retString += self.node.Sip(depth+indent)
    if self.moreNodes:
      retString += self.moreNodes.Sip(depth)
    return retString
  def Xip(self, depth=0):
    retString = ''
    if self.node:
      retString += self.node.Xip(depth+indent)
    if self.moreNodes:
      retString += self.moreNodes.Xip(depth)
    return retString

#individual type declaration
#name - name the type referenced by
#arglist - list of the types basic elements
class datatypeNode(docBase):
  def __init__(self, root, name, args):
    self.nodetype = 'typeNode'
    self.root = root
    self.name = name
    self.args = args
  def Sip(self, depth=0):
    retString = ''
    indentString = ' '*depth
    retString += indentString + 'datatype ' + self.name + '\n'
    retString += indentString + '{\n'
    retString += self.args.Sip(depth)
    retString += indentString + '}\n'

    retString = self.AddBlock('datatype', retString, indentString)

    return retString
  def Xip(self, depth=0):
    retString = ''
    indentString = ' '*depth
    retString += indentString + '<datatype name="' + self.name + '">\n'
    if self.args:
      retString += self.args.Xip(depth)
    retString += indentString + '</datatype>\n'

    retString = self.AddBlock('datatype', retString, indentString)

    return retString

#one name, atomic type pair
#name - name of athe var
#ptype - primitive type (e.g. 'float', 'integer', 'char')
class argNode(docBase):
  def __init__(self, root, name, ptype, size=0):
    self.nodetype = 'argNode'
    self.root = root
    self.name = name
    self.ptype = ptype
    self.size = size
  def Sip(self, depth=0):
    retString = ''
    indentString = ' '*depth
    retString += indentString + self.name+':'+self.ptype
    if self.size != 0:
      retString += '[' + self.size + ']'
    retString += '\n'
    return retString
  def Xip(self, depth=0):
    retString = ''
    indentString = ' '*depth
    if self.size != 0:
      retString += indentString + '<arrayArg name="' + self.name + '" type="' + self.ptype + '" size="' + self.size + '"/>\n'
    else:
      retString += indentString + '<arg name="'+self.name+'" type="'+self.ptype+'"/>\n'
    return retString

class filterNode(docBase):
  def __init__(self, root, name, uri, inType, outType, paramType):
    self.name = name
    self.uri = uri
    self.inType = inType
    self.outType = outType
    self.paramType = paramType
  def Xip(self, depth=0):
    retString = ''
    indentString = ' '*depth
    retString += indentString + '<filter name="' + self.name + '"\n'
    retString += indentString + ' uri="' + self.uri + '"\n'
    if self.inType == 'ANY':
      retString += indentString + ' inType="*"\n'
    else:
      retString += indentString + ' inType="' + self.inType + '"\n'
    retString += indentString + ' outType="' + self.outType + '"\n'
    if self.paramType != None:
      retString += indentString + ' takesParams="1"\n'
      retString += indentString + ' paramType="' + self.paramType + '"'
    else:
      retString += indentString + ' takesParams="0"'

    retString += '/>\n'

    return retString

#individual pipe declaration
#name - name the type referenced by

class pipeNode(docBase):
  def __init__(self, root, name, input, output):
    self.nodetype = 'pipeNode'
    self.root = root
    self.name = name
    self.input = input
    self.output = output
  def Find(self,whattype, name):
    if name == self.name:
      return self
    else:
      return None
  def Sip(self, depth=0):
    retString = ''
    indentString = ' '*depth
    retString += indentString + 'pipe '+self.name+'\n'
    retString += indentString + '{\n'
    if self.input:
      retString += indentString + ' '*indent + 'inports\n'
      retString += indentString + ' '*indent + '{\n'
      retString += self.input.Sip(depth+indent)
      retString += indentString + ' '*indent + '}\n'
    if self.output:
      retString += indentString + ' '*indent + 'outports\n'
      retString += indentString + ' '*indent + '{\n'
      retString += self.output.Sip(depth+indent)
      retString += indentString + ' '*indent + '}\n'
    retString += indentString + '}\n'

    retString = self.AddBlock('pipe', retString, indentString) 
    return retString

  def Xip(self, depth=0):
    retString = ''
    indentString = ' '*depth
    retString += indentString + '<pipe name="' + self.name + '">\n' 
    if self.input:
      retString += self.input.Xip(depth)
    if self.output:
      retString += self.output.Xip(depth)
    retString += indentString + '</pipe>\n'

    retString = self.AddBlock('pipe', retString, indentString) 
    return retString

class portNode(docBase):
  def __init__(self, root, name, inout, datatype, polarity):
    self.nodetype = 'portNode'
    self.root = root
    self.name = name
    self.inout = inout
    self.datatype = datatype
    self.polarity = polarity
  def Sip(self, depth=0):
    retString = ''
    indentString = ' '*depth
    retString += indentString + self.name + ':' + self.datatype + '\n'
    return retString
  def Xip(self, depth=0):
    retString = ''
    indentString = ' '*depth
    if self.inout == 'in':
      retString += indentString + '<inport name="' + self.name + '" '
    else:
      retString += indentString + '<outport name="' + self.name + '" '
    retString += 'type="' + self.datatype + '"'
    retString += '/>\n'
    return retString

class composedPipeNode(docBase):
  def __init__(self, root, name, pipes, inports, outports, connections):
    self.nodetype = 'composedPipeNode'
    self.root = root
    self.name = name
    self.pipes = pipes
    self.inports = inports
    self.outports = outports
    self.connections = connections
  def Sip(self, depth=0):
    retString = ''
    indentString = ' '*depth
    retString += indentString + 'composedPipe ' + self.name + '\n'
    retString += indentString + '{\n'
    if self.pipes:
      retString += indentString + ' '*indent + 'pipes\n'
      retString += indentString + ' '*indent + '{\n'
      retString += self.pipes.Sip(depth+indent)
      retString += indentString + ' '*indent + '}\n'

    if self.inports:
      retString += indentString + ' '*indent + 'inports\n'
      retString += indentString + ' '*indent + '{\n'
      retString += self.inports.Sip(depth+indent)
      retString += indentString + ' '*indent + '}\n'

    if self.outports:
      retString += indentString + ' '*indent + 'outports\n'
      retString += indentString + ' '*indent + '{\n'
      retString += self.outports.Sip(depth+indent)
      retString += indentString + ' '*indent + '}\n'

    if self.connections:
      retString += indentString + ' '*indent + 'connections\n'
      retString += indentString + ' '*indent + '{\n'
      retString += self.connections.Sip(depth+indent)
      retString += indentString + ' '*indent + '}\n'

    retString += indentString + '}\n'

    retString = self.AddBlock('composedPipe', retString, indentString)

    return retString

  def Xip(self, depth=0):
    retString = ''
    indentString = ' '*depth
    retString += indentString + '<composedPipe name="' + self.name + '">\n'
    if self.pipes:
      retString += indentString + ' '*indent + '<declarations>\n'
      retString += self.pipes.Xip(depth+indent)
      retString += indentString + ' '*indent + '</declarations>\n'

    retString += indentString + ' '*indent + '<ports>\n'
    if self.inports:
      retString += self.inports.Xip(depth+indent)
    if self.outports:
      retString += self.outports.Xip(depth+indent)
    retString += indentString + ' '*indent + '</ports>\n'

    if self.connections:
      retString += indentString + ' '*indent + '<connections>\n'
      retString += self.connections.Xip(depth+indent)
      retString += indentString + ' '*indent + '</connections>\n'

    retString += indentString + '</composedPipe>\n'

    retString = self.AddBlock('composedPipe', retString, indentString)

    return retString

class pipeDeclNode(docBase):
  def __init__(self, root, name, pipeclass):
    self.nodetype = 'pipeDeclNode'
    self.root = root
    self.name = name
    self.pipeclass = pipeclass
  def Sip(self, depth=0):
    retString = ''
    indentString = ' '*depth
    retString += indentString + self.name + ':' + self.pipeclass + '\n'
    return retString
  def Xip(self, depth=0):
    retString = ''
    indentString = ' '*depth
    retString += indentString + '<pipe name="' + self.name + '" '
    retString += 'class="' + self.pipeclass + '"'
    retString += '/>\n'
    return retString

class portDeclNode(docBase):
  def __init__(self, root, name, frompipe, fromport, inout):
    self.nodetype = 'portDeclNode'
    self.root = root
    self.name = name
    self.frompipe = frompipe
    self.fromport = fromport
    self.inout = inout
  def Sip(self, depth=0):
    retString = ''
    indentString = ' '*depth
    retString += indentString + self.name + ':' + self.frompipe + '.' + self.fromport + '\n'
    return retString
  def Xip(self, depth=0):
    retString = ''
    indentString = ' '*depth
    if self.inout == 'in':
      retString += indentString + '<inport name="' + self.name + '" '
    else:
      retString += indentString + '<outport name="' + self.name + '" '
    #retString += 'type="' + self.datatype + '" '
    retString += 'frompipe="' + self.frompipe + '" '
    retString += 'fromport="' + self.fromport + '" '
    retString += '/>\n'
    return retString

class connectionNode(docBase):
  def __init__(self, root, frompipe, fromport, topipe, toport):
    self.nodetype = 'connectionNode'
    self.root = root
    self.frompipe = frompipe
    self.fromport = fromport
    self.topipe = topipe
    self.toport = toport
  def Sip(self, depth=0):
    retString = ''
    indentString = ' '*depth
    retString += indentString + self.frompipe + '.' + self.fromport
    retString += ' -> ' + self.topipe + '.' + self.toport + '\n'
    return retString
  def Xip(self, depth=0):
    retString = ''
    indentString = ' '*depth
    retString += indentString + '<connection>\n'
    retString += indentString + ' '*indent + '<from pipe="' + self.frompipe + '" '
    retString += 'port="' + self.fromport + '"/>\n'
    retString += indentString + ' '*indent + '<to pipe="' + self.topipe + '" '
    retString += 'port="' + self.toport + '"/>\n'
    retString += indentString + '</connection>\n'
    return retString

# a class to build a tree to hold the parsed
# data from the SIP file so it can be converted
# into XIP, or whatever else

#root of a tree - that describes a SIP/XIP document
#has 4 sections
#types - points to list of types
#filters - points to list of filters
#aspects - points to list of aspects

indent = 2
useBlock = 'no'

class docBase:
  def Find(self, whattype, name):
    print 'Find: Not implemented on the object'
  def Sip(self, depth=0):
    return 'Sip: Not implemented on this object'
  def Xip(self, depth=0):
    print 'Xip: Not implemented on this object'
  def AddBlock(self, prefix, retString, indentString):
    if useBlock == 'yes':
      retString = indentString + '<block name="' + prefix + '_' + self.name + '">\n' + retString
      retString += indentString + '</block>\n'
    return retString


class xipDoc(docBase): 
  'Parent node for a XIP doc'
  blockNum = 0
  def __init__(self, datatypes, filters, pipes, aspects):
    self.nodetype = 'xipDoc'
    self.datatypes = datatypes
    self.filters = filters
    self.pipes = pipes
    self.aspects = aspects
    self.root = self  # we are always the root of the tree
  def Find( self, whattype, name ):
    if whattype == 'pipe':
      return self.pipes.Find( whattype, name )
    else:
      return None

    return retString

  def Sip(self, depth=0):
    retString = ''
    if self.datatypes: 
      retString += self.datatypes.Sip(depth)
    if self.filters: 
      retString += self.filters.Sip(depth)
    if self.pipes: 
      retString += self.pipes.Sip(depth)
    if self.composedPipe: 
      retString += self.composedPipe.Sip(depth+indent)

    if useBlock == 'yes':
      retString = '<blockSip>\n' + retString + '</blockSip>\n'

    return retString

  def Xip(self, depth=0):
    # our header for an XML file
    retString = ''
    indentString = ' '*depth
    retString += indentString + '<?xml version="1.0" ?>\n'
    #retString += '<!DOCTYPE InfopipeSpec SYSTEM "InfopipeSpec.dtd">\n'
    retString += indentString + '<xip name="sip2xip">\n'

    if self.datatypes:
      retString += indentString + ' '*indent + '<datatypes>\n'
      retString += self.datatypes.Xip(depth+indent)
      retString += indentString + ' '*indent + '</datatypes>\n'
    if self.filters:
      retString += indentString + ' '*indent + '<filters>\n'
      retString += self.filters.Xip(depth+indent)
      retString += indentString + ' '*indent + '</filters>\n'
    if self.pipes:
      retString += indentString + ' '*indent + '<pipes>\n'
      retString += self.pipes.Xip(depth+indent)
      retString += indentString + ' '*indent + '</pipes>\n'
    if self.composedPipe:
      retString += self.composedPipe.Xip(depth+indent)
    retString += indentString + '</xip>\n'
    return retString

class objectList(docBase):
  def __init__(self, root, objectType, node, moreNodes):
    self.root = root
    self.objectType = objectType
    self.node = node
    self.moreNodes = moreNodes
  def Sip(self, depth=0):
    retString = ''
    if self.node:
      retString += self.node.Sip(depth+indent)
    if self.moreNodes:
      retString += self.moreNodes.Sip(depth)
    return retString
  def Xip(self, depth=0):
    retString = ''
    if self.node:
      retString += self.node.Xip(depth+indent)
    if self.moreNodes:
      retString += self.moreNodes.Xip(depth)
    return retString

#individual type declaration
#name - name the type referenced by
#arglist - list of the types basic elements
class datatypeNode(docBase):
  def __init__(self, root, name, args):
    self.nodetype = 'typeNode'
    self.root = root
    self.name = name
    self.args = args
  def Sip(self, depth=0):
    retString = ''
    indentString = ' '*depth
    retString += indentString + 'datatype ' + self.name + '\n'
    retString += indentString + '{\n'
    retString += self.args.Sip(depth)
    retString += indentString + '}\n'

    retString = self.AddBlock('datatype', retString, indentString)

    return retString
  def Xip(self, depth=0):
    retString = ''
    indentString = ' '*depth
    retString += indentString + '<datatype name="' + self.name + '">\n'
    if self.args:
      retString += self.args.Xip(depth)
    retString += indentString + '</datatype>\n'

    retString = self.AddBlock('datatype', retString, indentString)

    return retString

#one name, atomic type pair
#name - name of athe var
#ptype - primitive type (e.g. 'float', 'integer', 'char')
class argNode(docBase):
  def __init__(self, root, name, ptype, size=0):
    self.nodetype = 'argNode'
    self.root = root
    self.name = name
    self.ptype = ptype
    self.size = size
  def Sip(self, depth=0):
    retString = ''
    indentString = ' '*depth
    retString += indentString + self.name+':'+self.ptype
    if self.size != 0:
      retString += '[' + self.size + ']'
    retString += '\n'
    return retString
  def Xip(self, depth=0):
    retString = ''
    indentString = ' '*depth
    if self.size != 0:
      retString += indentString + '<arrayArg name="' + self.name + '" type="' + self.ptype + '" size="' + self.size + '"/>\n'
    else:
      retString += indentString + '<arg name="'+self.name+'" type="'+self.ptype+'"/>\n'
    return retString

class filterNode(docBase):
  def __init__(self, root, name, uri, inType, outType, paramType):
    self.name = name
    self.uri = uri
    self.inType = inType
    self.outType = outType
    self.paramType = paramType
  def Sip(self, depth=0):
    retString = ''
    indentString = ' '*depth
    retString += indentString + 'filter ' + self.name + '\n'
    retString += indentString + '{\n'
    retString += indentString + ' '*indent + 'uri:' + self.uri + '\n'
    retString += indentString + ' '*indent + 'in:' + self.inType + '\n'
    retString += indentString + ' '*indent + 'out:' + self.outType + '\n'
    if self.paramType != None:
      retString += indentString + ' '*indent + 'param:' + self.paramType + '\n'
    retString += indentString + '}\n'

    retString = self.AddBlock('filter', retString, indentString) 

    return retString

  def Xip(self, depth=0):
    retString = ''
    indentString = ' '*depth
    retString += indentString + '<filter name="' + self.name + '"\n'
    retString += indentString + ' uri="' + self.uri + '"\n'
    if self.inType == 'ANY':
      retString += indentString + ' inType="*"\n'
    else:
      retString += indentString + ' inType="' + self.inType + '"\n'
    retString += indentString + ' outType="' + self.outType + '"\n'
    if self.paramType != None:
      retString += indentString + ' takesParams="1"\n'
      retString += indentString + ' paramType="' + self.paramType + '"'
    else:
      retString += indentString + ' takesParams="0"'

    retString += '/>\n'

    retString = self.AddBlock('filter', retString, indentString) 

    return retString

#individual pipe declaration
#name - name the type referenced by

class pipeNode(docBase):
  def __init__(self, root, name, input, output):
    self.nodetype = 'pipeNode'
    self.root = root
    self.name = name
    self.input = input
    self.output = output
  def Find(self,whattype, name):
    if name == self.name:
      return self
    else:
      return None
  def Sip(self, depth=0):
    retString = ''
    indentString = ' '*depth
    retString += indentString + 'pipe '+self.name+'\n'
    retString += indentString + '{\n'
    if self.input:
      retString += indentString + ' '*indent + 'inports\n'
      retString += indentString + ' '*indent + '{\n'
      retString += self.input.Sip(depth+indent)
      retString += indentString + ' '*indent + '}\n'
    if self.output:
      retString += indentString + ' '*indent + 'outports\n'
      retString += indentString + ' '*indent + '{\n'
      retString += self.output.Sip(depth+indent)
      retString += indentString + ' '*indent + '}\n'
    retString += indentString + '}\n'

    retString = self.AddBlock('pipe', retString, indentString) 
    return retString

  def Xip(self, depth=0):
    retString = ''
    indentString = ' '*depth
    retString += indentString + '<pipe name="' + self.name + '">\n' 
    if self.input:
      retString += self.input.Xip(depth)
    if self.output:
      retString += self.output.Xip(depth)
    retString += indentString + '</pipe>\n'

    retString = self.AddBlock('pipe', retString, indentString) 
    return retString

class portNode(docBase):
  def __init__(self, root, name, inout, datatype, polarity):
    self.nodetype = 'portNode'
    self.root = root
    self.name = name
    self.inout = inout
    self.datatype = datatype
    self.polarity = polarity
  def Sip(self, depth=0):
    retString = ''
    indentString = ' '*depth
    retString += indentString + self.name + ':' + self.datatype + '\n'
    return retString
  def Xip(self, depth=0):
    retString = ''
    indentString = ' '*depth
    if self.inout == 'in':
      retString += indentString + '<inport name="' + self.name + '" '
    else:
      retString += indentString + '<outport name="' + self.name + '" '
    retString += 'type="' + self.datatype + '"'
    retString += '/>\n'
    return retString

class composedPipeNode(docBase):
  def __init__(self, root, name, pipes, inports, outports, connections):
    self.nodetype = 'composedPipeNode'
    self.root = root
    self.name = name
    self.pipes = pipes
    self.inports = inports
    self.outports = outports
    self.connections = connections
  def Sip(self, depth=0):
    retString = ''
    indentString = ' '*depth
    retString += indentString + 'composedPipe ' + self.name + '\n'
    retString += indentString + '{\n'
    if self.pipes:
      retString += indentString + ' '*indent + 'pipes\n'
      retString += indentString + ' '*indent + '{\n'
      retString += self.pipes.Sip(depth+indent)
      retString += indentString + ' '*indent + '}\n'

    if self.inports:
      retString += indentString + ' '*indent + 'inports\n'
      retString += indentString + ' '*indent + '{\n'
      retString += self.inports.Sip(depth+indent)
      retString += indentString + ' '*indent + '}\n'

    if self.outports:
      retString += indentString + ' '*indent + 'outports\n'
      retString += indentString + ' '*indent + '{\n'
      retString += self.outports.Sip(depth+indent)
      retString += indentString + ' '*indent + '}\n'

    if self.connections:
      retString += indentString + ' '*indent + 'connections\n'
      retString += indentString + ' '*indent + '{\n'
      retString += self.connections.Sip(depth+indent)
      retString += indentString + ' '*indent + '}\n'

    retString += indentString + '}\n'

    retString = self.AddBlock('composedPipe', retString, indentString)

    return retString

  def Xip(self, depth=0):
    retString = ''
    indentString = ' '*depth
    retString += indentString + '<composedPipe name="' + self.name + '">\n'
    if self.pipes:
      retString += indentString + ' '*indent + '<declarations>\n'
      retString += self.pipes.Xip(depth+indent)
      retString += indentString + ' '*indent + '</declarations>\n'

    retString += indentString + ' '*indent + '<ports>\n'
    if self.inports:
      retString += self.inports.Xip(depth+indent)
    if self.outports:
      retString += self.outports.Xip(depth+indent)
    retString += indentString + ' '*indent + '</ports>\n'

    if self.connections:
      retString += indentString + ' '*indent + '<connections>\n'
      retString += self.connections.Xip(depth+indent)
      retString += indentString + ' '*indent + '</connections>\n'

    retString += indentString + '</composedPipe>\n'

    retString = self.AddBlock('composedPipe', retString, indentString)

    return retString

class pipeDeclNode(docBase):
  def __init__(self, root, name, pipeclass):
    self.nodetype = 'pipeDeclNode'
    self.root = root
    self.name = name
    self.pipeclass = pipeclass
  def Sip(self, depth=0):
    retString = ''
    indentString = ' '*depth
    retString += indentString + self.name + ':' + self.pipeclass + '\n'
    return retString
  def Xip(self, depth=0):
    retString = ''
    indentString = ' '*depth
    retString += indentString + '<pipe name="' + self.name + '" '
    retString += 'class="' + self.pipeclass + '"'
    retString += '/>\n'
    return retString

class portDeclNode(docBase):
  def __init__(self, root, name, frompipe, fromport, inout):
    self.nodetype = 'portDeclNode'
    self.root = root
    self.name = name
    self.frompipe = frompipe
    self.fromport = fromport
    self.inout = inout
  def Sip(self, depth=0):
    retString = ''
    indentString = ' '*depth
    retString += indentString + self.name + ':' + self.frompipe + '.' + self.fromport + '\n'
    return retString
  def Xip(self, depth=0):
    retString = ''
    indentString = ' '*depth
    if self.inout == 'in':
      retString += indentString + '<inport name="' + self.name + '" '
    else:
      retString += indentString + '<outport name="' + self.name + '" '
    #retString += 'type="' + self.datatype + '" '
    retString += 'frompipe="' + self.frompipe + '" '
    retString += 'fromport="' + self.fromport + '" '
    retString += '/>\n'
    return retString

class connectionNode(docBase):
  def __init__(self, root, frompipe, fromport, topipe, toport):
    self.nodetype = 'connectionNode'
    self.root = root
    self.frompipe = frompipe
    self.fromport = fromport
    self.topipe = topipe
    self.toport = toport
  def Sip(self, depth=0):
    retString = ''
    indentString = ' '*depth
    retString += indentString + self.frompipe + '.' + self.fromport
    retString += ' -> ' + self.topipe + '.' + self.toport + '\n'
    return retString
  def Xip(self, depth=0):
    retString = ''
    indentString = ' '*depth
    retString += indentString + '<connection>\n'
    retString += indentString + ' '*indent + '<from pipe="' + self.frompipe + '" '
    retString += 'port="' + self.fromport + '"/>\n'
    retString += indentString + ' '*indent + '<to pipe="' + self.topipe + '" '
    retString += 'port="' + self.toport + '"/>\n'
    retString += indentString + '</connection>\n'
    return retString

