#include <fstream.h>
#include <XalanTransformer/XalanTransformer.hpp>
#include <XalanTransformer/XalanCompiledStylesheetDefault.hpp>
#include <XalanTransformer/XercesDOMParsedSource.hpp>
#include <XSLT/XSLTInputSource.hpp>
#include <XercesParserLiaison/XercesParserLiaison.hpp>
#include <XercesParserLiaison/XercesDocumentBridge.hpp>
#include <XalanTransformer/XercesDOMParsedSource.hpp>
#include <XalanTransformer/XercesDOMWrapperParsedSource.hpp>
#include <XercesParserLiaison/XercesDOMSupport.hpp>
#include <XalanDOM/XalanDOMException.hpp>
#include "isgUtil.hpp"

int initializeXSLTProcessor()
{
  XMLPlatformUtils::Initialize();
  XalanTransformer::initialize();

  return 0;
}

static int initialized = 0;

int doXSLTstep( DOM_Document nsDataSource, DOM_Document tStylesheet, DOM_Document &result )
{
  int retval;
  char *thisFunc = "doXSLTstep(): ";

  nsDataSource.normalize();
  tStylesheet.normalize();

  // create our transformer object
  XalanTransformer transformEngine;

  // where we store the output
  // cout << thisFunc << "Creating parser liaison and dom support." << endl;
  XercesDOMSupport theDOMSupport;
  XercesParserLiaison theParserLiaison(theDOMSupport);


  // now we have to import a Xalan doc.
  //cout << thisFunc << "Bringing NS into Xalan world." << endl;
  XercesDOMWrapperParsedSource theParsedSource(nsDataSource, theParserLiaison, theDOMSupport);
  /*
  XalanDocument *theXalanNSDocument = theParserLiaison.createDocument(nsDataSource);
  XSLTInputSource theNSInput(theXalanNSDocument);
  result = theParserLiaison.mapXercesDocument( theXalanNSDocument );
  ofstream nsfile("tempNSout");
  nsfile << result << endl;
  nsfile.close();
  */

	// our stylesheet to apply
  // cout << thisFunc << "Bringing SS into Xalan world." << endl;
  XercesDOMWrapperParsedSource theSSParsedSource(tStylesheet, theParserLiaison, theDOMSupport);
//  XalanDocument *theXalanSSDocument = theParserLiaison.createDocument(tStylesheet);
//  XSLTInputSource theSSInput(theXalanSSDocument);
//  result = theParserLiaison.mapXercesDocument( theXalanSSDocument );
//  ofstream ssfile("tempSSout");
//  ssfile << result << endl;
//  ssfile.close();

  // to hold our output
  // cout << thisFunc << "Creating output doc." << endl;
  XalanDocument *domOut = theParserLiaison.createDocument();
  XSLTResultTarget theXSLTResult(domOut);

  
  // compile the style sheet
  const XalanCompiledStylesheet* theCompiledTemplate = 0;
  retval = transformEngine.compileStylesheet( theSSParsedSource.getDocument(), theCompiledTemplate );
  if( retval )
  {
    cout << thisFunc << "******Error compiling stylesheet.******" << endl;
    return retval;
  }
  else if( theCompiledTemplate == 0)
  {
    cout << thisFunc << "******* compiled is 0 *********" << endl;
  }
  

// working statements
  retval = transformEngine.transform( theParsedSource, theCompiledTemplate, theXSLTResult);
  //retval = transformEngine.transform( nsDataSource.getDocument(), tStylesheet.getDocument(), theXSLTResult);
  //cout << thisFunc << theParserLiaison.mapXercesDocument( theSSInput ) << endl;
  if(retval)
  {
    cout << thisFunc << "XalanError:" << transformEngine.getLastError() << endl;
    cout << endl;
  }


//  cout << thisFunc << "Mapping back to Xerces." << endl;
  // build something that can go back to the Xerces and the rest of the program
  result = theParserLiaison.mapXercesDocument( theXSLTResult.getDocument() );
//  cout << thisFunc << theParserLiaison.mapXercesDocument( theXSLTResult.getDocument() ) << endl;


  if(result.getDocumentElement() == NULL )
  {
    cout << thisFunc << "'result' is null" << endl;
    return 1;
  }
  else
  {
//    cout << result << endl << "+++++++++++";
  }

  return 0;
}

/* Xerces Xalan trial and error
 * 1) Can't find transform function that accepts DOM_Document from Xerces
 *    Hard to follow trail through abstract classes
 * 2) make guess to use XercesDocumentBridge. This can give us a XalanDocument
 *    which would think could fit right in to transform(). No. transform fails.
 * 3) Try putting Bridge first in XSLTInputSource class. No. XSLTInputSource
 *    seemingly succeeds. transform fails.
 * 4) Try compiling stylesheet first. Compile seeemingly succeeds. Transform fails.
 * 5) Find additional example on web. It takes
 *    a) modifications to parser before parse time
 *    b) special classes other than what I guessed
 * 6) Turns out that was the wrong example
 * 7) Try second example. Still must take care on options given
 *    to parser at parse time. 
 *    a) wrap DOM_Document in a Xalan class
 *    b) wrap that in ANOTHER Xalan class XSLTInputSource
 *    c) give that to Xalan method transform with has a signature
 *       transform( XSLTInputSource, XSLTInputSource, XSLTResultTarget )
 *    d) That doesn't work!
 *    (Apparently, the parser continues to behave as if the XSLTInputSource 
 *    is a file)
 * 8) Not knowing problem, print out results of wrapping to XSLTInputSource.
 *    This is simple, just ask it to map back to Xerces. What's printed
 *    out reveals a problem earlier in code while cloning documents.
 *    Find a better way to clone documents. Not a huge problem. (Previous
 *    solution was probably based on suggested function by gcc)
 * 9) Xerces problem now fixed, still can't use Xalan
 * 10) Wenchang suggests reading Xalan source code.
 * 11) Apparently, using transform(XSLTInputSource, XSLTInputSource, ...)
 *     Xalan tries to *re-parse* the input as if it were a raw stream of
 *     data. Ugh. This is despite the fact that XSLTInputSource 
 *     *specifically* claims XalanNode* is valid.
 *     Need to find a different transform function.
 * 12) Hunt through code. Find function transform( XalanParsedSource *,...)
 *     Class-cast to XercesDOMParsedSource both SS and NS.
 * 13) Compiler balks. Cast only NS.
 * 14) Success. Note: SS is *still* XSLTInputSource! XalanTransformer does not treat
 *     similar args in a uniform manner (i.e. parse both of same type)
 * 15) able to get SS to sort-of work, now getting unknown error and now SS output     
 */
