// outAspect.cpp - implementation of functions to output the DOM document 
//
#include <fstream.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>

#include "isg.hpp"
#include "out.hpp"

// fills in code for each node of the pipe
static int fillAspectAtomChildren( DOM_Element theAtom, DOM_Element condEl );
static int fillAspectAtom( DOM_Element theAtom, DOM_Element theAspectInfo );
static int fillAspect( DOM_Element inTemplate, DOM_Element inNodeSet );
static int pruneAspects( DOM_Element templtEl );
static int collapseAspects( DOM_Element tEl );
static DOMString lookupCondInfo( DOM_Element condNode, DOMString whichTag, DOMString whichAtt );
static int updateAspectMarks( DOM_Element e );

// this accepts the template and turns on aspect code in it
// aspectDesc is ??
int generateAspectCode( DOM_Element templateElement, DOM_Element aspectDesc )
{
  DOM_Element aspectTemplate;
  DOM_NodeList aspectTemplateList;

  DOM_NodeList aspectList;
  DOMString aspectName;

  int i,j;

  //cout << "generateAspectCode(): enter." << endl;

  // choose the aspects from the template
  aspectTemplateList = templateElement.getElementsByTagName("aspect");

  // get the list of aspects we need to apply
  aspectList = aspectDesc.getElementsByTagName("aspect");

  // for each aspect specified for the pipe
  for( i = 0; i < aspectList.getLength(); i++ )
  {
    // find out which aspect it is
    aspectName = (DOM_ELEMENT(aspectList.item(i))).getAttribute("name");

    // search all aspects in the template
    for( j = 0; j < aspectTemplateList.getLength(); j++ )
    {
      aspectTemplate = DOM_ELEMENT(aspectTemplateList.item(j));

      // if name matches
      if( aspectTemplate.getAttribute("name").equals( aspectName ) )
      {
        // mark it for output
        markElement( aspectTemplate );

        // fill in the Aspect (template, description)
        fillAspect( aspectTemplate, DOM_ELEMENT(aspectList.item(i)) );
      }
    }
  }
  
  updateAspectMarks( templateElement );

  // get rid of extraneous aspects
  pruneAspects( templateElement );


  // merge the aspects into the main template
  //cout << "generateAspectCode():collapsing start." << endl;
  collapseAspects( templateElement );
  //cout << "generateAspectCode():collapsing done." << endl;
//cout << endl << "*************************************************" << endl;
//  cout << templateElement << endl;
//cout << endl << "*************************************************" << endl;

  //cout << "generateAspectCode():exit" << endl;
  return 0;
}

static int collapseAspects( DOM_Element tEl )
{
  int i;
  DOM_Node temp, aspect;
  DOM_Node restart;
  DOM_NodeList children;

  // base case
  if( tEl == NULL ) return 0;

  children = tEl.getChildNodes();
  for( i = 0; i < children.getLength(); i++ )
  {
//    cout << "collapseAspects(): i = " << i << " length = " << children.getLength() << endl;
    // we know the child is not an aspect
    if( children.item(i).getNodeName().equals("aspect") )
    {
      // our aspect to collapse
      DOM_Element aEl = DOM_ELEMENT( children.item(i) );
      while( (temp = aEl.getFirstChild()) != NULL )
      {
        //cout << "collapseAspects(): moving child." << endl;
        // remove the first child
        aEl.removeChild( temp );
        // re-add in front of "aspect" element
        tEl.insertBefore( temp, aEl );
      }
      // now remove the aspect node
      tEl.removeChild(aEl);
      i = i - 1;
    }
  // recurse first
  collapseAspects( DOM_ELEMENT( children.item(i) ) );
  }
}

// when we prune we remove unmarked aspects - aspects which weren't 
// specified in the ISL code
static int pruneAspects( DOM_Element templtEl )
{
  DOM_NodeList anl; // list of aspects
  DOM_Node parent; 
  DOM_Element currAspect;
  int i;

  anl = templtEl.getElementsByTagName("aspect");

  for( i = anl.getLength(); i > 0; i-- )
  {
    currAspect = DOM_ELEMENT(anl.item(i - 1));
    if( !isElementMarked( currAspect ) )
    {
      parent = currAspect.getParentNode();
      parent.removeChild( currAspect );
    }
  }
  return 0;
}

static int fillAspectAtomChildren( DOM_Element theAtom, DOM_Element condEl )
{
  DOM_Node tNode;
  DOM_Element curChild;
  DOMString newData;

  tNode = theAtom.getFirstChild();
  while( tNode != NULL )
    {
    curChild = (DOM_Element &)tNode;
//    cout << "fillAspectAtomChildren(): looping now at "; curChild.getNodeName().print(); cout << endl;

    if( curChild.getNodeName().equals("lookup"))
      {
//      cout << "fillAspectAtomChildren(): looking up ";
//     curChild.getAttribute("fromTag").print();cout << " ";curChild.getAttribute("attName").print();
//      cout << endl;
      newData = lookupCondInfo( condEl, 
                                curChild.getAttribute("fromTag"), 
                                curChild.getAttribute("attName") );
      makeTextAndMarkNode( curChild, newData );
      }
    else
      {
      // ignore
      }
    tNode = curChild.getNextSibling();
    }
  return 0;
}


static int fillAspectAtom( DOM_Element theAtom, DOM_Element theAspectInfo )
{

  DOM_Node tNode;
  DOM_Element tElem;
  DOM_NodeList condList = theAspectInfo.getElementsByTagName("condition");
  DOM_Element curCond; // the current condition node
  DOM_Element *atomArray;
  int i; 

  // we must duplicate the atom for as many conditions are in the aspect
  // a place to store all the new slots
  atomArray = new DOM_Element[ condList.getLength() ];

  // now duplicate them all 
  for( i = 0; i < (condList.getLength() - 1); i++ )
    {
//    cout << "fillAspectAtom(): clone." << endl;
    tNode = theAtom.cloneNode(true);
    atomArray[i] = (DOM_Element &)tNode;
    theAtom.getParentNode().insertBefore( tNode, theAtom.getNextSibling() );
    }
  atomArray[i] = theAtom;
//  cout << "fillAspectAtom(): did " << i+1 << " clones." << endl;

  // now fill them in
  for( i = 0; i < condList.getLength(); i++ )
    {
    tNode = condList.item(i);
    fillAspectAtomChildren( atomArray[i], (DOM_Element &)tNode);
    //pprint( atomArray[i] );
    }

  return 0;
}

// given the template for aspect code and the values supplied for the
// aspect, this fills in the values
static int fillAspect( DOM_Element inTemplate, DOM_Element inNodeSet )
{
  int i, numAtoms;
  DOM_Node tNode;
  DOM_Element tElem;

  DOM_NodeList lookupList;
  DOM_NodeList atomNodeList;
  DOM_NodeList nameList;

  DOM_Element *atomNodeArray;

  DOMString nsName;

  tNode = inTemplate.getFirstChild();

  // collect all the atoms in an array so we don't count duplicates later
  // I don't like this any more than you do. But it will work for now
  atomNodeList = inTemplate.getElementsByTagName( "atom" );
  atomNodeArray = new DOM_Element[ atomNodeList.getLength() ];

  numAtoms = atomNodeList.getLength();
  for( i = numAtoms; i > 0; i-- )
    {
    tNode = atomNodeList.item(i-1);
    atomNodeArray[i-1] = (DOM_Element &)tNode;
    }
  for( i = 0; i < numAtoms; i++ )
    {
    fillAspectAtom( atomNodeArray[i], inNodeSet );
    }

/* 
  nsName = DOM_ELEMENT(inNodeSet.getParentNode().getParentNode()).getAttribute("pipeName");
  nameList = inTemplate.getElementsByTagName( "thisPipeName" );
  cout << "pipe name: ";
  nsName.println();
  for( i = 0; i < nameList.getLength(); i++ )
  {
    makeTextAndMarkNode( DOM_ELEMENT(nameList.item(i)), nsName );
  }
*/  
  /* 
  while( tNode != NULL )
    {
    // convert to an element
    tElem = (DOM_Element &) tNode;
 
    // now we need to fill in elements and atoms
    if( tElem.getNodeName().equals("atom") )
      {
      fillAspectAtom( tElem, inNodeSet );
      }

    // get the next child to process
    tNode = tNode.getNextSibling();
    }
  */
  delete [] atomNodeArray;
  return 0;
}

static DOMString lookupCondInfo( DOM_Element condNode, DOMString whichTag, DOMString whichAtt )
{
  DOM_NodeList nl;
  DOM_Node tNode;
  DOM_Element tElem;
  nl = condNode.getElementsByTagName(whichTag);
  tNode = nl.item(0);
  tElem = (DOM_Element &)tNode;
  //cout << "lookupCondInfo(): ask
  return tElem.getAttribute(whichAtt);
}



// updates the print markings on all nodes, a node is "marked" if
// it has any node beneath it that is marked
// when a node is marked, all the text nodes beneath it will be marked
static int updateAspectMarks( DOM_Element e )
{
  DOM_Node tnode;
  DOM_Text texnode;
  DOM_Element e2;
  int thisMarked = 0;

  // if we're already marked, we must still update sub-tree, but
  // we don't have to check each node as it completes
  if( isElementMarked(e) && e.getNodeName().equals("aspect") )
    thisMarked = 1;

  // check children first
  tnode = e.getFirstChild();
  while( tnode != NULL )
    {
      if( tnode.getNodeType() == DOM_Node::ELEMENT_NODE )
        {
        e2 = (DOM_Element &)tnode;
        // update the child
        updateAspectMarks( e2 );
        
        // if we end up with a marked child, then we must 
        // mark this node if not already done. Keep track of whether 
        // we already have decided to mark it for speed.
        if( !thisMarked && isElementMarked(e2) && e.getNodeName().equals("aspect") )
          {
          // set the marking
          thisMarked = 1;
          markElement(e);
          }
        }
      tnode = tnode.getNextSibling();
    }
}
