<xsl:stylesheet version="1.0"                                                
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform">

<!-- ============================================= -->
<!--      The Connection_Manager_CPP template      -->
<!-- ============================================= -->


<xsl:template name="Connection_Manager_CPP">
  <xsl:param name="name"/>

  <!-- Capitalized name -->
  <xsl:variable name="name_all_capital">
    <xsl:value-of select="translate($name,'abcdefghijklmnopqrstuvwxyz','ABCDEFGHIJKLMNOPQRSTUVWXYZ')"/>
  </xsl:variable>
  <xsl:variable name="name_all_small">
    <xsl:value-of
    select="translate($name,'ABCDEFGHIJKLMNOPQRSTUVWXYZ','abcdefghijklmnopqrstuvwxyz')"/>
  </xsl:variable>

  <xsl:value-of select="$empty_line"/>
  <xsl:value-of select="$empty_line"/>

  <file><name>
    <xsl:value-of select="$dir_avstreams"/><xsl:value-of select="$name"/>.cpp</name><content>

  <xsl:value-of select="$file_header"/>

  <xsl:call-template name="comment">
    <xsl:with-param name="value">
      <xsl:value-of select="concat($name,'.cpp')"/>
    </xsl:with-param>
  </xsl:call-template>


  <!-- The system include -->
  <xsl:call-template name="dependency">
    <xsl:with-param name="files">
      <xsl:value-of select="concat('{',$name,'.h}')"/></xsl:with-param>
  </xsl:call-template>

  <!--
  <xsl:call-template name="dependency">
    <xsl:with-param name="files">{NegotiatorC.h}{NamingHelper_T.h}</xsl:with-param>
  </xsl:call-template>
  -->


  <xsl:value-of select="$empty_line"/>

  <!-- class methods -->
  <!-- Constructor -->
  <xsl:call-template name="method_imp">
    <xsl:with-param name="name">
      <xsl:value-of select="concat($name, '::', $name)"/></xsl:with-param>
    <xsl:with-param name="type">none</xsl:with-param>
    <xsl:with-param name="params">{none}</xsl:with-param>
    <xsl:with-param name="content">
{
}
    </xsl:with-param>
  </xsl:call-template>

  <!-- Destructor -->
  <xsl:call-template name="method_imp">
    <xsl:with-param name="name">
      <xsl:value-of select="concat($name, '::~', $name)"/></xsl:with-param>
    <xsl:with-param name="type">none</xsl:with-param>
    <xsl:with-param name="params">{none}</xsl:with-param>
    <xsl:with-param name="content">
{
}
    </xsl:with-param>
  </xsl:call-template>

  <xsl:call-template name="method_imp">
    <xsl:with-param name="name">
      <xsl:value-of select="concat($name, '::init')"/></xsl:with-param>
    <xsl:with-param name="type">int</xsl:with-param>
    <xsl:with-param name="params">{CORBA::ORB_ptr orb}</xsl:with-param>
    <xsl:with-param name="content">
{
  // Initialize the naming service
  if (this->naming_client_.init (orb) != 0)
    ACE_ERROR_RETURN ((LM_ERROR,
                       " (%P|%t) Unable to initialize "
                       "the TAO_Naming_Client. \n"),
                      -1);
  this->_orb = CORBA::ORB::_duplicate(orb);
  return 0;
}
    </xsl:with-param>
  </xsl:call-template>


  <xsl:call-template name="method_imp">
    <xsl:with-param name="name">
      <xsl:value-of select="concat($name, '::bind_to_receivers')"/></xsl:with-param>
    <xsl:with-param name="type">void</xsl:with-param>
    <xsl:with-param name="params">{const ACE_CString &amp;sender_name}{AVStreams::MMDevice_ptr sender}{CORBA::Environment &amp;ACE_TRY_ENV}</xsl:with-param>
    <xsl:with-param name="content">
{
  this->sender_name_ =
    sender_name;

  CosNaming::Name name (1);
  name.length (1);

  ACE_TRY
    {
      // Try binding the sender context in the NS
      name [0].id =
        CORBA::string_dup (this->sender_name_.c_str ());

      this->sender_context_ =
        this->naming_client_->bind_new_context (name,
                                                ACE_TRY_ENV);
      ACE_TRY_CHECK;

      //
      // We reach here if there was no exception raised in
      // &lt;bind_new_context&gt;.
      // We have successfully created the sender_context.
      // We then create a receiver context.
      //

      // Create the context for storing the receivers
      name [0].id =
        CORBA::string_dup ("Receivers");

      // Try binding the receivers context under the sender context.
      this->receiver_context_ =
        this->sender_context_->bind_new_context (name,
                                                 ACE_TRY_ENV);
      ACE_TRY_CHECK;
    }
  ACE_CATCH (CosNaming::NamingContext::AlreadyBound, al_ex)
    {
      //
      // The sender context already exists, probably created by the
      // receiver(s).
      //

      // Get the sender context.
      name [0].id =
        CORBA::string_dup (this->sender_name_.c_str ());

      CORBA::Object_var object =
        this->naming_client_->resolve (name,
                                       ACE_TRY_ENV);
      ACE_CHECK;

      this->sender_context_ =
        CosNaming::NamingContext::_narrow (object.in ());

      // Find the Receiver context.
      name [0].id =
        CORBA::string_dup ("Receivers");

      object =
        this->sender_context_->resolve (name,
                                        ACE_TRY_ENV);
      ACE_CHECK;

      this->receiver_context_ =
        CosNaming::NamingContext::_narrow (object.in ());

      this->find_receivers (ACE_TRY_ENV);
      ACE_CHECK;
    }
  ACE_ENDTRY;
  ACE_CHECK;

  name [0].id =
    CORBA::string_dup (this->sender_name_.c_str ());

  // Register the sender object with the sender context.
  this->sender_context_->rebind (name,
                                 sender,
                                 ACE_TRY_ENV);
  ACE_CHECK;
}
    </xsl:with-param>
  </xsl:call-template>

  <xsl:call-template name="method_imp">
    <xsl:with-param name="name">
      <xsl:value-of select="concat($name, '::find_receivers')"/></xsl:with-param>
    <xsl:with-param name="type">void</xsl:with-param>
    <xsl:with-param name="params">{CORBA::Environment &amp;ACE_TRY_ENV}</xsl:with-param>
    <xsl:with-param name="content">
{
  CosNaming::BindingIterator_var iterator;
  CosNaming::BindingList_var binding_list;
  const CORBA::ULong chunk = 100;

  // Get the list of receivers registered for this sender.
  this->receiver_context_->list (chunk,
                                 binding_list,
                                 iterator,
                                 ACE_TRY_ENV);
  ACE_CHECK;

  // Add the receivers found in the bindinglist to the receivers.
  this->add_to_receivers (binding_list,
                          ACE_TRY_ENV);
  ACE_CHECK;

  if (!CORBA::is_nil (iterator.in ()))
    {
      CORBA::Boolean more = 1;

      // Check to see if there are more receivers listed.
      while (more)
        {
          more = iterator->next_n (chunk,
                                   binding_list,
                                   ACE_TRY_ENV);
          ACE_CHECK;

          this->add_to_receivers (binding_list,
                                  ACE_TRY_ENV);
          ACE_CHECK;
        }
    }
}
    </xsl:with-param>
  </xsl:call-template>


  <xsl:call-template name="method_imp">
    <xsl:with-param name="name">
      <xsl:value-of select="concat($name, '::add_to_receivers')"/></xsl:with-param>
    <xsl:with-param name="type">void</xsl:with-param>
    <xsl:with-param name="params">{CosNaming::BindingList &amp;binding_list}{CORBA::Environment&amp; ACE_TRY_ENV}</xsl:with-param>
    <xsl:with-param name="content">
{
  for (CORBA::ULong i = 0;
       i &lt; binding_list.length ();
       i++)
    {
      // Get the receiver name from the binding list.
      ACE_CString receiver_name =
        binding_list [i].binding_name [0].id.in ();

      CosNaming::Name name (1);
      name.length (1);
      name [0].id =
        CORBA::string_dup (receiver_name.c_str ());

      // Resolve the reference of the receiver from the receiver
      // context.
      CORBA::Object_var obj =
        this->receiver_context_->resolve (name,
                                          ACE_TRY_ENV);

      AVStreams::MMDevice_var receiver_device =
        AVStreams::MMDevice::_narrow (obj.in ());

      // Add this receiver to the receiver map.
      this->receivers_.bind (receiver_name,
                             receiver_device);
    }
}
    </xsl:with-param>
  </xsl:call-template>


  <xsl:call-template name="method_imp">
    <xsl:with-param name="name">
      <xsl:value-of select="concat($name, '::connect_to_receivers')"/></xsl:with-param>
    <xsl:with-param name="type">void</xsl:with-param>
    <xsl:with-param name="params">{AVStreams::MMDevice_ptr sender}{AVStreams::streamQoS&amp; the_qos}{int use_qos}{CORBA::Environment &amp;ACE_TRY_ENV}</xsl:with-param>
    <xsl:with-param name="content">
{

  ACE_CString protocol;
  if( use_qos == 1)
  {
          protocol="QoS_UDP";
  }
  else
  {
          protocol="UDP";
  }

  // Connect to all receivers that we know about.
  for (Receivers::iterator iterator = this->receivers_.begin ();
       iterator != this->receivers_.end ();
       ++iterator)
    {
      // Initialize the QoS
      //AVStreams::streamQoS_var the_qos (new AVStreams::streamQoS);

      ACE_CString flowname;
      flowname =
        this->sender_name_ +
        "_" +
        (*iterator).ext_id_;

      // Create the forward flow specification to describe the flow.
      TAO_Forward_FlowSpec_Entry sender_entry (flowname.c_str (),
                                               "IN",
                                               "USER_DEFINED",
                                               "",
                                               protocol.c_str(),
                                               0);

      // Set the flow specification for the stream between receiver
      // and distributer
      AVStreams::flowSpec flow_spec (1);
      flow_spec.length (1);
      flow_spec [0] =
        CORBA::string_dup (sender_entry.entry_to_string ());

      // Create the stream control for this stream.
      TAO_StreamCtrl *streamctrl;
      ACE_NEW (streamctrl,
               TAO_StreamCtrl);

      // Servant Reference Counting to manage lifetime
      PortableServer::ServantBase_var safe_streamctrl =
        streamctrl;

      // Register streamctrl.
      AVStreams::StreamCtrl_var streamctrl_object =
        streamctrl->_this (ACE_TRY_ENV);
      ACE_CHECK;

      CORBA::ORB_ptr orb = TAO_AV_CORE::instance ()->orb(); 
      ACE_TString s = orb->object_to_string( streamctrl_object.in() );
      char tmp[20];
      sprintf(tmp, "%u", (unsigned int)ACE_OS::getpid() );
      ACE_TString fname("/tmp/streamctrl1_");
      fname += tmp;
      fname += ".ior";
      ofstream ior_file(fname.c_str(), ios::out|ios::trunc);
      ACE_Log_Msg* logger = ACE_Log_Msg::instance();
      logger->msg_ostream(&amp;ior_file);
      logger->set_flags(ACE_Log_Msg::OSTREAM); 
      logger->log(LM_DEBUG, "%s\n", s.c_str() );
      logger->clr_flags(ACE_Log_Msg::OSTREAM);

      // Bind the flowname and the corresponding stream controller to
      // the stream controller map
      this->streamctrls_.bind (flowname,
                               streamctrl_object);

      if(the_qos.length() &gt; 0) {
        the_qos [0].QoSType =  CORBA::string_dup ( flowname.c_str() ); 
      }
      // Bind the sender and receiver MMDevices.
      ACE_DEBUG((LM_DEBUG, "====STARTED bind_devs here %s %d\n",__FILE__,__LINE__));
      ACE_DEBUG((LM_DEBUG, "=== Flowname: %s\n", flowname.c_str() ));
      for(CORBA::ULong i=0; i &lt; flow_spec.length(); i++){
            ACE_DEBUG((LM_DEBUG, "Flow spec: %s\n", flow_spec[i].in() ));       
      }

      int result =
        streamctrl->bind_devs (sender,
                               (*iterator).int_id_.in(),
                               the_qos/*.inout ()*/,
                               flow_spec,
                               ACE_TRY_ENV);

      ACE_CHECK;
      ACE_DEBUG((LM_DEBUG, "====STOPPED bind_devs here %s %d\n",__FILE__,__LINE__));
    }
}
    </xsl:with-param>
  </xsl:call-template>

  <!-- 
  <xsl:call-template name="method_imp">
    <xsl:with-param name="name">
      <xsl:value-of select="concat($name, '::connect_negotiator')"/></xsl:with-param>
    <xsl:with-param name="type">void</xsl:with-param>
    <xsl:with-param name="params">{TAO_StreamEndPoint *endpoint}{const char *flowname}{CORBA::Environment &amp;ACE_TRY_ENV}</xsl:with-param>
    <xsl:with-param name="content">
{
  AVStreams::Negotiator_var negotiator_obj = AVStreams::Negotiator::_nil();

  ACE_TRY
    {
      // Get a reference to the Negotiator from the Naming Service
      CosNaming::Name name;
      name.length(1);
      name[0].id = CORBA::string_dup("NEGOTIATOR");
      name[0].kind = CORBA::string_dup("");

      CORBA::ORB_ptr orb = TAO_AV_CORE::instance ()->orb();

      CosNaming::NamingContext_var nc =
        NamingHelper&lt;CosNaming::NamingContext&gt;::resolve_init
        (orb, "NameService", 1, 0);

      negotiator_obj =
        NamingHelper&lt;AVStreams::Negotiator&gt;::resolve_name(nc.in(),
                                                          name, 5, ACE_Time_Value(1, 0));

      Negotiator::UAV_Negotiator_Factory_var factory =
          Negotiator::UAV_Negotiator_Factory::_narrow(negotiator_obj.in());

      if(! CORBA::is_nil(factory.in())) {
        negotiator_obj = factory->make_negotiator(CORBA::string_dup(flowname));
      }

      if(! CORBA::is_nil( negotiator_obj.in() ) ) {
        endpoint->set_negotiator (negotiator_obj.in());
        ACE_DEBUG((LM_DEBUG, "(%N,%l) Negotiator has been set\n"));
      }
    }
  ACE_CATCH(CosNaming::NamingContext::NotFound, ex1)
    {
    }
  ACE_CATCHANY{
    ACE_PRINT_EXCEPTION (ACE_ANY_EXCEPTION,"Distributor_Sender_StreamEndPoint::get_callback\n");
  }

  ACE_ENDTRY;
}
    </xsl:with-param>
  </xsl:call-template>
  -->

  <xsl:call-template name="method_imp">
    <xsl:with-param name="name">
      <xsl:value-of select="concat($name, '::bind_to_sender')"/></xsl:with-param>
    <xsl:with-param name="type">void</xsl:with-param>
    <xsl:with-param name="params">{const ACE_CString &amp;sender_name}{const ACE_CString &amp;receiver_name}{AVStreams::MMDevice_ptr receiver}{CORBA::Environment &amp;ACE_TRY_ENV}</xsl:with-param>
    <xsl:with-param name="content">
{
  this->sender_name_ =
    sender_name;

  this->receiver_name_ =
    receiver_name;

  this->receiver_ =
    AVStreams::MMDevice::_duplicate (receiver);

  CosNaming::Name name (1);
  name.length (1);

  int sender_context_exists = 0;

  ACE_TRY
    {
      // Try binding the sender context in the NS
      name [0].id =
        CORBA::string_dup (this->sender_name_.c_str ());

      CORBA::Object_var object =
        this->naming_client_->resolve (name,
                                       ACE_TRY_ENV);
      ACE_TRY_CHECK;

      //
      // We reach here if there was no exception raised in &lt;resolve&gt;.
      // Therefore, there must be a valid sender context available.
      //
      sender_context_exists = 1;

      this->sender_context_ =
        CosNaming::NamingContext::_narrow (object.in ());

      name [0].id =
        CORBA::string_dup ("Receivers");

      // Find the receivers context under the sender's context
      object =
        this->sender_context_->resolve (name,
                                        ACE_TRY_ENV);
      ACE_CHECK;

      this->receiver_context_ =
        CosNaming::NamingContext::_narrow (object.in ());
    }
  ACE_CATCH (CosNaming::NamingContext::NotFound, al_ex)
    {
      name [0].id =
        CORBA::string_dup (this->sender_name_.c_str ());

      // Create the sender context
      this->sender_context_ =
        this->naming_client_->bind_new_context (name,
                                                ACE_TRY_ENV);
      ACE_CHECK;

      name [0].id =
        CORBA::string_dup ("Receivers");

      // Create the receivers context under the sender's context
      this->receiver_context_ =
        this->sender_context_->bind_new_context (name,
                                                 ACE_TRY_ENV);
      ACE_CHECK;
    }
  ACE_ENDTRY;
  ACE_CHECK;

  //
  // At this point we either have resolved the receiver context or we
  // have created a new one.
  //
  name [0].id =
    CORBA::string_dup (this->receiver_name_.c_str ());

  // Register this receiver object under the receiver context.
  this->receiver_context_->rebind (name,
                                   receiver,
                                   ACE_TRY_ENV);
  ACE_CHECK;

  //
  // Check if the sender was registered.  Note that if we created the
  // sender context, there is no point in checking for the sender.
  //
  if (sender_context_exists)
    {
      ACE_TRY
        {
          // Try binding the sender under the sender context
          name [0].id =
            CORBA::string_dup (this->sender_name_.c_str ());

          CORBA::Object_var object =
            this->sender_context_->resolve (name,
                                            ACE_TRY_ENV);
          ACE_TRY_CHECK;

          this->sender_ =
            AVStreams::MMDevice::_narrow (object.in ());
        }
      ACE_CATCH (CosNaming::NamingContext::NotFound, al_ex)
        {
          // No problem if the sender was not there.
        }
      ACE_ENDTRY;
      ACE_CHECK;
    }
}
    </xsl:with-param>
  </xsl:call-template>



  <xsl:call-template name="method_imp">
    <xsl:with-param name="name">
      <xsl:value-of select="concat($name, '::connect_to_sender')"/></xsl:with-param>
    <xsl:with-param name="type">void</xsl:with-param>
    <xsl:with-param name="params">{AVStreams::streamQoS&amp; the_qos}{int use_qos}{CORBA::Environment &amp;ACE_TRY_ENV}</xsl:with-param>
    <xsl:with-param name="content">
{
  if (CORBA::is_nil (this->sender_.in ()))
    return;

  ACE_CString protocol;
  if( use_qos == 1)
  {
          protocol="QoS_UDP";
  }
  else
  {
          protocol="UDP";
  }

  ACE_CString flowname;
  flowname =
    this->sender_name_ +
    "_" +
    this->receiver_name_;

  // Create the forward flow specification to describe the flow.
  TAO_Forward_FlowSpec_Entry sender_entry (flowname.c_str (),
                                           "IN",
                                           "USER_DEFINED",
                                           "",
                                           protocol.c_str(),
                                           0);

  // Set the flow specification for the stream between sender and
  // receiver.
  AVStreams::flowSpec flow_spec (1);
  flow_spec.length (1);
  flow_spec [0] =
    CORBA::string_dup (sender_entry.entry_to_string ());

  // Create the stream control for this stream
  TAO_StreamCtrl* streamctrl;
  ACE_NEW (streamctrl,
           TAO_StreamCtrl);

  // Servant Reference Counting to manage lifetime
  //PortableServer::ServantBase_var safe_streamctrl =
  //  streamctrl;

  // Register streamctrl.
  AVStreams::StreamCtrl_var streamctrl_object =
    streamctrl->_this (ACE_TRY_ENV);
  ACE_CHECK;

      CORBA::ORB_ptr orb = TAO_AV_CORE::instance ()->orb(); 
      ACE_TString s = orb->object_to_string( streamctrl_object.in() );
      char tmp[20];
      ACE_OS::sprintf(tmp, "%u", (unsigned int)ACE_OS::getpid());
      ACE_TString fname("/tmp/streamctrl2_");
      fname += tmp;
      fname += ".ior";
      fstream ior_file(fname.c_str(), ios::out);
      ACE_Log_Msg* logger = ACE_Log_Msg::instance();
      logger->msg_ostream(&amp;ior_file);
      logger->set_flags(ACE_Log_Msg::OSTREAM);
      logger->log(LM_DEBUG, "%s\n", s.c_str() );
      logger->clr_flags(ACE_Log_Msg::OSTREAM);

  //
  // Since senders terminate the streams, we don't need the streamctrl
  // for these.
  //
  // this->streamctrls_.bind (flowname,
  //                          streamctrl_object);

  // Initialize the  QoS
  //AVStreams::streamQoS_var the_qos (new AVStreams::streamQoS);

  // Connect the sender and receiver devices.
  ACE_DEBUG((LM_DEBUG, "====STARTED bind_devs here %s %d\n",__FILE__,__LINE__));      ACE_DEBUG((LM_DEBUG, "=== Flowname: %s\n", flowname.c_str() ));
      for(CORBA::ULong i=0; i &lt; flow_spec.length(); i++){
            ACE_DEBUG((LM_DEBUG, "Flow spec: %s\n", flow_spec[i].in() ));       
      }
  CORBA::Boolean result =
    streamctrl->bind_devs (this->sender_.in (),
                           this->receiver_.in (),
                           the_qos/*.inout ()*/,
                           flow_spec,
                           ACE_TRY_ENV);
  ACE_CHECK;
  ACE_DEBUG((LM_DEBUG, "====STOPPED bind_devs here %s %d\n",__FILE__,__LINE__));
  if (result == 0)
    ACE_ERROR ((LM_ERROR,
                "Streamctrl::bind_devs failed\n"));

  // Start the data sending.
  AVStreams::flowSpec start_spec;
  streamctrl->start (start_spec,
                     ACE_TRY_ENV);
  ACE_CHECK;
}
    </xsl:with-param>
  </xsl:call-template>



  <xsl:call-template name="method_imp">
    <xsl:with-param name="name">
      <xsl:value-of select="concat($name, '::add_streamctrl')"/></xsl:with-param>
    <xsl:with-param name="type">void</xsl:with-param>
    <xsl:with-param name="params">{const ACE_CString &amp;flowname}{TAO_StreamEndPoint *endpoint}{CORBA::Environment &amp;ACE_TRY_ENV}</xsl:with-param>
    <xsl:with-param name="content">
{
  // Get the stream controller for this endpoint.
  CORBA::Any_ptr streamctrl_any =
    endpoint->get_property_value ("Related_StreamCtrl",
                                  ACE_TRY_ENV);
  ACE_CHECK;

  AVStreams::StreamCtrl_ptr streamctrl;

  *streamctrl_any >>= streamctrl;

  this->streamctrls_.bind (flowname,
                           streamctrl);
}
    </xsl:with-param>
  </xsl:call-template>

  <xsl:call-template name="method_imp">
    <xsl:with-param name="name">
      <xsl:value-of select="concat($name, '::destroy')"/></xsl:with-param>
    <xsl:with-param name="type">void</xsl:with-param>
    <xsl:with-param name="params">{CORBA::Environment &amp;ACE_TRY_ENV}</xsl:with-param>
    <xsl:with-param name="content">
{
  AVStreams::flowSpec stop_spec;

  // Destroy all the stream controls.
  for (StreamCtrls::iterator iterator = this->streamctrls_.begin ();
       iterator != this->streamctrls_.end ();
       ++iterator)
    {
      (*iterator).int_id_->destroy (stop_spec,
                                    ACE_TRY_ENV);
      ACE_CHECK;
    }
}
    </xsl:with-param>
  </xsl:call-template>

  <xsl:call-template name="method_imp">
    <xsl:with-param name="name">
      <xsl:value-of select="concat($name, '::destroy')"/></xsl:with-param>
    <xsl:with-param name="type">void</xsl:with-param>
    <xsl:with-param name="params">{const ACE_CString &amp;flowname}{CORBA::Environment &amp;ACE_TRY_ENV}</xsl:with-param>
    <xsl:with-param name="content">
{
  this->protocol_objects_.unbind (flowname);
  this->receivers_.unbind (flowname);

  AVStreams::StreamCtrl_var streamctrl;
  this->streamctrls_.unbind (flowname,
                             streamctrl);

  AVStreams::flowSpec stop_spec;
  streamctrl->destroy (stop_spec,
                       ACE_TRY_ENV);
  ACE_CHECK;
}
    </xsl:with-param>
  </xsl:call-template>

  <xsl:call-template name="method_imp">
    <xsl:with-param name="name">
      <xsl:value-of select="concat($name, '::receivers')"/></xsl:with-param>
    <xsl:with-param name="type">Connection_Manager::Receivers &amp;</xsl:with-param>
    <xsl:with-param name="params">{none}</xsl:with-param>
    <xsl:with-param name="content">
{
  return this->receivers_;
}
    </xsl:with-param>
  </xsl:call-template>


  <xsl:call-template name="method_imp">
    <xsl:with-param name="name">
      <xsl:value-of select="concat($name, '::protocol_objects')"/></xsl:with-param>
    <xsl:with-param name="type">Connection_Manager::Protocol_Objects &amp;</xsl:with-param>
    <xsl:with-param name="params">{void}</xsl:with-param>
    <xsl:with-param name="content">
{
  return this->protocol_objects_;
}
    </xsl:with-param>
  </xsl:call-template>


  <xsl:call-template name="method_imp">
    <xsl:with-param name="name">
      <xsl:value-of select="concat($name, '::streamctrls')"/></xsl:with-param>
    <xsl:with-param name="type">Connection_Manager::StreamCtrls &amp;</xsl:with-param>
    <xsl:with-param name="params">{none}</xsl:with-param>
    <xsl:with-param name="content">
{
  return this->streamctrls_;
}
    </xsl:with-param>
  </xsl:call-template>

  <!-- post typedef -->
  <xsl:text>
#if defined (ACE_HAS_EXPLICIT_TEMPLATE_INSTANTIATION)

template class ACE_Hash_Map_Entry&lt;ACE_CString, AVStreams::MMDevice_var>;
template class ACE_Hash_Map_Manager&lt;ACE_CString, AVStreams::MMDevice_var, ACE_Null_Mutex>;
template class ACE_Hash_Map_Manager_Ex&lt;ACE_CString, AVStreams::MMDevice_var, ACE_Hash&lt;ACE_CString>, ACE_Equal_To&lt;ACE_CString>, ACE_Null_Mutex>;
template class ACE_Hash_Map_Iterator_Base_Ex&lt;ACE_CString, AVStreams::MMDevice_var, ACE_Hash&lt;ACE_CString>, ACE_Equal_To&lt;ACE_CString>, ACE_Null_Mutex>;
template class ACE_Hash_Map_Iterator_Ex&lt;ACE_CString, AVStreams::MMDevice_var, ACE_Hash&lt;ACE_CString>, ACE_Equal_To&lt;ACE_CString>, ACE_Null_Mutex>;
template class ACE_Hash_Map_Reverse_Iterator_Ex&lt;ACE_CString, AVStreams::MMDevice_var, ACE_Hash&lt;ACE_CString>, ACE_Equal_To&lt;ACE_CString>, ACE_Null_Mutex>;

template class ACE_Hash_Map_Entry&lt;ACE_CString, TAO_AV_Protocol_Object *>;
template class ACE_Hash_Map_Manager&lt;ACE_CString, TAO_AV_Protocol_Object *, ACE_Null_Mutex>;
template class ACE_Hash_Map_Manager_Ex&lt;ACE_CString, TAO_AV_Protocol_Object *, ACE_Hash&lt;ACE_CString>, ACE_Equal_To&lt;ACE_CString>, ACE_Null_Mutex>;
template class ACE_Hash_Map_Iterator_Base_Ex&lt;ACE_CString, TAO_AV_Protocol_Object *, ACE_Hash&lt;ACE_CString>, ACE_Equal_To&lt;ACE_CString>, ACE_Null_Mutex>;
template class ACE_Hash_Map_Iterator_Ex&lt;ACE_CString, TAO_AV_Protocol_Object *, ACE_Hash&lt;ACE_CString>, ACE_Equal_To&lt;ACE_CString>, ACE_Null_Mutex>;
template class ACE_Hash_Map_Reverse_Iterator_Ex&lt;ACE_CString, TAO_AV_Protocol_Object *, ACE_Hash&lt;ACE_CString>, ACE_Equal_To&lt;ACE_CString>, ACE_Null_Mutex>;

template class ACE_Hash_Map_Entry&lt;ACE_CString, AVStreams::StreamCtrl_var>;
template class ACE_Hash_Map_Manager&lt;ACE_CString, AVStreams::StreamCtrl_var, ACE_Null_Mutex>;
template class ACE_Hash_Map_Manager_Ex&lt;ACE_CString, AVStreams::StreamCtrl_var, ACE_Hash&lt;ACE_CString>, ACE_Equal_To&lt;ACE_CString>, ACE_Null_Mutex>;
template class ACE_Hash_Map_Iterator_Base_Ex&lt;ACE_CString, AVStreams::StreamCtrl_var, ACE_Hash&lt;ACE_CString>, ACE_Equal_To&lt;ACE_CString>, ACE_Null_Mutex>;
template class ACE_Hash_Map_Iterator_Ex&lt;ACE_CString, AVStreams::StreamCtrl_var, ACE_Hash&lt;ACE_CString>, ACE_Equal_To&lt;ACE_CString>, ACE_Null_Mutex>;
template class ACE_Hash_Map_Reverse_Iterator_Ex&lt;ACE_CString, AVStreams::StreamCtrl_var, ACE_Hash&lt;ACE_CString>, ACE_Equal_To&lt;ACE_CString>, ACE_Null_Mutex>;
//template class NamingHelper&lt;CosNaming::NamingContext>;
//template class NamingHelper&lt;AVStreams::Negotiator>;

#elif defined (ACE_HAS_TEMPLATE_INSTANTIATION_PRAGMA)

#pragma instantiate ACE_Hash_Map_Entry&lt;ACE_CString, AVStreams::MMDevice_var>
#pragma instantiate ACE_Hash_Map_Manager&lt;ACE_CString, AVStreams::MMDevice_var, ACE_Null_Mutex>
#pragma instantiate ACE_Hash_Map_Manager_Ex&lt;ACE_CString, AVStreams::MMDevice_var, ACE_Hash&lt;ACE_CString>, ACE_Equal_To&lt;ACE_CString>, ACE_Null_Mutex>
#pragma instantiate ACE_Hash_Map_Iterator_Base_Ex&lt;ACE_CString, AVStreams::MMDevice_var, ACE_Hash&lt;ACE_CString>, ACE_Equal_To&lt;ACE_CString>, ACE_Null_Mutex>
#pragma instantiate ACE_Hash_Map_Iterator_Ex&lt;ACE_CString, AVStreams::MMDevice_var, ACE_Hash&lt;ACE_CString>, ACE_Equal_To&lt;ACE_CString>, ACE_Null_Mutex>
#pragma instantiate ACE_Hash_Map_Reverse_Iterator_Ex&lt;ACE_CString, AVStreams::MMDevice_var, ACE_Hash&lt;ACE_CString>, ACE_Equal_To&lt;ACE_CString>, ACE_Null_Mutex>

#pragma instantiate ACE_Hash_Map_Entry&lt;ACE_CString, TAO_AV_Protocol_Object *>
#pragma instantiate ACE_Hash_Map_Manager&lt;ACE_CString, TAO_AV_Protocol_Object *, ACE_Null_Mutex>
#pragma instantiate ACE_Hash_Map_Manager_Ex&lt;ACE_CString, TAO_AV_Protocol_Object *, ACE_Hash&lt;ACE_CString>, ACE_Equal_To&lt;ACE_CString>, ACE_Null_Mutex>
#pragma instantiate ACE_Hash_Map_Iterator_Base_Ex&lt;ACE_CString, TAO_AV_Protocol_Object *, ACE_Hash&lt;ACE_CString>, ACE_Equal_To&lt;ACE_CString>, ACE_Null_Mutex>
#pragma instantiate ACE_Hash_Map_Iterator_Ex&lt;ACE_CString, TAO_AV_Protocol_Object *, ACE_Hash&lt;ACE_CString>, ACE_Equal_To&lt;ACE_CString>, ACE_Null_Mutex>
#pragma instantiate ACE_Hash_Map_Reverse_Iterator_Ex&lt;ACE_CString, TAO_AV_Protocol_Object *, ACE_Hash&lt;ACE_CString>, ACE_Equal_To&lt;ACE_CString>, ACE_Null_Mutex>

#pragma instantiate ACE_Hash_Map_Entry&lt;ACE_CString, AVStreams::StreamCtrl_var>
#pragma instantiate ACE_Hash_Map_Manager&lt;ACE_CString, AVStreams::StreamCtrl_var, ACE_Null_Mutex>
#pragma instantiate ACE_Hash_Map_Manager_Ex&lt;ACE_CString, AVStreams::StreamCtrl_var, ACE_Hash&lt;ACE_CString>, ACE_Equal_To&lt;ACE_CString>, ACE_Null_Mutex>
#pragma instantiate ACE_Hash_Map_Iterator_Base_Ex&lt;ACE_CString, AVStreams::StreamCtrl_var, ACE_Hash&lt;ACE_CString>, ACE_Equal_To&lt;ACE_CString>, ACE_Null_Mutex>
#pragma instantiate ACE_Hash_Map_Iterator_Ex&lt;ACE_CString, AVStreams::StreamCtrl_var, ACE_Hash&lt;ACE_CString>, ACE_Equal_To&lt;ACE_CString>, ACE_Null_Mutex>
#pragma instantiate ACE_Hash_Map_Reverse_Iterator_Ex&lt;ACE_CString, AVStreams::StreamCtrl_var, ACE_Hash&lt;ACE_CString>, ACE_Equal_To&lt;ACE_CString>, ACE_Null_Mutex>
#pragma instantiate NamingHelper&lt;CosNaming::NamingContext>;
#pragma instantiate NamingHelper&lt;AVStreams::Negotiator>;

#endif /* ACE_HAS_EXPLICIT_TEMPLATE_INSTANTIATION */
</xsl:text>


   </content></file>
</xsl:template>
<!-- ============================================= -->
<!--  End of The Connection_Manager_CPP template   -->
<!-- ============================================= -->




  
<!-- ============================================= -->
<!--      The Connection_Manager_H template        -->
<!-- ============================================= -->


<xsl:template name="Connection_Manager_H">
  <xsl:param name="name"/>

  <!-- Capitalized name -->
  <xsl:variable name="name_all_capital">
    <xsl:value-of select="translate($name,'abcdefghijklmnopqrstuvwxyz','ABCDEFGHIJKLMNOPQRSTUVWXYZ')"/>
  </xsl:variable>
  <xsl:variable name="name_all_small">
    <xsl:value-of
    select="translate($name,'ABCDEFGHIJKLMNOPQRSTUVWXYZ','abcdefghijklmnopqrstuvwxyz')"/>
  </xsl:variable>

  <xsl:value-of select="$empty_line"/>
  <xsl:value-of select="$empty_line"/>

  <file><name>
     <xsl:value-of select="$dir_avstreams"/><xsl:value-of select="$name"/>.h</name><content>

  <xsl:value-of select="$file_header"/>

  <xsl:call-template name="comment">
    <xsl:with-param name="value">
      <xsl:value-of select="concat($name,'.h')"/>
    </xsl:with-param>
  </xsl:call-template>
  <!-- include ifdef-endif -->

  <xsl:value-of select="$empty_line"/>

#ifndef _<xsl:value-of select="$name_all_capital"/>_MANAGER_H_
#define _<xsl:value-of select="$name_all_capital"/>_MANAGER_H_


  <xsl:value-of select="$empty_line"/>

  <!-- The system include -->
  <xsl:call-template name="use">
    <xsl:with-param name="files">{orbsvcs/Naming/Naming_Utils.h}{orbsvcs/AV/AVStreams_i.h}{orbsvcs/AV/Protocol_Factory.h}{tao/PortableServer/PortableServer.h}</xsl:with-param>
  </xsl:call-template>

  <xsl:value-of select="$empty_line"/>


  <xsl:call-template name="comment">
    <xsl:with-param name="value">
      <xsl:value-of select="concat($name,' class: This is a helper class that allows the senders bind to multiple receivers and receivers to bind to senders. It also lets the senders and receivers disconnect streams.')"/>
    </xsl:with-param>
  </xsl:call-template>

  <xsl:value-of select="$empty_line"/>
  <!-- The class declaration -->
  <xsl:call-template name="class">
    <xsl:with-param name="name"><xsl:value-of select="$name"/></xsl:with-param>
  </xsl:call-template>

  <!-- The class method -->
  <xsl:call-template name="method">
    <xsl:with-param name="name"><xsl:value-of select="$name"/></xsl:with-param>
    <xsl:with-param name="visibility">public</xsl:with-param>
    <xsl:with-param name="type">none</xsl:with-param>
    <xsl:with-param name="params">{none}</xsl:with-param>
  </xsl:call-template>

  <xsl:call-template name="method">
    <xsl:with-param name="name"><xsl:value-of select="concat('~',$name)"/></xsl:with-param>
    <xsl:with-param name="visibility">public</xsl:with-param>
    <xsl:with-param name="type">none</xsl:with-param>
    <xsl:with-param name="params">{none}</xsl:with-param>
  </xsl:call-template>

  <xsl:call-template name="method">
    <xsl:with-param name="name">init</xsl:with-param>
    <xsl:with-param name="visibility">public</xsl:with-param>
    <xsl:with-param name="type">int</xsl:with-param>
    <xsl:with-param name="params">{CORBA::ORB_ptr orb}</xsl:with-param>
  </xsl:call-template>

  <xsl:call-template name="method">
    <xsl:with-param name="name">bind_to_receivers</xsl:with-param>
    <xsl:with-param name="visibility">public</xsl:with-param>
    <xsl:with-param name="type">void</xsl:with-param>
    <xsl:with-param name="params">{const ACE_CString &amp;sender_name}{AVStreams::MMDevice_ptr sender}{CORBA::Environment &amp;ACE_TRY_ENV = TAO_default_environment ()}</xsl:with-param>
  </xsl:call-template>
  
  <xsl:call-template name="method">
    <xsl:with-param name="name">connect_to_receivers</xsl:with-param>
    <xsl:with-param name="visibility">public</xsl:with-param>
    <xsl:with-param name="type">void</xsl:with-param>
    <xsl:with-param name="params">{AVStreams::MMDevice_ptr sender}{AVStreams::streamQoS&amp; the_qos}{int use_qos}{CORBA::Environment &amp;ACE_TRY_ENV = TAO_default_environment ()}</xsl:with-param>
  </xsl:call-template>

  <xsl:call-template name="method">
    <xsl:with-param name="name">bind_to_sender</xsl:with-param>
    <xsl:with-param name="visibility">public</xsl:with-param>
    <xsl:with-param name="type">void</xsl:with-param>
    <xsl:with-param name="params">{const ACE_CString &amp;sender_name}{const ACE_CString &amp;receiver_name}{AVStreams::MMDevice_ptr receiver}{CORBA::Environment &amp;ACE_TRY_ENV = TAO_default_environment ()}</xsl:with-param>
  </xsl:call-template>


  <xsl:call-template name="method">
    <xsl:with-param name="name">connect_to_sender</xsl:with-param>
    <xsl:with-param name="visibility">public</xsl:with-param>
    <xsl:with-param name="type">void</xsl:with-param>
    <xsl:with-param name="params">{AVStreams::streamQoS &amp;the_qos}{int use_qos}{CORBA::Environment&amp; ACE_TRY_ENV = TAO_default_environment ()}</xsl:with-param>
  </xsl:call-template>

  <!--
  <xsl:call-template name="method">
    <xsl:with-param name="name">connect_to_negotiator</xsl:with-param>
    <xsl:with-param name="visibility">public</xsl:with-param>
    <xsl:with-param name="type">void</xsl:with-param>
    <xsl:with-param name="params">{TAO_StreamEndPoint *endpoint}{const char *flowname}{CORBA::Environment &amp;ACE_TRY_ENV = TAO_default_environment ()}</xsl:with-param>
  </xsl:call-template>
  -->

  <xsl:call-template name="method">
    <xsl:with-param name="name">destroy</xsl:with-param>
    <xsl:with-param name="visibility">public</xsl:with-param>
    <xsl:with-param name="type">void</xsl:with-param>
    <xsl:with-param name="params">{CORBA::Environment &amp; = TAO_default_environment ()}</xsl:with-param>
  </xsl:call-template>

  <xsl:call-template name="method">
    <xsl:with-param name="name">destroy</xsl:with-param>
    <xsl:with-param name="visibility">public</xsl:with-param>
    <xsl:with-param name="type">void</xsl:with-param>
    <xsl:with-param name="params">{const ACE_CString &amp;flowname}{CORBA::Environment &amp; = TAO_default_environment ()}</xsl:with-param>
  </xsl:call-template>

  <xsl:call-template name="method">
    <xsl:with-param name="name">get_callback</xsl:with-param>
    <xsl:with-param name="visibility">public</xsl:with-param>
    <xsl:with-param name="type">int</xsl:with-param>
    <xsl:with-param name="params">{const char *flowname}{TAO_AV_Callback *&amp;callback}</xsl:with-param>
  </xsl:call-template>

  <xsl:call-template name="method">
    <xsl:with-param name="name">add_streamctrl</xsl:with-param>
    <xsl:with-param name="visibility">public</xsl:with-param>
    <xsl:with-param name="type">void</xsl:with-param>
    <xsl:with-param name="params">{const ACE_CString &amp;flowname}{TAO_StreamEndPoint *endpoint}{CORBA::Environment &amp;ACE_TRY_ENV = TAO_default_environment ()}</xsl:with-param>
  </xsl:call-template>

  <!-- The forward typedef -->
<xsl:text>

  // Map of receivers.
  typedef ACE_Hash_Map_Manager&lt;ACE_CString,
                               AVStreams::MMDevice_var,
                               ACE_Null_Mutex>
          Receivers;

  // Map of protocol objects.
  typedef ACE_Hash_Map_Manager&lt;ACE_CString,
                               TAO_AV_Protocol_Object *,
                               ACE_Null_Mutex&gt;
          Protocol_Objects;

  // Map of streamctrl.
  typedef ACE_Hash_Map_Manager&lt;ACE_CString,
                               AVStreams::StreamCtrl_var,
                               ACE_Null_Mutex&gt;
          StreamCtrls;

</xsl:text>

  <xsl:call-template name="method">
    <xsl:with-param name="name">receivers</xsl:with-param>
    <xsl:with-param name="visibility">public</xsl:with-param>
    <xsl:with-param name="type">Receivers&amp;</xsl:with-param>
    <xsl:with-param name="params">{none}</xsl:with-param>
  </xsl:call-template>

  <xsl:call-template name="method">
    <xsl:with-param name="name">protocol_objects</xsl:with-param>
    <xsl:with-param name="visibility">public</xsl:with-param>
    <xsl:with-param name="type">Protocol_Objects&amp;</xsl:with-param>
    <xsl:with-param name="params">{none}</xsl:with-param>
  </xsl:call-template>
  
  <xsl:call-template name="method">
    <xsl:with-param name="name">streamctrls</xsl:with-param>
    <xsl:with-param name="visibility">public</xsl:with-param>
    <xsl:with-param name="type">StreamCtrls&amp;</xsl:with-param>
    <xsl:with-param name="params">{none}</xsl:with-param>
  </xsl:call-template>

  <xsl:call-template name="method">
    <xsl:with-param name="name">find_receivers</xsl:with-param>
    <xsl:with-param name="visibility">protected</xsl:with-param>
    <xsl:with-param name="type">void</xsl:with-param>
    <xsl:with-param name="params">{CORBA::Environment &amp;ACE_TRY_ENV}</xsl:with-param>
  </xsl:call-template>


  <xsl:call-template name="method">
    <xsl:with-param name="name">add_to_receivers</xsl:with-param>
    <xsl:with-param name="visibility">protected</xsl:with-param>
    <xsl:with-param name="type">void</xsl:with-param>
    <xsl:with-param name="params">{CosNaming::BindingList &amp;binding_list}{CORBA::Environment&amp; ACE_TRY_ENV}</xsl:with-param>
  </xsl:call-template>


  <!-- The class member field -->
  <xsl:call-template name="data_member">
    <xsl:with-param name="name">naming_client_</xsl:with-param>
    <xsl:with-param name="visibility">protected</xsl:with-param>
    <xsl:with-param name="type">TAO_Naming_Client</xsl:with-param>
  </xsl:call-template>

  <xsl:call-template name="data_member">
    <xsl:with-param name="name">receivers_</xsl:with-param>
    <xsl:with-param name="visibility">protected</xsl:with-param>
    <xsl:with-param name="type">Receivers</xsl:with-param>
  </xsl:call-template>

  <xsl:call-template name="data_member">
    <xsl:with-param name="name">protocol_objects_</xsl:with-param>
    <xsl:with-param name="visibility">protected</xsl:with-param>
    <xsl:with-param name="type">Protocol_Objects</xsl:with-param>
  </xsl:call-template>

  <xsl:call-template name="data_member">
    <xsl:with-param name="name">streamctrls_</xsl:with-param>
    <xsl:with-param name="visibility">protected</xsl:with-param>
    <xsl:with-param name="type">StreamCtrls</xsl:with-param>
  </xsl:call-template>

  <xsl:call-template name="data_member">
    <xsl:with-param name="name">sender_name_</xsl:with-param>
    <xsl:with-param name="visibility">protected</xsl:with-param>
    <xsl:with-param name="type">ACE_CString</xsl:with-param>
  </xsl:call-template>

  <xsl:call-template name="data_member">
    <xsl:with-param name="name">sender_</xsl:with-param>
    <xsl:with-param name="visibility">protected</xsl:with-param>
    <xsl:with-param name="type">AVStreams::MMDevice_var</xsl:with-param>
  </xsl:call-template>

  <xsl:call-template name="data_member">
    <xsl:with-param name="name">sender_context_</xsl:with-param>
    <xsl:with-param name="visibility">protected</xsl:with-param>
    <xsl:with-param name="type">CosNaming::NamingContext_var</xsl:with-param>
  </xsl:call-template>

  <xsl:call-template name="data_member">
    <xsl:with-param name="name">receiver_name_</xsl:with-param>
    <xsl:with-param name="visibility">protected</xsl:with-param>
    <xsl:with-param name="type">ACE_CString</xsl:with-param>
  </xsl:call-template>

  <xsl:call-template name="data_member">
    <xsl:with-param name="name">receiver_</xsl:with-param>
    <xsl:with-param name="visibility">protected</xsl:with-param>
    <xsl:with-param name="type">AVStreams::MMDevice_var</xsl:with-param>
  </xsl:call-template>

  <xsl:call-template name="data_member">
    <xsl:with-param name="name">receiver_context_</xsl:with-param>
    <xsl:with-param name="visibility">protected</xsl:with-param>
    <xsl:with-param name="type">CosNaming::NamingContext_var</xsl:with-param>
  </xsl:call-template>

  <xsl:call-template name="data_member">
    <xsl:with-param name="name">_orb</xsl:with-param>
    <xsl:with-param name="visibility">private</xsl:with-param>
    <xsl:with-param name="type">CORBA::ORB_var</xsl:with-param>
  </xsl:call-template>



  <!-- The closing of class -->
  <xsl:text>
};// class: </xsl:text>
<xsl:value-of select="$name"/>

<xsl:text>

</xsl:text>

  <xsl:value-of select="$empty_line"/>

#endif // _<xsl:value-of select="$name_all_capital"/>_MANAGER_H_



  </content></file>
</xsl:template>

</xsl:stylesheet>
