<xsl:stylesheet version="1.0"                                                
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform">

<!-- ============================================= -->
<!--      The Connection_Manager_CPP template      -->
<!-- ============================================= -->


<xsl:template name="CtrlManager_CPP">
  <xsl:param name="name"/>

  <!-- Capitalized name -->
  <xsl:variable name="name_all_capital">
    <xsl:value-of select="translate($name,'abcdefghijklmnopqrstuvwxyz','ABCDEFGHIJKLMNOPQRSTUVWXYZ')"/>
  </xsl:variable>
  <xsl:variable name="name_all_small">
    <xsl:value-of
    select="translate($name,'ABCDEFGHIJKLMNOPQRSTUVWXYZ','abcdefghijklmnopqrstuvwxyz')"/>
  </xsl:variable>

  <xsl:value-of select="$empty_line"/>
  <xsl:value-of select="$empty_line"/>

  <file><name>
    <xsl:value-of select="$dir_output_root"/><xsl:value-of select="$name"/>.cpp</name><content>

  <xsl:value-of select="$file_header"/>

  <xsl:call-template name="comment">
    <xsl:with-param name="value">
      <xsl:value-of select="concat($name,'.cpp')"/>
    </xsl:with-param>
  </xsl:call-template>


  <!-- The system include -->
  <xsl:call-template name="dependency">
    <xsl:with-param name="files">
      <xsl:value-of select="concat('{',$name,'.h}')"/></xsl:with-param>
  </xsl:call-template>

  <xsl:value-of select="$empty_line"/>

  <!-- class methods -->
  <!-- Constructor -->
  <xsl:call-template name="method_imp">
    <xsl:with-param name="name">
      <xsl:value-of select="concat($name, '::', $name)"/></xsl:with-param>
    <xsl:with-param name="type">none</xsl:with-param>
    <xsl:with-param name="params">{none}</xsl:with-param>
    <xsl:with-param name="content">
{
}
    </xsl:with-param>
  </xsl:call-template>

  <!-- Destructor -->
  <xsl:call-template name="method_imp">
    <xsl:with-param name="name">
      <xsl:value-of select="concat($name, '::~', $name)"/></xsl:with-param>
    <xsl:with-param name="type">none</xsl:with-param>
    <xsl:with-param name="params">{none}</xsl:with-param>
    <xsl:with-param name="content">
{
}
    </xsl:with-param>
  </xsl:call-template>

  <xsl:call-template name="method_imp">
    <xsl:with-param name="name">
      <xsl:value-of select="concat($name, '::init')"/></xsl:with-param>
    <xsl:with-param name="type">int</xsl:with-param>
    <xsl:with-param name="params">{CORBA::ORB_ptr orb}</xsl:with-param>
    <xsl:with-param name="content">
{
  if (0 != this->namingClient.init(orb))
    ACE_ERROR_RETURN ((LM_ERROR, " (%P|%t) Unable to init " "TAO_Naming_client\n."), -1);

  this->_orb = CORBA::ORB::_duplicate(orb);
  return 0;
}
    </xsl:with-param>
  </xsl:call-template>


  <xsl:call-template name="method_imp">
    <xsl:with-param name="name">
      <xsl:value-of select="concat($name, '::bindToReceivers')"/></xsl:with-param>
    <xsl:with-param name="type">void</xsl:with-param>
    <xsl:with-param name="params">{const ACE_CString &amp;senderName}{const
    ACE_CString &amp; receiverName}{AVStreams::MMDevice_ptr sender}{CORBA::Environment &amp;ACE_TRY_ENV}</xsl:with-param>
    <xsl:with-param name="content">
{
  this->senderName = senderName;

  CosNaming::Name name (1);
  name.length (1);

  ACE_TRY
    {
      name[0].id =
        CORBA::string_dup (senderName.c_str());

      this->senderContext =
        this->namingClient->bind_new_context (name
	                                      ACE_ENV_ARG_PARAMETER);
      ACE_TRY_CHECK;

      ACE_DEBUG((LM_DEBUG,
                 "(%P|%t) CtrlManager::bindToReceiver(): after senderContext: senderName: %s, receiverName: %s\n",senderName.c_str(), receiverName.c_str()));

      name [0].id =
        CORBA::string_dup (receiverName.c_str()/*"Receivers"*/);

      this->receiverContext =
        this->senderContext->bind_new_context (name
	                                       ACE_ENV_ARG_PARAMETER);
      ACE_TRY_CHECK;

      ACE_DEBUG((LM_DEBUG,
                 "(%P|%t) CtrlManager::bindToReceiver(): after receiverContext:senderName: %s, receiverName: %s\n",senderName.c_str(), receiverName.c_str()));

    }
  ACE_CATCH (CosNaming::NamingContext::AlreadyBound, al_ex)
    {
      name[0].id = CORBA::string_dup (this->senderName.c_str ());

      ACE_DEBUG((LM_DEBUG,
                 "(%P|%t) CtrlManager::bindToReceiver(): ACE_CATCH\n                 senderName: %s, receiverName: %s\n",senderName.c_str(), receiverName.c_str()));

      CORBA::Object_var object =
        this->namingClient->resolve (name
	                             ACE_ENV_ARG_PARAMETER);
      ACE_CHECK;

      this->senderContext =
        CosNaming::NamingContext::_narrow (object.in ());

      name[0].id = CORBA::string_dup (receiverName.c_str()/*"Receivers"*/);

      object =
        this->senderContext->resolve (name
                                      ACE_ENV_ARG_PARAMETER);
      ACE_CHECK;

      this->receiverContext =
        CosNaming::NamingContext::_narrow (object.in ());

      this->resolveReceivers (ACE_TRY_ENV);
      ACE_CHECK;

      ACE_DEBUG((LM_DEBUG,
                 "(%P|%t) CtrlManager::bindToReceiver(): ACE_CATCH\nreceivers resolved\n"));

    }
  ACE_ENDTRY;
  ACE_CHECK;

  name [0].id =
    CORBA::string_dup (this->senderName.c_str ());

  this->senderContext->rebind (name, sender
                               ACE_ENV_ARG_PARAMETER);
  ACE_CHECK;
}
    </xsl:with-param>
  </xsl:call-template>

  <xsl:call-template name="method_imp">
    <xsl:with-param name="name">
      <xsl:value-of select="concat($name, '::connectToReceivers')"/></xsl:with-param>
    <xsl:with-param name="type">void</xsl:with-param>
    <xsl:with-param name="params">{AVStreams::MMDevice_ptr
    sender}{AVStreams::streamQoS&amp; theQos}{int useQos}{CORBA::Environment &amp;ACE_TRY_ENV}</xsl:with-param>
    <xsl:with-param name="content">
{
  ACE_CString protocol = "UDP";

  CORBA::Any_var property;
  CORBA::Any newProperty;

  AVStreams::flowSpec *ext_sender_flows, *ext_receiver_flows;
  AVStreams::flowSpec sender_flows, receiver_flows;
  AVStreams::flowSpec flow_spec(1);
  flow_spec.length(1);


  TAO_StreamCtrl* streamCtrl;
  AVStreams::StreamCtrl_var streamCtrl_object;

  ACE_TRY
    {
      property = sender->get_property_value("Flows");
      ACE_TRY_CHECK;
      *property >>= ext_sender_flows;
      sender_flows = AVStreams::flowSpec(*ext_sender_flows);
      ACE_DEBUG((LM_DEBUG,
                 " (%P|%t) CtrlManager::connectToReceivers(): sender has Flows\n"));
    }
  ACE_CATCH (CosPropertyService::PropertyNotFound, prop_ex)
    {
      ACE_DEBUG((LM_DEBUG,
                 " (%P|%t) CtrlManager::connectToReceivers(): sender has NO Flows\n"));
      sender_flows.length(0);
    }
  ACE_ENDTRY;
  ACE_CHECK;


  for (Receivers::iterator iterator = this->receivers.begin();
       iterator != this->receivers.end();
       ++iterator)
    {
      ACE_CString flowname =
        this->senderName +
        "_"+
        (*iterator).ext_id_;

      TAO_Forward_FlowSpec_Entry sender_entry (flowname.c_str(),
                                               "IN",
                                               "USER_DEFINED",
                                               "",
                                               protocol.c_str(),
                                               0);

      ACE_DEBUG((LM_DEBUG,
                 "(%P|%t)CtrlManager::connectToReceivers(): loop flowname: %s\n",flowname.c_str()));

      flow_spec[0] = CORBA::string_dup (sender_entry.entry_to_string());

      ACE_NEW (streamCtrl, TAO_StreamCtrl);

      PortableServer::ServantBase_var safe_streamCtrl = streamCtrl;

      streamCtrl_object = streamCtrl->_this (ACE_ENV_SINGLE_ARG_PARAMETER);
      ACE_CHECK;

      newProperty &lt;&lt;= streamCtrl_object.in();

      sender->define_property("Related_StreamCtrl", newProperty);

      //Register into the streamCtrls map
      this->streamCtrls.bind (flowname, streamCtrl_object);

      ACE_TRY_EX (label1)
        {
          property = (*iterator).int_id_ -> get_property_value("Flows");
          ACE_TRY_CHECK_EX(label);
          *property >>= ext_sender_flows;
          receiver_flows = AVStreams::flowSpec(*ext_receiver_flows);
          ACE_DEBUG((LM_DEBUG,
                     " (%P|%t) CtrlManager::connectToReceivers(): *iterator has Flows\n"));

        }
      ACE_CATCH (CosPropertyService::PropertyNotFound, prop_ex)
        {
          ACE_DEBUG((LM_DEBUG,
                     " (%P|%t) CtrlManager::connectToReceivers(): *iterator has NO Flows\n"));

          receiver_flows.length(0);
        }
      ACE_ENDTRY;
      ACE_CHECK;

      if (theQos.length() > 0)
        theQos[0].QoSType = CORBA::string_dup (flowname.c_str());

      int result = streamCtrl->bind_devs (sender,
                                          (*iterator).int_id_.in(),
                                          theQos,
                                          flow_spec
                                          ACE_ENV_ARG_PARAMETER);
      ACE_CHECK;
      ACE_DEBUG((LM_DEBUG, "(%P|%t)CtrlManager::bind_devs() here %s %d %d\n",__FILE__,__LINE__, result));

      for (unsigned int i=0; i&lt;flow_spec.length(); i++) {
        ACE_DEBUG((LM_DEBUG,
                   "(%P|%t)Adding flow spec: %s\n",
                   flow_spec[i].in()
                   ));

        sender_flows.length(sender_flows.length()+1);
        sender_flows [sender_flows.length()-1] =
          CORBA::string_dup (flow_spec[i]);

        receiver_flows.length(receiver_flows.length()+1);
        receiver_flows [receiver_flows.length()-1] =
          CORBA::string_dup (flow_spec[i]);
      }

      newProperty &lt;&lt;= receiver_flows;
      (*iterator).int_id_ -> define_property("Flows", newProperty);
    }//End of loop


  newProperty &lt;&lt;= sender_flows;
  sender->define_property("Flows", newProperty);
}
    </xsl:with-param>
  </xsl:call-template>


  <xsl:call-template name="method_imp">
    <xsl:with-param name="name">
      <xsl:value-of select="concat($name, '::bindToSender')"/></xsl:with-param>
    <xsl:with-param name="type">void</xsl:with-param>
    <xsl:with-param name="params">{const ACE_CString &amp;senderName}{const ACE_CString &amp;receiverName}{AVStreams::MMDevice_ptr receiver}{CORBA::Environment &amp;ACE_TRY_ENV}</xsl:with-param>
    <xsl:with-param name="content">
{
  this->senderName = senderName;
  this->receiverName = receiverName;

  this->receiver = AVStreams::MMDevice::_duplicate (receiver);

  int senderContextExt = 0;
  CosNaming::Name name (1);
  name.length (1);

  ACE_TRY
    {
      name[0].id =
        CORBA::string_dup (senderName.c_str());

      CORBA::Object_var object = this->namingClient->resolve(name
                                                             ACE_ENV_ARG_PARAMETER);
      ACE_TRY_CHECK;

      //if no exception, then...
      senderContextExt = 1;

      ACE_DEBUG((LM_DEBUG,
                 "(%P|%t)CtrlManager::bindToSender(): senderContextExt =1\n"));

      this->senderContext =
        CosNaming::NamingContext::_narrow(object.in());

      name [0].id =
        CORBA::string_dup (receiverName.c_str()/*"Receivers"*/);

      object = this->senderContext->resolve (name
                                             ACE_ENV_ARG_PARAMETER);

      ACE_CHECK;

      this->receiverContext =
        CosNaming::NamingContext::_narrow (object.in());

      ACE_DEBUG((LM_DEBUG,
                 "(%P|%t)CtrlManager::bindToSender(): senderContextExt =1\n
                  receiverContext (0x%d) resolved.\n", &amp;receiverContext));

    }
  ACE_CATCH (CosNaming::NamingContext::NotFound, al_ex)
    {
      ACE_DEBUG((LM_DEBUG,
                 "(%P|%t)CtrlManager::bindToSender(): ACE_CATCH\n"));

      name[0].id = CORBA::string_dup (this->senderName.c_str ());

      this->senderContext =
        this->namingClient->bind_new_context (name
                                              ACE_ENV_ARG_PARAMETER);
      ACE_CHECK;

      name[0].id = CORBA::string_dup (receiverName.c_str()/*"Receivers"*/);

      this->receiverContext =
        this->senderContext->bind_new_context(name
                                              ACE_ENV_ARG_PARAMETER);

      ACE_CHECK;
    }
  ACE_ENDTRY;
  ACE_CHECK;

  name[0].id = CORBA::string_dup (this->receiverName.c_str());

  this->receiverContext->rebind (name, receiver
	                         ACE_ENV_ARG_PARAMETER);
  ACE_CHECK;

  if (senderContextExt) {
    ACE_TRY
      {
        name[0].id = CORBA::string_dup(this->senderName.c_str());
        CORBA::Object_var object =
          this->senderContext->resolve(name
                                       ACE_ENV_ARG_PARAMETER);

        ACE_TRY_CHECK;

        this->sender = AVStreams::MMDevice::_narrow (object.in());

        ACE_DEBUG((LM_DEBUG,
                   "(%P|%t)CtrlManager::bindToSender(): if (senderContextExt), sender
found at (0x%d)\n", &amp;sender));

      }
    ACE_CATCH (CosNaming::NamingContext::NotFound, al_ex)
      {
        ACE_DEBUG((LM_DEBUG,
                   "(%P|%t)CtrlManager::bindToSender(): if (senderContextExt), ACE_CATCH, naming not found exception"));

      }
    ACE_ENDTRY;
    ACE_CHECK;
  }
}
    </xsl:with-param>
  </xsl:call-template>

  <xsl:call-template name="method_imp">
    <xsl:with-param name="name">
      <xsl:value-of select="concat($name, '::connectToSender')"/></xsl:with-param>
    <xsl:with-param name="type">void</xsl:with-param>
    <xsl:with-param name="params">{AVStreams::streamQoS&amp; theQos}{int useQos}{CORBA::Environment &amp;ACE_TRY_ENV}</xsl:with-param>
    <xsl:with-param name="content">
{
  if (CORBA::is_nil(this->sender.in())) {
    ACE_DEBUG((LM_DEBUG, " (%P|%t) ConnectToSender(): sender is_nil, exit\n"));
    return;
  }

  ACE_DEBUG((LM_DEBUG, " (%P|%t) ConnectToSender(): sender is_not_nil\n"));

  ACE_CString protocol = "UDP";

  ACE_CString flowname =
    this->senderName +
    "_" +
    this->receiverName;

  TAO_Forward_FlowSpec_Entry sender_entry(flowname.c_str(),
                                          "IN",
                                          "USER_DEFINED",
                                          "",
                                          protocol.c_str(),
                                          0);

  ACE_DEBUG((LM_DEBUG, " (%P|%t)CtrlManager::connectToSender(): flowname: %s\n",flowname.c_str()));


  AVStreams::flowSpec flow_spec(1);
  flow_spec.length(1);
  flow_spec[0] = CORBA::string_dup(sender_entry.entry_to_string());

  CORBA::Any_var property;
  CORBA::Any newProperty;

  AVStreams::flowSpec *ext_sender_flows, sender_flows;
  AVStreams::flowSpec *ext_receiver_flows, receiver_flows;

  TAO_StreamCtrl* streamCtrl;
  AVStreams::StreamCtrl_var streamCtrl_object;

  ACE_DEBUG((LM_DEBUG, " (%P|%t)CtrlManager::connectToSender(): before streamCtrl NeW\n"));

  ACE_NEW (streamCtrl, TAO_StreamCtrl);

  ACE_DEBUG((LM_DEBUG, " (%P|%t)CtrlManager::connectToSender(): after streamCtrl NeW\n"));

  streamCtrl_object = streamCtrl->_this(ACE_ENV_SINGLE_ARG_PARAMETER);
  ACE_CHECK;

  //this->streamCtrls.bind (flowname, streamCtrl_object);
  ACE_DEBUG((LM_DEBUG, " (%P|%t)CtrlManager::connectToSender(): streamCtrl_object\n"));

  ACE_TRY
    {
      ACE_DEBUG((LM_DEBUG, " (%P|%t)CtrlManager::connectToSender(): enter TRY\n"));
      property = sender->get_property_value("Flows");
      ACE_TRY_CHECK;

      ACE_DEBUG((LM_DEBUG, " (%P|%t)CtrlManager::connectToSender(): enter TRY2\n"));

      *property >>= ext_sender_flows;
      sender_flows = AVStreams::flowSpec (*ext_sender_flows);
      ACE_DEBUG((LM_DEBUG, " (%P|%t)CtrlManager::connectToSender(): sender has Flow\n"));
    }
  ACE_CATCH (CosPropertyService::PropertyNotFound, prop_ex)
    {
      ACE_DEBUG((LM_DEBUG, " (%P|%t)CtrlManager::connectToSender(): sender has NO Flow\n"));
      sender_flows.length(0);
    }
  ACE_ENDTRY;
  ACE_CHECK;


  ACE_TRY
    {
      property = receiver->get_property_value("Flows");
      ACE_TRY_CHECK;
      *property >>= ext_receiver_flows;
      sender_flows = AVStreams::flowSpec(*ext_receiver_flows);
      ACE_DEBUG((LM_DEBUG, " (%P|%t)CtrlManager::connectToSender(): receiverer has Flow\n"));
    }
  ACE_CATCH (CosPropertyService::PropertyNotFound, prop_ex)
    {
      ACE_DEBUG((LM_DEBUG, " (%P|%t)CtrlManager::connectToSender(): receiver has NO Flow\n"));
      receiver_flows.length(0);
    }
  ACE_ENDTRY;
  ACE_CHECK;

  for (unsigned int i = 0; i &lt; flow_spec.length(); i++) {
    ACE_DEBUG((LM_DEBUG, "(%P|%t)Flow spec: %s\n", flow_spec[i].in()));
    sender_flows.length( sender_flows.length()+1);
    sender_flows [sender_flows.length()-1] = CORBA::string_dup(flow_spec[i]);

    receiver_flows.length( receiver_flows.length()+1);
    receiver_flows [receiver_flows.length()-1] =
      CORBA::string_dup(flow_spec[i]);
  }

  CORBA::Boolean result =
    streamCtrl->bind_devs (this->sender.in(),
                           this->receiver.in(),
                           theQos,
                           flow_spec
                           ACE_ENV_ARG_PARAMETER);
  ACE_CHECK;
  ACE_DEBUG((LM_DEBUG, "(%P|%t)bind_devs here %s %d\n",__FILE__,__LINE__));

  if (0==result)
    ACE_ERROR ((LM_ERROR,
                "bind_devs failed!\n"
                ));

  AVStreams::flowSpec start_spec;
  streamCtrl->start (start_spec
                     ACE_ENV_ARG_PARAMETER);
  ACE_CHECK;

  newProperty &lt;&lt;= sender_flows;
  this->sender->define_property("Flows", newProperty);

  newProperty &lt;&lt;= streamCtrl_object.in();
  this->sender->define_property("Related_StreamCtrl", newProperty);

  newProperty &lt;&lt;= receiver_flows;
  this->receiver->define_property("Flows", newProperty);
}
    </xsl:with-param>
  </xsl:call-template>


  <xsl:call-template name="method_imp">
    <xsl:with-param name="name">
      <xsl:value-of select="concat($name, '::destroy')"/></xsl:with-param>
    <xsl:with-param name="type">void</xsl:with-param>
    <xsl:with-param name="params">{CORBA::Environment &amp;ACE_TRY_ENV}</xsl:with-param>
    <xsl:with-param name="content">
{
  AVStreams::flowSpec stop_spec;

  for (StreamCtrls::iterator iterator = this->streamCtrls.begin ();
       iterator != this->streamCtrls.end ();
       ++iterator)
    {
      (*iterator).int_id_->destroy (stop_spec
                                    ACE_ENV_ARG_PARAMETER);
      ACE_CHECK;
    }
}
    </xsl:with-param>
  </xsl:call-template>

  <xsl:call-template name="method_imp">
    <xsl:with-param name="name">
      <xsl:value-of select="concat($name, '::destroy')"/></xsl:with-param>
    <xsl:with-param name="type">void</xsl:with-param>
    <xsl:with-param name="params">{const ACE_CString &amp;flowname}{CORBA::Environment &amp;ACE_TRY_ENV}</xsl:with-param>
    <xsl:with-param name="content">
{
  this->protocolObjects.unbind (flowname);
  this->receivers.unbind (flowname);

  AVStreams::StreamCtrl_var streamCtrl;
  this->streamCtrls.unbind (flowname,
                            streamCtrl);

  AVStreams::flowSpec stop_spec;
  streamCtrl->destroy (stop_spec
                       ACE_ENV_ARG_PARAMETER);
  ACE_CHECK;
}
    </xsl:with-param>
  </xsl:call-template>



  <xsl:call-template name="method_imp">
    <xsl:with-param name="name">
      <xsl:value-of select="concat($name, '::getReceivers')"/></xsl:with-param>
    <xsl:with-param name="type">CtrlManager::Receivers &amp;</xsl:with-param>
    <xsl:with-param name="params">{none}</xsl:with-param>
    <xsl:with-param name="content">
{
  return this->receivers;
}
    </xsl:with-param>
  </xsl:call-template>


  <xsl:call-template name="method_imp">
    <xsl:with-param name="name">
      <xsl:value-of select="concat($name, '::getProtocolObjects')"/></xsl:with-param>
    <xsl:with-param name="type">CtrlManager::ProtocolObjects &amp;</xsl:with-param>
    <xsl:with-param name="params">{void}</xsl:with-param>
    <xsl:with-param name="content">
{
  return this->protocolObjects;
}
    </xsl:with-param>
  </xsl:call-template>


  <xsl:call-template name="method_imp">
    <xsl:with-param name="name">
      <xsl:value-of select="concat($name, '::getStreamctrls')"/></xsl:with-param>
    <xsl:with-param name="type">CtrlManager::StreamCtrls &amp;</xsl:with-param>
    <xsl:with-param name="params">{none}</xsl:with-param>
    <xsl:with-param name="content">
{
  return this->streamCtrls;
}
    </xsl:with-param>
  </xsl:call-template>




  <xsl:call-template name="method_imp">
    <xsl:with-param name="name">
      <xsl:value-of select="concat($name, '::addToStreamCtrlsMap')"/></xsl:with-param>
    <xsl:with-param name="type">void</xsl:with-param>
    <xsl:with-param name="params">{const ACE_CString &amp;flowname}{TAO_StreamEndPoint *endpoint}{CORBA::Environment &amp;ACE_TRY_ENV}</xsl:with-param>
    <xsl:with-param name="content">
{
  CORBA::Any_ptr streamCtrl_ptr =
    endpoint->get_property_value ("Related_StreamCtrl"
                                  ACE_ENV_ARG_PARAMETER);
  ACE_CHECK;

  AVStreams::StreamCtrl_ptr streamCtrl;

  *streamCtrl_ptr >>= streamCtrl;

  this->streamCtrls.bind (flowname,
                          streamCtrl);
}
    </xsl:with-param>
  </xsl:call-template>


  <xsl:call-template name="method_imp">
    <xsl:with-param name="name">
      <xsl:value-of select="concat($name, '::addToReceiversMap')"/></xsl:with-param>
    <xsl:with-param name="type">void</xsl:with-param>
    <xsl:with-param name="params">{CosNaming::BindingList&amp; bindingList}{CORBA::Environment &amp;ACE_TRY_ENV}</xsl:with-param>
    <xsl:with-param name="content">
{
  for (unsigned int i = 0; i&lt;bindingList.length(); i++) {
    ACE_CString rName = bindingList[i].binding_name[0].id.in();

    CosNaming::Name name(1);
    name.length(1);
    name[0].id = CORBA::string_dup (rName.c_str());

    CORBA::Object_var obj = this->receiverContext->resolve(name
                                                           ACE_ENV_ARG_PARAMETER);

    AVStreams::MMDevice_var rDevice = AVStreams::MMDevice::_narrow(obj.in());

    this->receivers.bind (rName, rDevice);

    ACE_DEBUG((LM_DEBUG, " (%P|%t) addToReceiversMap(): binding &lt;%s> and device &lt;0x%x>\n", rName.c_str(), &amp;rDevice));
  }
}
    </xsl:with-param>
  </xsl:call-template>



  <xsl:call-template name="method_imp">
    <xsl:with-param name="name">
      <xsl:value-of select="concat($name, '::resolveReceivers')"/></xsl:with-param>
    <xsl:with-param name="type">void</xsl:with-param>
    <xsl:with-param name="params">{CORBA::Environment &amp;ACE_TRY_ENV}</xsl:with-param>
    <xsl:with-param name="content">
{
  CosNaming::BindingIterator_var iterator;
  CosNaming::BindingList_var binding_list;
  const CORBA::ULong chunk = 100;

  //Turn context into a list
  this->receiverContext->list (chunk,
                               binding_list,
                               iterator
                               ACE_ENV_ARG_PARAMETER);
  ACE_CHECK;

  this->addToReceiversMap (binding_list, ACE_TRY_ENV);
  ACE_CHECK;

  if (!CORBA::is_nil (iterator.in ()))
    {
      CORBA::Boolean more = 1;

      while (more)
        {
          more = iterator->next_n (chunk, binding_list
                                   ACE_ENV_ARG_PARAMETER);
          ACE_CHECK;

          this->addToReceiversMap (binding_list, ACE_TRY_ENV);
          ACE_CHECK;
        }
    }
}
    </xsl:with-param>
  </xsl:call-template>





  <!-- post typedef -->
  <xsl:text>
#if defined (ACE_HAS_EXPLICIT_TEMPLATE_INSTANTIATION)

template class ACE_Hash_Map_Entry&lt;ACE_CString, AVStreams::MMDevice_var>;
template class ACE_Hash_Map_Manager&lt;ACE_CString, AVStreams::MMDevice_var, ACE_Null_Mutex>;
template class ACE_Hash_Map_Manager_Ex&lt;ACE_CString, AVStreams::MMDevice_var, ACE_Hash&lt;ACE_CString>, ACE_Equal_To&lt;ACE_CString>, ACE_Null_Mutex>;
template class ACE_Hash_Map_Iterator_Base_Ex&lt;ACE_CString, AVStreams::MMDevice_var, ACE_Hash&lt;ACE_CString>, ACE_Equal_To&lt;ACE_CString>, ACE_Null_Mutex>;
template class ACE_Hash_Map_Iterator_Ex&lt;ACE_CString, AVStreams::MMDevice_var, ACE_Hash&lt;ACE_CString>, ACE_Equal_To&lt;ACE_CString>, ACE_Null_Mutex>;
template class ACE_Hash_Map_Reverse_Iterator_Ex&lt;ACE_CString, AVStreams::MMDevice_var, ACE_Hash&lt;ACE_CString>, ACE_Equal_To&lt;ACE_CString>, ACE_Null_Mutex>;

template class ACE_Hash_Map_Entry&lt;ACE_CString, TAO_AV_Protocol_Object *>;
template class ACE_Hash_Map_Manager&lt;ACE_CString, TAO_AV_Protocol_Object *, ACE_Null_Mutex>;
template class ACE_Hash_Map_Manager_Ex&lt;ACE_CString, TAO_AV_Protocol_Object *, ACE_Hash&lt;ACE_CString>, ACE_Equal_To&lt;ACE_CString>, ACE_Null_Mutex>;
template class ACE_Hash_Map_Iterator_Base_Ex&lt;ACE_CString, TAO_AV_Protocol_Object *, ACE_Hash&lt;ACE_CString>, ACE_Equal_To&lt;ACE_CString>, ACE_Null_Mutex>;
template class ACE_Hash_Map_Iterator_Ex&lt;ACE_CString, TAO_AV_Protocol_Object *, ACE_Hash&lt;ACE_CString>, ACE_Equal_To&lt;ACE_CString>, ACE_Null_Mutex>;
template class ACE_Hash_Map_Reverse_Iterator_Ex&lt;ACE_CString, TAO_AV_Protocol_Object *, ACE_Hash&lt;ACE_CString>, ACE_Equal_To&lt;ACE_CString>, ACE_Null_Mutex>;

template class ACE_Hash_Map_Entry&lt;ACE_CString, AVStreams::StreamCtrl_var>;
template class ACE_Hash_Map_Manager&lt;ACE_CString, AVStreams::StreamCtrl_var, ACE_Null_Mutex>;
template class ACE_Hash_Map_Manager_Ex&lt;ACE_CString, AVStreams::StreamCtrl_var, ACE_Hash&lt;ACE_CString>, ACE_Equal_To&lt;ACE_CString>, ACE_Null_Mutex>;
template class ACE_Hash_Map_Iterator_Base_Ex&lt;ACE_CString, AVStreams::StreamCtrl_var, ACE_Hash&lt;ACE_CString>, ACE_Equal_To&lt;ACE_CString>, ACE_Null_Mutex>;
template class ACE_Hash_Map_Iterator_Ex&lt;ACE_CString, AVStreams::StreamCtrl_var, ACE_Hash&lt;ACE_CString>, ACE_Equal_To&lt;ACE_CString>, ACE_Null_Mutex>;
template class ACE_Hash_Map_Reverse_Iterator_Ex&lt;ACE_CString, AVStreams::StreamCtrl_var, ACE_Hash&lt;ACE_CString>, ACE_Equal_To&lt;ACE_CString>, ACE_Null_Mutex>;


#elif defined (ACE_HAS_TEMPLATE_INSTANTIATION_PRAGMA)

#pragma instantiate ACE_Hash_Map_Entry&lt;ACE_CString, AVStreams::MMDevice_var>
#pragma instantiate ACE_Hash_Map_Manager&lt;ACE_CString, AVStreams::MMDevice_var, ACE_Null_Mutex>
#pragma instantiate ACE_Hash_Map_Manager_Ex&lt;ACE_CString, AVStreams::MMDevice_var, ACE_Hash&lt;ACE_CString>, ACE_Equal_To&lt;ACE_CString>, ACE_Null_Mutex>
#pragma instantiate ACE_Hash_Map_Iterator_Base_Ex&lt;ACE_CString, AVStreams::MMDevice_var, ACE_Hash&lt;ACE_CString>, ACE_Equal_To&lt;ACE_CString>, ACE_Null_Mutex>
#pragma instantiate ACE_Hash_Map_Iterator_Ex&lt;ACE_CString, AVStreams::MMDevice_var, ACE_Hash&lt;ACE_CString>, ACE_Equal_To&lt;ACE_CString>, ACE_Null_Mutex>
#pragma instantiate ACE_Hash_Map_Reverse_Iterator_Ex&lt;ACE_CString, AVStreams::MMDevice_var, ACE_Hash&lt;ACE_CString>, ACE_Equal_To&lt;ACE_CString>, ACE_Null_Mutex>

#pragma instantiate ACE_Hash_Map_Entry&lt;ACE_CString, TAO_AV_Protocol_Object *>
#pragma instantiate ACE_Hash_Map_Manager&lt;ACE_CString, TAO_AV_Protocol_Object *, ACE_Null_Mutex>
#pragma instantiate ACE_Hash_Map_Manager_Ex&lt;ACE_CString, TAO_AV_Protocol_Object *, ACE_Hash&lt;ACE_CString>, ACE_Equal_To&lt;ACE_CString>, ACE_Null_Mutex>
#pragma instantiate ACE_Hash_Map_Iterator_Base_Ex&lt;ACE_CString, TAO_AV_Protocol_Object *, ACE_Hash&lt;ACE_CString>, ACE_Equal_To&lt;ACE_CString>, ACE_Null_Mutex>
#pragma instantiate ACE_Hash_Map_Iterator_Ex&lt;ACE_CString, TAO_AV_Protocol_Object *, ACE_Hash&lt;ACE_CString>, ACE_Equal_To&lt;ACE_CString>, ACE_Null_Mutex>
#pragma instantiate ACE_Hash_Map_Reverse_Iterator_Ex&lt;ACE_CString, TAO_AV_Protocol_Object *, ACE_Hash&lt;ACE_CString>, ACE_Equal_To&lt;ACE_CString>, ACE_Null_Mutex>

#pragma instantiate ACE_Hash_Map_Entry&lt;ACE_CString, AVStreams::StreamCtrl_var>
#pragma instantiate ACE_Hash_Map_Manager&lt;ACE_CString, AVStreams::StreamCtrl_var, ACE_Null_Mutex>
#pragma instantiate ACE_Hash_Map_Manager_Ex&lt;ACE_CString, AVStreams::StreamCtrl_var, ACE_Hash&lt;ACE_CString>, ACE_Equal_To&lt;ACE_CString>, ACE_Null_Mutex>
#pragma instantiate ACE_Hash_Map_Iterator_Base_Ex&lt;ACE_CString, AVStreams::StreamCtrl_var, ACE_Hash&lt;ACE_CString>, ACE_Equal_To&lt;ACE_CString>, ACE_Null_Mutex>
#pragma instantiate ACE_Hash_Map_Iterator_Ex&lt;ACE_CString, AVStreams::StreamCtrl_var, ACE_Hash&lt;ACE_CString>, ACE_Equal_To&lt;ACE_CString>, ACE_Null_Mutex>
#pragma instantiate ACE_Hash_Map_Reverse_Iterator_Ex&lt;ACE_CString, AVStreams::StreamCtrl_var, ACE_Hash&lt;ACE_CString>, ACE_Equal_To&lt;ACE_CString>, ACE_Null_Mutex>


#endif /* ACE_HAS_EXPLICIT_TEMPLATE_INSTANTIATION */
</xsl:text>


   </content></file>
</xsl:template>
<!-- ============================================= -->
<!--  End of The CtrlManager_CPP template   -->
<!-- ============================================= -->




  
<!-- ============================================= -->
<!--      The CtrlManager_H template        -->
<!-- ============================================= -->


<xsl:template name="CtrlManager_H">
  <xsl:param name="name"/>

  <!-- Capitalized name -->
  <xsl:variable name="name_all_capital">
    <xsl:value-of select="translate($name,'abcdefghijklmnopqrstuvwxyz','ABCDEFGHIJKLMNOPQRSTUVWXYZ')"/>
  </xsl:variable>
  <xsl:variable name="name_all_small">
    <xsl:value-of
    select="translate($name,'ABCDEFGHIJKLMNOPQRSTUVWXYZ','abcdefghijklmnopqrstuvwxyz')"/>
  </xsl:variable>

  <xsl:value-of select="$empty_line"/>
  <xsl:value-of select="$empty_line"/>

  <file><name>
     <xsl:value-of select="$dir_output_root"/><xsl:value-of select="$name"/>.h</name><content>

  <xsl:value-of select="$file_header"/>

  <xsl:call-template name="comment">
    <xsl:with-param name="value">
      <xsl:value-of select="concat($name,'.h')"/>
    </xsl:with-param>
  </xsl:call-template>
  <!-- include ifdef-endif -->

  <xsl:value-of select="$empty_line"/>

#ifndef _<xsl:value-of select="$name_all_capital"/>_MANAGER_H_
#define _<xsl:value-of select="$name_all_capital"/>_MANAGER_H_


  <xsl:value-of select="$empty_line"/>

  <!-- The system include -->
  <xsl:call-template name="use">
    <xsl:with-param name="files">{orbsvcs/Naming/Naming_Utils.h}{orbsvcs/AV/AVStreams_i.h}{orbsvcs/AV/Protocol_Factory.h}</xsl:with-param>
  </xsl:call-template>

  <xsl:value-of select="$empty_line"/>


  <xsl:call-template name="comment">
    <xsl:with-param name="value">
      <xsl:value-of select="concat($name,' class: This is a helper class that allows the senders bind to multiple receivers and receivers to bind to senders. It also lets the senders and receivers disconnect streams.')"/>
    </xsl:with-param>
  </xsl:call-template>

  <xsl:value-of select="$empty_line"/>
  <!-- The class declaration -->
  <xsl:call-template name="class">
    <xsl:with-param name="name"><xsl:value-of select="$name"/></xsl:with-param>
  </xsl:call-template>

  <!-- The class method -->
  <xsl:call-template name="method">
    <xsl:with-param name="name"><xsl:value-of select="$name"/></xsl:with-param>
    <xsl:with-param name="visibility">public</xsl:with-param>
    <xsl:with-param name="type">none</xsl:with-param>
    <xsl:with-param name="params">{none}</xsl:with-param>
  </xsl:call-template>

  <xsl:call-template name="method">
    <xsl:with-param name="name"><xsl:value-of select="concat('~',$name)"/></xsl:with-param>
    <xsl:with-param name="visibility">public</xsl:with-param>
    <xsl:with-param name="type">none</xsl:with-param>
    <xsl:with-param name="params">{none}</xsl:with-param>
  </xsl:call-template>

  <xsl:call-template name="method">
    <xsl:with-param name="name">init</xsl:with-param>
    <xsl:with-param name="visibility">public</xsl:with-param>
    <xsl:with-param name="type">int</xsl:with-param>
    <xsl:with-param name="params">{CORBA::ORB_ptr orb}</xsl:with-param>
  </xsl:call-template>

  <xsl:call-template name="method">
    <xsl:with-param name="name">bindToReceivers</xsl:with-param>
    <xsl:with-param name="visibility">public</xsl:with-param>
    <xsl:with-param name="type">void</xsl:with-param>
    <xsl:with-param name="params">{const ACE_CString &amp;senderName}{const
    ACE_CString &amp; receiverName}{AVStreams::MMDevice_ptr sender}{CORBA::Environment &amp;ACE_TRY_ENV = TAO_default_environment ()}</xsl:with-param>
  </xsl:call-template>
  
  <xsl:call-template name="method">
    <xsl:with-param name="name">connectToReceivers</xsl:with-param>
    <xsl:with-param name="visibility">public</xsl:with-param>
    <xsl:with-param name="type">void</xsl:with-param>
    <xsl:with-param name="params">{AVStreams::MMDevice_ptr sender}{AVStreams::streamQoS&amp; the_qos}{int use_qos}{CORBA::Environment &amp;ACE_TRY_ENV = TAO_default_environment ()}</xsl:with-param>
  </xsl:call-template>

  <xsl:call-template name="method">
    <xsl:with-param name="name">bindToSender</xsl:with-param>
    <xsl:with-param name="visibility">public</xsl:with-param>
    <xsl:with-param name="type">void</xsl:with-param>
    <xsl:with-param name="params">{const ACE_CString &amp;senderName}{const ACE_CString &amp;receiverName}{AVStreams::MMDevice_ptr receiver}{CORBA::Environment &amp;ACE_TRY_ENV = TAO_default_environment ()}</xsl:with-param>
  </xsl:call-template>


  <xsl:call-template name="method">
    <xsl:with-param name="name">connectToSender</xsl:with-param>
    <xsl:with-param name="visibility">public</xsl:with-param>
    <xsl:with-param name="type">void</xsl:with-param>
    <xsl:with-param name="params">{AVStreams::streamQoS &amp;the_qos}{int use_qos}{CORBA::Environment&amp; ACE_TRY_ENV = TAO_default_environment ()}</xsl:with-param>
  </xsl:call-template>

  <xsl:call-template name="method">
    <xsl:with-param name="name">destroy</xsl:with-param>
    <xsl:with-param name="visibility">public</xsl:with-param>
    <xsl:with-param name="type">void</xsl:with-param>
    <xsl:with-param name="params">{CORBA::Environment &amp; = TAO_default_environment ()}</xsl:with-param>
  </xsl:call-template>

  <xsl:call-template name="method">
    <xsl:with-param name="name">destroy</xsl:with-param>
    <xsl:with-param name="visibility">public</xsl:with-param>
    <xsl:with-param name="type">void</xsl:with-param>
    <xsl:with-param name="params">{const ACE_CString &amp;flowname}{CORBA::Environment &amp; = TAO_default_environment ()}</xsl:with-param>
  </xsl:call-template>

  <!-- The forward typedef -->
<xsl:text>

  typedef ACE_Hash_Map_Manager&lt;ACE_CString,
                               AVStreams::MMDevice_var,
                               ACE_Null_Mutex>
          Receivers;

  typedef ACE_Hash_Map_Manager&lt;ACE_CString,
                               TAO_AV_Protocol_Object *,
                               ACE_Null_Mutex>
          ProtocolObjects;

  typedef ACE_Hash_Map_Manager&lt;ACE_CString,
                               AVStreams::StreamCtrl_var,
                               ACE_Null_Mutex>
          StreamCtrls;
</xsl:text>

  <xsl:call-template name="method">
    <xsl:with-param name="name">addToStreamCtrlsMap</xsl:with-param>
    <xsl:with-param name="visibility">public</xsl:with-param>
    <xsl:with-param name="type">void</xsl:with-param>
    <xsl:with-param name="params">{const ACE_CString &amp;flowname}{TAO_StreamEndPoint *endpoint}{CORBA::Environment &amp;ACE_TRY_ENV = TAO_default_environment ()}</xsl:with-param>
  </xsl:call-template>


  <xsl:call-template name="method">
    <xsl:with-param name="name">getReceivers</xsl:with-param>
    <xsl:with-param name="visibility">public</xsl:with-param>
    <xsl:with-param name="type">Receivers&amp;</xsl:with-param>
    <xsl:with-param name="params">{none}</xsl:with-param>
  </xsl:call-template>

  <xsl:call-template name="method">
    <xsl:with-param name="name">getProtocolObjects</xsl:with-param>
    <xsl:with-param name="visibility">public</xsl:with-param>
    <xsl:with-param name="type">ProtocolObjects&amp;</xsl:with-param>
    <xsl:with-param name="params">{none}</xsl:with-param>
  </xsl:call-template>
  
  <xsl:call-template name="method">
    <xsl:with-param name="name">getStreamctrls</xsl:with-param>
    <xsl:with-param name="visibility">public</xsl:with-param>
    <xsl:with-param name="type">StreamCtrls&amp;</xsl:with-param>
    <xsl:with-param name="params">{none}</xsl:with-param>
  </xsl:call-template>

  <xsl:call-template name="method">
    <xsl:with-param name="name">resolveReceivers</xsl:with-param>
    <xsl:with-param name="visibility">protected</xsl:with-param>
    <xsl:with-param name="type">void</xsl:with-param>
    <xsl:with-param name="params">{CORBA::Environment &amp;ACE_TRY_ENV}</xsl:with-param>
  </xsl:call-template>


  <xsl:call-template name="method">
    <xsl:with-param name="name">addToReceiversMap</xsl:with-param>
    <xsl:with-param name="visibility">protected</xsl:with-param>
    <xsl:with-param name="type">void</xsl:with-param>
    <xsl:with-param name="params">{CosNaming::BindingList &amp;binding_list}{CORBA::Environment&amp; ACE_TRY_ENV}</xsl:with-param>
  </xsl:call-template>


  <!-- The class member field -->
  <xsl:call-template name="data_member">
    <xsl:with-param name="name">namingClient</xsl:with-param>
    <xsl:with-param name="visibility">protected</xsl:with-param>
    <xsl:with-param name="type">TAO_Naming_Client</xsl:with-param>
  </xsl:call-template>

  <xsl:call-template name="data_member">
    <xsl:with-param name="name">receivers</xsl:with-param>
    <xsl:with-param name="visibility">protected</xsl:with-param>
    <xsl:with-param name="type">Receivers</xsl:with-param>
  </xsl:call-template>

  <xsl:call-template name="data_member">
    <xsl:with-param name="name">protocolObjects</xsl:with-param>
    <xsl:with-param name="visibility">protected</xsl:with-param>
    <xsl:with-param name="type">ProtocolObjects</xsl:with-param>
  </xsl:call-template>

  <xsl:call-template name="data_member">
    <xsl:with-param name="name">streamCtrls</xsl:with-param>
    <xsl:with-param name="visibility">protected</xsl:with-param>
    <xsl:with-param name="type">StreamCtrls</xsl:with-param>
  </xsl:call-template>

  <xsl:call-template name="data_member">
    <xsl:with-param name="name">senderName</xsl:with-param>
    <xsl:with-param name="visibility">protected</xsl:with-param>
    <xsl:with-param name="type">ACE_CString</xsl:with-param>
  </xsl:call-template>

  <xsl:call-template name="data_member">
    <xsl:with-param name="name">sender</xsl:with-param>
    <xsl:with-param name="visibility">protected</xsl:with-param>
    <xsl:with-param name="type">AVStreams::MMDevice_var</xsl:with-param>
  </xsl:call-template>

  <xsl:call-template name="data_member">
    <xsl:with-param name="name">senderContext</xsl:with-param>
    <xsl:with-param name="visibility">protected</xsl:with-param>
    <xsl:with-param name="type">CosNaming::NamingContext_var</xsl:with-param>
  </xsl:call-template>

  <xsl:call-template name="data_member">
    <xsl:with-param name="name">receiverName</xsl:with-param>
    <xsl:with-param name="visibility">protected</xsl:with-param>
    <xsl:with-param name="type">ACE_CString</xsl:with-param>
  </xsl:call-template>

  <xsl:call-template name="data_member">
    <xsl:with-param name="name">receiver</xsl:with-param>
    <xsl:with-param name="visibility">protected</xsl:with-param>
    <xsl:with-param name="type">AVStreams::MMDevice_var</xsl:with-param>
  </xsl:call-template>

  <xsl:call-template name="data_member">
    <xsl:with-param name="name">receiverContext</xsl:with-param>
    <xsl:with-param name="visibility">protected</xsl:with-param>
    <xsl:with-param name="type">CosNaming::NamingContext_var</xsl:with-param>
  </xsl:call-template>

  <xsl:call-template name="data_member">
    <xsl:with-param name="name">_orb</xsl:with-param>
    <xsl:with-param name="visibility">private</xsl:with-param>
    <xsl:with-param name="type">CORBA::ORB_var</xsl:with-param>
  </xsl:call-template>



  <!-- The closing of class -->
  <xsl:text>
};// class: </xsl:text>
<xsl:value-of select="$name"/>

<xsl:text>

</xsl:text>

  <xsl:value-of select="$empty_line"/>

#endif // _<xsl:value-of select="$name_all_capital"/>_MANAGER_H_



  </content></file>
</xsl:template>

</xsl:stylesheet>
