<xsl:stylesheet version="1.0"                                                
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform">

<!-- ================================================================= -->
<!-- $Id: DataFlow.xsl,v 1.1.1.1 2003/06/11 19:11:44 wyan Exp $                                                              -->
<!-- Wenchang                                                          -->
<!-- ================================================================= -->

<!-- ================================= -->
<!--      The DataFlow_CPP template      -->
<!-- ================================= -->

<xsl:template name="DataFlow_CPP">
  <xsl:param name="name"/>

  <!-- Capitalized name -->
  <xsl:variable name="name_all_capital">
    <xsl:value-of select="translate($name,'abcdefghijklmnopqrstuvwxyz','ABCDEFGHIJKLMNOPQRSTUVWXYZ')"/>
  </xsl:variable>
  <xsl:variable name="name_all_small">
    <xsl:value-of
    select="translate($name,'ABCDEFGHIJKLMNOPQRSTUVWXYZ','abcdefghijklmnopqrstuvwxyz')"/>
  </xsl:variable>

  <xsl:value-of select="$empty_line"/>
  <xsl:value-of select="$empty_line"/>

  <file><name>
    <xsl:value-of select="$dir_output_root"/><xsl:value-of select="$name"/>.cpp</name><content>

  <xsl:value-of select="$file_header"/>

  <xsl:call-template name="comment">
    <xsl:with-param name="value">
      <xsl:value-of select="concat($name,'.cpp')"/>
    </xsl:with-param>
  </xsl:call-template>


  <!-- The system include -->
  <xsl:call-template name="use">
    <xsl:with-param name="files">{string.h}{iostream.h}</xsl:with-param>
  </xsl:call-template>

  <xsl:call-template name="dependency">
    <xsl:with-param name="files">
      <xsl:value-of select="concat('{',$name,'.h}')"/></xsl:with-param>
  </xsl:call-template>

  <xsl:value-of select="$empty_line"/>
  <!-- The implementation of related class method -->
  <!-- Cstr -->
  <xsl:call-template name="method_imp">
    <xsl:with-param name="name">
      <xsl:value-of select="concat($name, '::', $name)"/></xsl:with-param>
    <xsl:with-param name="content">
{
  data.buff = NULL;
}
    </xsl:with-param>
  </xsl:call-template>
  <!-- End of Cstr() -->

  <!-- Dstr -->
  <xsl:call-template name="method_imp">
    <xsl:with-param name="name">
      <xsl:value-of select="concat($name, '::~', $name)"/></xsl:with-param>
    <xsl:with-param name="content">
{
  if (data.buff)
    delete [] data.buff;
  data.buff = NULL;
}
    </xsl:with-param>
  </xsl:call-template>
  <!-- Endof Dstr() -->

  <!-- Cpy Cstr -->
  <xsl:call-template name="method_imp">
    <xsl:with-param name="name">
      <xsl:value-of select="concat($name, '::', $name)"/></xsl:with-param>
      <xsl:with-param name="params"><xsl:value-of select="concat('{const ', $name, '&amp; d}')"/></xsl:with-param>
    <xsl:with-param name="content">
{
  infop = d.infop;
  data.SIZE = d.data.SIZE;
  if (!d.data.buff) {
    data.buff = NULL;
  }
  else {
    if (data.buff)
      delete [] data.buff, data.buff = NULL;

    data.buff = new Type[data.SIZE];
    memcpy(data.buff, d.data.buff, data.SIZE);
  }
}
    </xsl:with-param>
  </xsl:call-template>
  <!-- End of Cpy Cstr() -->


  <!-- setINFOP() -->
  <xsl:call-template name="method_imp">
    <xsl:with-param name="name">
      <xsl:value-of select="concat($name, '::', 'setINFOP')"/></xsl:with-param>
    <xsl:with-param name="type">void</xsl:with-param>
      <xsl:with-param name="params"><xsl:value-of select="concat('{const ', 'INFOP', '&amp; infop}')"/></xsl:with-param>
    <xsl:with-param name="content">
{
  this->infop = infop;
}
    </xsl:with-param>
  </xsl:call-template>
  <!-- End of setINFOP() -->

  <!-- setPayload () -->
  <xsl:call-template name="method_imp">
    <xsl:with-param name="name">
      <xsl:value-of select="concat($name, '::', 'setPayload')"/></xsl:with-param>
    <xsl:with-param name="type">void</xsl:with-param>
      <xsl:with-param name="params"><xsl:value-of select="concat('{const ', 'Data', '&amp; data}')"/></xsl:with-param>
    <xsl:with-param name="content">
{
  this->data.SIZE = data.SIZE;
  if (!data.buff)
    this->data.buff = NULL;
  else {
    if (this->data.buff)
      delete [] this->data.buff, this->data.buff = NULL;

    this->data.buff = new Type[data.SIZE];
    memcpy(this->data.buff, data.buff, data.SIZE);
  }
}
    </xsl:with-param>
  </xsl:call-template>
  <!-- End of setPayload() -->


  <!-- getPayload () -->
  <xsl:call-template name="method_imp">
    <xsl:with-param name="name">
      <xsl:value-of select="concat($name, '::', 'getPayload')"/></xsl:with-param>
    <xsl:with-param name="type">void</xsl:with-param>
      <xsl:with-param name="params"><xsl:value-of select="concat('{Data*', ' d}')"/></xsl:with-param>
    <xsl:with-param name="content">
{
  d->SIZE = data.SIZE;

  if (!data.buff)
    d->buff = NULL;
  else {
    if (d->buff) delete [] d->buff;
    d->buff = new Type[data.SIZE];
    memcpy(d->buff, data.buff, data.SIZE);
  }
}
    </xsl:with-param>
  </xsl:call-template>
  <!-- End of getINFOP() -->

  <!-- getHeader () -->
  <xsl:call-template name="method_imp">
    <xsl:with-param name="name">
      <xsl:value-of select="concat($name, '::', 'getHeader')"/></xsl:with-param>
    <xsl:with-param name="type">INFOP&amp;</xsl:with-param>
    <xsl:with-param name="content">
{
  return infop;
}
    </xsl:with-param>
  </xsl:call-template>
  <!-- End of getHeader() -->

  <!-- assignment = () -->
  <xsl:call-template name="method_imp">
    <xsl:with-param name="name">
      <xsl:value-of select="concat($name, '::', 'operator = ')"/></xsl:with-param>
    <xsl:with-param name="type"><xsl:value-of select="concat($name, '&amp;')"/></xsl:with-param>
      <xsl:with-param name="params"><xsl:value-of select="concat('{const ',$name,'&amp;d}')"/></xsl:with-param>
    <xsl:with-param name="content">
{
 if (this!=&amp;d)
    {
      infop = d.infop;
      data.SIZE = d.data.SIZE;
      if (!d.data.buff) {
        data.buff = NULL;
      }
      else {
        if (data.buff)
          delete [] data.buff, data.buff = NULL;

        data.buff = new Type[data.SIZE];
        memcpy(data.buff, d.data.buff, data.SIZE);
      }
    }
  return *this;
}
    </xsl:with-param>
  </xsl:call-template>
  <!-- End of assignment = () -->


  <!-- dumpDataFlow () -->
  <xsl:call-template name="method_imp">
    <xsl:with-param name="name">
      <xsl:value-of select="concat($name, '::', 'dumpDataFlow')"/></xsl:with-param>
    <xsl:with-param name="type">void</xsl:with-param>
    <xsl:with-param name="content">
{
  cout&lt;&lt;" [DataFlow::dumpDataFlow():\n";
  cout&lt;&lt;"     data.SIZE: "&lt;&lt;data.SIZE&lt;&lt;endl;
  if (NULL!=data.buff)
    for (int i = 0; i&lt;data.SIZE; i++) cout&lt;&lt;data.buff[i];

  cout&lt;&lt;endl;
}
    </xsl:with-param>
  </xsl:call-template>
  <!-- End of dumpDataFlow () -->









  <!-- Cstr -->
  <xsl:call-template name="method_imp">
    <xsl:with-param name="name">
      <xsl:value-of select="concat('CtrlFlow', '::', 'CtrlFlow')"/></xsl:with-param>
    <xsl:with-param name="content">
{
  data.buff = NULL;
}
    </xsl:with-param>
  </xsl:call-template>
  <!-- End of Cstr() -->

  <!-- Dstr -->
  <xsl:call-template name="method_imp">
    <xsl:with-param name="name">
      <xsl:value-of select="concat('CtrlFlow', '::~', 'CtrlFlow')"/></xsl:with-param>
    <xsl:with-param name="content">
{
  if (data.buff)
    delete [] data.buff;
  data.buff = NULL;
}
    </xsl:with-param>
  </xsl:call-template>
  <!-- Endof Dstr() -->

  <!-- Cpy Cstr -->
  <xsl:call-template name="method_imp">
    <xsl:with-param name="name">
      <xsl:value-of select="concat('CtrlFlow', '::', 'CtrlFlow')"/></xsl:with-param>
      <xsl:with-param name="params"><xsl:value-of select="concat('{const ', 'CtrlFlow', '&amp; d}')"/></xsl:with-param>
    <xsl:with-param name="content">
{
  infocp = d.infocp;
  data.SIZE = d.data.SIZE;
  if (!d.data.buff) {
    data.buff = NULL;
  }
  else {
    if (data.buff)
      delete [] data.buff, data.buff = NULL;

    data.buff = new Type[data.SIZE];
    memcpy(data.buff, d.data.buff, data.SIZE);
  }
}
    </xsl:with-param>
  </xsl:call-template>
  <!-- End of Cpy Cstr() -->


  <!-- setINFOCP() -->
  <xsl:call-template name="method_imp">
    <xsl:with-param name="name">
      <xsl:value-of select="concat('CtrlFlow', '::', 'setINFOCP')"/></xsl:with-param>
    <xsl:with-param name="type">void</xsl:with-param>
      <xsl:with-param name="params"><xsl:value-of select="concat('{const ', 'INFOCP', '&amp; infocp}')"/></xsl:with-param>
    <xsl:with-param name="content">
{
  this->infocp = infocp;
}
    </xsl:with-param>
  </xsl:call-template>
  <!-- End of setINFOCP() -->

  <!-- setPayload () -->
  <xsl:call-template name="method_imp">
    <xsl:with-param name="name">
      <xsl:value-of select="concat('CtrlFlow', '::', 'setPayload')"/></xsl:with-param>
    <xsl:with-param name="type">void</xsl:with-param>
      <xsl:with-param name="params"><xsl:value-of select="concat('{const ', 'Data', '&amp; data}')"/></xsl:with-param>
    <xsl:with-param name="content">
{
  this->data.SIZE = data.SIZE;
  if (!data.buff)
    this->data.buff = NULL;
  else {
    if (this->data.buff)
      delete [] this->data.buff, this->data.buff = NULL;

    this->data.buff = new Type[data.SIZE];
    memcpy(this->data.buff, data.buff, data.SIZE);
  }
}
    </xsl:with-param>
  </xsl:call-template>
  <!-- End of setPayload() -->


  <!-- getPayload () -->
  <xsl:call-template name="method_imp">
    <xsl:with-param name="name">
      <xsl:value-of select="concat('CtrlFlow', '::', 'getPayload')"/></xsl:with-param>
    <xsl:with-param name="type">void</xsl:with-param>
      <xsl:with-param name="params"><xsl:value-of select="concat('{Data*', ' d}')"/></xsl:with-param>
    <xsl:with-param name="content">
{
  d->SIZE = data.SIZE;

  if (!data.buff)
    d->buff = NULL;
  else {
    if (d->buff) delete [] d->buff;
    d->buff = new Type[data.SIZE];
    memcpy(d->buff, data.buff, data.SIZE);
  }
}
    </xsl:with-param>
  </xsl:call-template>
  <!-- End of getINFOP() -->

  <!-- getHeader () -->
  <xsl:call-template name="method_imp">
    <xsl:with-param name="name">
      <xsl:value-of select="concat('CtrlFlow', '::', 'getHeader')"/></xsl:with-param>
    <xsl:with-param name="type">INFOCP&amp;</xsl:with-param>
    <xsl:with-param name="content">
{
  return infocp;
}
    </xsl:with-param>
  </xsl:call-template>
  <!-- End of getHeader() -->


  <!-- assignment = () -->
  <xsl:call-template name="method_imp">
    <xsl:with-param name="name">
      <xsl:value-of select="concat('CtrlFlow', '::', 'operator = ')"/></xsl:with-param>
    <xsl:with-param name="type"><xsl:value-of select="concat('CtrlFlow', '&amp;')"/></xsl:with-param>
      <xsl:with-param name="params"><xsl:value-of select="concat('{const ','CtrlFlow','&amp;d}')"/></xsl:with-param>
    <xsl:with-param name="content">
{
 if (this!=&amp;d)
    {
      infocp = d.infocp;
      data.SIZE = d.data.SIZE;
      if (!d.data.buff) {
        data.buff = NULL;
      }
      else {
        if (data.buff)
          delete [] data.buff, data.buff = NULL;

        data.buff = new Type[data.SIZE];
        memcpy(data.buff, d.data.buff, data.SIZE);
      }
    }
  return *this;
}
    </xsl:with-param>
  </xsl:call-template>
  <!-- End of assignment = () -->


  <!-- dumpCtrlFlow () -->
  <xsl:call-template name="method_imp">
    <xsl:with-param name="name">
      <xsl:value-of select="concat('CtrlFlow', '::', 'dumpCtrlFlow')"/></xsl:with-param>
    <xsl:with-param name="type">void</xsl:with-param>
    <xsl:with-param name="content">
{
  cout&lt;&lt;" [DataFlow::dumpDataFlow():\n";
  cout&lt;&lt;"     data.SIZE: "&lt;&lt;data.SIZE&lt;&lt;endl;
  if (NULL!=data.buff)
    for (int i = 0; i&lt;data.SIZE; i++) cout&lt;&lt;data.buff[i];

  cout&lt;&lt;endl;
}
    </xsl:with-param>
  </xsl:call-template>
  <!-- End of dumpDataFlow () -->


   </content></file>
</xsl:template>
<!-- ================================= -->
<!--  End of The DataFlow_CPP template   -->
<!-- ================================= -->


  
<!-- ================================= -->
<!--      The DataFlow_H template        -->
<!-- ================================= -->


<xsl:template name="DataFlow_H">
  <xsl:param name="name"/>
  <xsl:param name="datatypes"/>

  <!-- Capitalized name -->
  <xsl:variable name="name_all_capital">
    <xsl:value-of select="translate($name,'abcdefghijklmnopqrstuvwxyz','ABCDEFGHIJKLMNOPQRSTUVWXYZ')"/>
  </xsl:variable>
  <xsl:variable name="name_all_small">
    <xsl:value-of
    select="translate($name,'ABCDEFGHIJKLMNOPQRSTUVWXYZ','abcdefghijklmnopqrstuvwxyz')"/>
  </xsl:variable>

  <xsl:value-of select="$empty_line"/>
  <xsl:value-of select="$empty_line"/>

  <file><name>
     <xsl:value-of select="$dir_output_root"/><xsl:value-of select="$name"/>.h</name><content>

  <xsl:value-of select="$file_header"/>

  <xsl:call-template name="comment">
    <xsl:with-param name="value">
      <xsl:value-of select="concat($name,'.h')"/>
    </xsl:with-param>
  </xsl:call-template>

  <!-- include ifdef-endif -->

#ifndef _<xsl:value-of select="$name_all_capital"/>_H_
#define _<xsl:value-of select="$name_all_capital"/>_H_

#include &quot;INFOP.h&quot;
#include &quot;typedef.h&quot;
  <xsl:value-of select="$empty_line"/>

  <xsl:call-template name="data_struct">
    <xsl:with-param name="name">
      <xsl:value-of select="$datatypes"/>
    </xsl:with-param>
  </xsl:call-template>

  <!-- DataFlow class -->
  <!-- =============================== -->
  <!-- The DataFlow  class -->
  <!--                                 -->

  <xsl:call-template name="comment">
    <xsl:with-param name="value">
      <xsl:value-of select="concat('class', $name)"/>
    </xsl:with-param>
  </xsl:call-template>

  <xsl:value-of select="$empty_line"/>
  <!-- The class declaration -->

  <xsl:call-template name="class">
    <xsl:with-param name="name"><xsl:value-of select="$name"/></xsl:with-param>
  </xsl:call-template>

  <!-- The class method -->

  <xsl:call-template name="method">
    <xsl:with-param name="name"><xsl:value-of select="$name"/></xsl:with-param>
    <xsl:with-param name="visibility">public</xsl:with-param>
  </xsl:call-template>

  <xsl:call-template name="method">
    <xsl:with-param name="name"><xsl:value-of select="concat('~',$name)"/></xsl:with-param>
    <xsl:with-param name="visibility">public</xsl:with-param>
  </xsl:call-template>

  <xsl:call-template name="method">
    <xsl:with-param name="name"><xsl:value-of select="$name"/></xsl:with-param>
    <xsl:with-param name="visibility">public</xsl:with-param>
    <xsl:with-param name="params"><xsl:value-of select="concat('{const ', $name, '&amp;}')"/></xsl:with-param>
  </xsl:call-template>

  <xsl:call-template name="method">
    <xsl:with-param name="name">setINFOP</xsl:with-param>
    <xsl:with-param name="visibility">public</xsl:with-param>
    <xsl:with-param name="type">void</xsl:with-param>
    <xsl:with-param name="params">{const INFOP&amp; infop}</xsl:with-param>
  </xsl:call-template>

  <xsl:call-template name="method">
    <xsl:with-param name="name">setPayload</xsl:with-param>
    <xsl:with-param name="visibility">public</xsl:with-param>
    <xsl:with-param name="type">void</xsl:with-param>
    <xsl:with-param name="params">{const Data&amp; data}</xsl:with-param>
  </xsl:call-template>

  <xsl:call-template name="method">
    <xsl:with-param name="name">getPayload</xsl:with-param>
    <xsl:with-param name="visibility">public</xsl:with-param>
    <xsl:with-param name="type">void</xsl:with-param>
    <xsl:with-param name="params">{Data* d}</xsl:with-param>
  </xsl:call-template>

  <xsl:call-template name="method">
    <xsl:with-param name="name">getHeader</xsl:with-param>
    <xsl:with-param name="visibility">public</xsl:with-param>
    <xsl:with-param name="type">INFOP&amp;</xsl:with-param>
  </xsl:call-template>

  <xsl:call-template name="method">
    <xsl:with-param name="name">operator = </xsl:with-param>
    <xsl:with-param name="visibility">public</xsl:with-param>
    <xsl:with-param name="type"><xsl:value-of select="concat($name,'&amp;')"/></xsl:with-param>
    <xsl:with-param name="params"><xsl:value-of select="concat('{const ', $name, '&amp; d}')"/></xsl:with-param>
  </xsl:call-template>

  <xsl:call-template name="method">
    <xsl:with-param name="name">dumpDataFlow</xsl:with-param>
    <xsl:with-param name="visibility">public</xsl:with-param>
    <xsl:with-param name="type">void</xsl:with-param>
  </xsl:call-template>

  <!-- The class member field -->
  <xsl:call-template name="data_member">
    <xsl:with-param name="name">infop</xsl:with-param>
    <xsl:with-param name="visibility">public</xsl:with-param>
    <xsl:with-param name="type">INFOP</xsl:with-param>
  </xsl:call-template>

  <xsl:call-template name="data_member">
    <xsl:with-param name="name">data</xsl:with-param>
    <xsl:with-param name="visibility">public</xsl:with-param>
    <xsl:with-param name="type">Data</xsl:with-param>
  </xsl:call-template>


  <!-- The closing of class -->

  <xsl:text>
};// class: </xsl:text>
<xsl:value-of select="$name"/>


  <!-- CtrlFlow class -->
  <!-- =============================== -->
  <!-- The  CtrlFlow  class -->
  <!--                                 -->

  <xsl:call-template name="comment">
    <xsl:with-param name="value">
      <xsl:value-of select="concat('class', ' CtrlFlow')"/>
    </xsl:with-param>
  </xsl:call-template>

  <xsl:value-of select="$empty_line"/>
  <!-- The class declaration -->

  <xsl:call-template name="class">
    <xsl:with-param name="name">CtrlFlow</xsl:with-param>
  </xsl:call-template>

  <!-- The class method -->

  <xsl:call-template name="method">
    <xsl:with-param name="name">CtrlFlow</xsl:with-param>
    <xsl:with-param name="visibility">public</xsl:with-param>
  </xsl:call-template>

  <xsl:call-template name="method">
    <xsl:with-param name="name">~CtrlFlow</xsl:with-param>
    <xsl:with-param name="visibility">public</xsl:with-param>
  </xsl:call-template>

  <xsl:call-template name="method">
    <xsl:with-param name="name">CtrlFlow</xsl:with-param>
    <xsl:with-param name="visibility">public</xsl:with-param>
    <xsl:with-param name="params"><xsl:value-of select="concat('{const ', 'CtrlFlow', '&amp;}')"/></xsl:with-param>
  </xsl:call-template>

  <xsl:call-template name="method">
    <xsl:with-param name="name">setINFOCP</xsl:with-param>
    <xsl:with-param name="visibility">public</xsl:with-param>
    <xsl:with-param name="type">void</xsl:with-param>
    <xsl:with-param name="params">{const INFOCP&amp; infocp}</xsl:with-param>
  </xsl:call-template>

  <xsl:call-template name="method">
    <xsl:with-param name="name">setPayload</xsl:with-param>
    <xsl:with-param name="visibility">public</xsl:with-param>
    <xsl:with-param name="type">void</xsl:with-param>
    <xsl:with-param name="params">{const Data&amp; data}</xsl:with-param>
  </xsl:call-template>

  <xsl:call-template name="method">
    <xsl:with-param name="name">getPayload</xsl:with-param>
    <xsl:with-param name="visibility">public</xsl:with-param>
    <xsl:with-param name="type">void</xsl:with-param>
    <xsl:with-param name="params">{Data* d}</xsl:with-param>
  </xsl:call-template>

  <xsl:call-template name="method">
    <xsl:with-param name="name">getHeader</xsl:with-param>
    <xsl:with-param name="visibility">public</xsl:with-param>
    <xsl:with-param name="type">INFOCP&amp;</xsl:with-param>
  </xsl:call-template>

  <xsl:call-template name="method">
    <xsl:with-param name="name">operator = </xsl:with-param>
    <xsl:with-param name="visibility">public</xsl:with-param>
    <xsl:with-param name="type"><xsl:value-of select="concat('CtrlFlow','&amp;')"/></xsl:with-param>
    <xsl:with-param name="params"><xsl:value-of select="concat('{const ', 'CtrlFlow', '&amp; d}')"/></xsl:with-param>
  </xsl:call-template>

  <xsl:call-template name="method">
    <xsl:with-param name="name">dumpCtrlFlow</xsl:with-param>
    <xsl:with-param name="visibility">public</xsl:with-param>
    <xsl:with-param name="type">void</xsl:with-param>
  </xsl:call-template>

  <!-- The class member field -->
  <xsl:call-template name="data_member">
    <xsl:with-param name="name">infocp</xsl:with-param>
    <xsl:with-param name="visibility">public</xsl:with-param>
    <xsl:with-param name="type">INFOCP</xsl:with-param>
  </xsl:call-template>

  <xsl:call-template name="data_member">
    <xsl:with-param name="name">data</xsl:with-param>
    <xsl:with-param name="visibility">public</xsl:with-param>
    <xsl:with-param name="type">Data</xsl:with-param>
  </xsl:call-template>


  <!-- The closing of class -->

  <xsl:text>
};// class: </xsl:text>
<xsl:value-of select="$name"/>
<xsl:text>


</xsl:text>


#endif // _<xsl:value-of select="$name_all_capital"/>_H_



  </content></file>

</xsl:template>











</xsl:stylesheet>
