<xsl:stylesheet version="1.0"                                                
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform">

<!-- ================================================================= -->
<!-- $Id: INFOP.xsl,v 1.1.1.1 2003/06/11 19:11:44 wyan Exp $                                                              -->
<!-- Wenchang                                                          -->
<!-- ================================================================= -->

<!-- ================================= -->
<!--      The INFOP_CPP template    -->
<!-- ================================= -->

<xsl:template name="INFOP_CPP">
  <xsl:param name="name"/>

  <!-- Capitalized name -->
  <xsl:variable name="name_all_capital">
    <xsl:value-of select="translate($name,'abcdefghijklmnopqrstuvwxyz','ABCDEFGHIJKLMNOPQRSTUVWXYZ')"/>
  </xsl:variable>
  <xsl:variable name="name_all_small">
    <xsl:value-of
    select="translate($name,'ABCDEFGHIJKLMNOPQRSTUVWXYZ','abcdefghijklmnopqrstuvwxyz')"/>
  </xsl:variable>

  <xsl:value-of select="$empty_line"/>
  <xsl:value-of select="$empty_line"/>

  <file><name>
    <xsl:value-of select="$dir_output_root"/><xsl:value-of select="$name"/>.cpp</name><content>

  <xsl:value-of select="$file_header"/>

  <xsl:call-template name="comment">
    <xsl:with-param name="value">
      <xsl:value-of select="concat($name,'.cpp')"/>
    </xsl:with-param>
  </xsl:call-template>


  <!-- The system include -->


  <xsl:call-template name="dependency">
    <xsl:with-param name="files">
      <xsl:value-of select="concat('{',$name,'.h}')"/></xsl:with-param>
  </xsl:call-template>

  <xsl:value-of select="$empty_line"/>

//Some consts about the transporting packages. Global.
//Modify MTU according to the abstract machine.
int MTU = 15360;
int AV_UDP_MTU = MTU;
int UDP_IP_HEADER_LEN = 42;

//information flows in the infopipe.
//INFOP:
//
//
//  0 1 2 3 4    8          16                     31
// +---+-+-+----+----------+-------------------------+
// |Ver|P|X|Mark| Type     |     Seq Number          |
// +---+-+-+----+----------+-------------------------+
// |             Time                                |
// +-------------------------------------------------+
// |             Length                              |
// +-------------------------------------------------+
// |             Source Identifier                   |
// +-------------------------------------------------+
//
//
// Ver: version number
// P: padding. The last octet of the padding is the value of the count of the
//    size of the padding.
// X: extension, reserved.
// Mark: additional fields besides `Type' describes the payload type.
//       {
//         if (Type==0x40 &amp;&amp; Mark==0xF)
//            Indicates a new PPM frame starts;
//         else if (Type==0x40 &amp;&amp; Mark==0x0)
//            Indicates a fragment of a PPM frame;
//         ...
//       }
// Type: payload type.
//       {
//         ...
//         0x40: PPM frame,   0x50 - 0x5F: 16 img compression format.
//                                         0x50: JPEG ...
//         0x60 - 0x6F: 16 formats related to multimedia format like
//                      MPEG and audio format.
//         ...
//       }
//
// Seq Number: assigns the order to the packets.
// Time: the timestamp value of the first octet of the payload data is sampled.
// Length: the length of the packet, including the header.
// Source Identifier: the source id (random?)
//
//
// INFOCP
//
//  0 1 2 3      8          16                     31
// +---+-+------+----------+-------------------------+
// |Ver|P|RecCt | Type     |     Length              |
// +---+-+------+----------+-------------------------+
// | Meta data starts here.                          |
// | In XML format.                                  |
// +-------------------------------------------------+
//
// Ver: version number
// P: padding.
// RecCt: the report count
// Type: the packet type, i.e. the report type.
//       {
//        ...
//        0x10: Sender report,     0x17: Receiver report
//                                       reports b/w, latency, jitter,
//                                       and packet loss, etc.
//        0x20: Control msg indicates adaptation request.
//              e.g. &lt;Adaptation>
//                     &lt;SecurityLevel>Low&lt;/SecurityLevel>
//                     &lt;ImgQuality>High&lt;/ImgQuality>
//                   &lt;/Adaptation>
//        0xFF: Bye-bye msg.
//       }
// Length: the length of the packet, including the header.
// Meta data: meta data in XML.
//

int INFOP_HEADER_SIZE = 16;
int INFOCP_HEADER_SIZE = 4;
int INFOP_MAX_SIZE = MTU - UDP_IP_HEADER_LEN;
int INFOCP_MAX_SIZE = MTU - UDP_IP_HEADER_LEN;
int INFOP_MAX_PAYLOAD_SIZE = INFOP_MAX_SIZE - INFOP_HEADER_SIZE;
int INFOCP_MAX_PAYLOAD_SIZE = INFOCP_MAX_SIZE - INFOCP_HEADER_SIZE  ;


#ifndef INFOP_STRUCT

struct INFOP_t {
  unsigned short ver: 2;   //The version
  unsigned short pad: 1;   //Padding flag
  unsigned short extension: 1;//Extension flag
  unsigned short marker: 4;//Marker flag
  unsigned short type: 8;  //Payload type
  unsigned short seq: 16; //Sequence number
  unsigned int time;  //Time stamp
  unsigned int length;//The length of the whole packet
  unsigned int src_id;//Source id
  //char data[1];       //Starting point of the payload data
};


struct INFOCP_t {
  unsigned short ver: 2;     //The version
  unsigned short pad: 1;     //The padding
  unsigned short rec_ct: 5;  //The report count
  unsigned short type: 8;    //The report type
  unsigned short length: 16; //The length of the packet
  //char data[1];     //The start of the meta data
};
#endif


   </content></file>
</xsl:template>
<!-- ================================= -->
<!--  End of The INFOP_CPP template   -->
<!-- ================================= -->


  
<!-- ================================= -->
<!--      The INFOP_H template        -->
<!-- ================================= -->


<xsl:template name="INFOP_H">
  <xsl:param name="name"/>

  <!-- Capitalized name -->
  <xsl:variable name="name_all_capital">
    <xsl:value-of select="translate($name,'abcdefghijklmnopqrstuvwxyz','ABCDEFGHIJKLMNOPQRSTUVWXYZ')"/>
  </xsl:variable>
  <xsl:variable name="name_all_small">
    <xsl:value-of
    select="translate($name,'ABCDEFGHIJKLMNOPQRSTUVWXYZ','abcdefghijklmnopqrstuvwxyz')"/>
  </xsl:variable>

  <xsl:value-of select="$empty_line"/>
  <xsl:value-of select="$empty_line"/>

  <file><name>
     <xsl:value-of select="$dir_output_root"/><xsl:value-of select="$name"/>.h</name><content>

  <xsl:value-of select="$file_header"/>

  <xsl:call-template name="comment">
    <xsl:with-param name="value">
      <xsl:value-of select="concat($name,'.h')"/>
    </xsl:with-param>
  </xsl:call-template>

  <!-- include ifdef-endif -->

#ifndef _<xsl:value-of select="$name_all_capital"/>_H_
#define _<xsl:value-of select="$name_all_capital"/>_H_

  <xsl:value-of select="$empty_line"/>

extern int MTU;
extern int AV_UDP_MTU;
extern int UDP_IP_HEADER_LEN;

extern int INFOP_HEADER_SIZE;
extern int INFOCP_HEADER_SIZE;
extern int INFOP_MAX_SIZE;
extern int INFOCP_MAX_SIZE;
extern int INFOP_MAX_PAYLOAD_SIZE;
extern int INFOCP_MAX_PAYLOAD_SIZE;


typedef struct INFOP_t  INFOP;
typedef struct INFOCP_t INFOCP;

//The struct INFOP
struct INFOP_t {
  unsigned short ver: 2;   //The version
  unsigned short pad: 1;   //Padding flag
  unsigned short extension: 1;//Extension flag
  unsigned short marker: 4;//Marker flag
  unsigned short type: 8;  //Payload type
  unsigned short seq: 16; //Sequence number
  unsigned int time;  //Time stamp
  unsigned int length;//The length of the whole packet
  unsigned int src_id;//Source id
  //  char data[1];  //Starting point of the payload data
};


//The struct INFOCP
struct INFOCP_t {
  unsigned short ver: 2;     //The version
  unsigned short pad: 1;     //The padding
  unsigned short rec_ct: 5;  //The report count
  unsigned short type: 8;    //The report type
  unsigned short length: 16; //The length of the packet
  //char data[1];     //The start of the meta data
};

#define INFOP_STRUCT


#define SET_INFOP(fieldname, s, value) ((s).##fieldname = (value))
#define SET_INFOCP(fieldname, s, value) ((s).##fieldname = (value))



<xsl:text>


</xsl:text>


#endif // _<xsl:value-of select="$name_all_capital"/>_H_



  </content></file>

</xsl:template>











</xsl:stylesheet>
