<xsl:stylesheet version="1.0"                                                
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform">

<!-- ================================================================= -->
<!-- $Id: common.xsl,v 1.2 2003/06/25 19:42:27 wyan Exp $                                                              -->
<!-- Wenchang                                                          -->
<!-- ================================================================= -->


<!-- ================ -->
<!-- The global stuff -->
<!-- ================ -->

<!-- the str delimeter used in debugging -->
<xsl:variable name="dlmt1">
$
</xsl:variable>

<!-- the empty line -->
<xsl:variable name="empty_line"><xsl:text>
</xsl:text></xsl:variable>

<!-- The header for all the generated files -->
<xsl:variable name="file_header">
<xsl:text>
  /* 
   * This file is generated by the Infopipe Stub Generator (ISG).
   * @ GaTech. 
   * Modify where necessarily.
   */
</xsl:text>
</xsl:variable>
<xsl:variable name="makefile_header">
<xsl:text>
############################################################## 
# This file is generated by the Infopipe Stub Generator (ISG).
# @ GaTech.
# Modify where necessarily.
##############################################################
</xsl:text>
</xsl:variable>

<!-- the indentation of member var/method visibility -->
<xsl:variable name="vis_idt"></xsl:variable>
<!-- the indentation of member var/method type -->
<xsl:variable name="type_idt"><xsl:value-of select="$vis_idt"/><xsl:text>  </xsl:text></xsl:variable>

<!-- The subdirectories -->
<xsl:variable name="dir_output_root">../../../../demos/UAV2/AV_gen/</xsl:variable>
<xsl:variable name="dir_imgFilters"><xsl:value-of select="$dir_output_root"/>imgFilters/</xsl:variable>
<xsl:variable name="dir_frondEnd"><xsl:value-of select="$dir_output_root"/>frontEnd/</xsl:variable>
<xsl:variable name="dir_idl"><xsl:value-of select="$dir_output_root"/>idl/</xsl:variable>





<!-- ========================================== -->
<!-- The type mapping in <datatypes> section of -->
<!-- SIP/XIP.                                   -->
<!-- ========================================== -->

<xsl:variable name="MAPPING_string">
  <map><xsl:value-of select="concat('Type','*')"/></map>
</xsl:variable>

<xsl:variable name="MAPPING_integer">
  <map><xsl:value-of select="concat('','int')"/></map>
</xsl:variable>


<xsl:template name="data_struct">
<xsl:param name="name"/>

  <xsl:if test="starts-with($name, '{')">
    <xsl:variable name="type">
      <xsl:value-of select="substring-before(substring-after($name,'{'), '}')"/>
    </xsl:variable>
    //&lt;block name="datatype_<xsl:value-of select="$type"/>">
    typedef struct <xsl:value-of select="$type"/>_t <xsl:value-of select="$type"/>;
    <xsl:value-of select="$empty_line"/>
    struct <xsl:value-of select="$type"/>_t {
    <xsl:for-each select="/xip/datatypes/datatype[@name=$type]/arg">
      <xsl:variable name="field">
        <xsl:value-of select="attribute::name"/>
      </xsl:variable>

      <xsl:variable name="pseudo_type">
        <xsl:value-of select="attribute::type"/>
      </xsl:variable>
   
        <xsl:if test="$pseudo_type = 'string'">
          <xsl:value-of select="$MAPPING_string"/>
        </xsl:if> 
        <xsl:if test="$pseudo_type = 'integer'">
          <xsl:value-of select="$MAPPING_integer"/>
        </xsl:if> <xsl:value-of select="concat(' ',$field)"/>;
    </xsl:for-each>};
    //&lt;/block>
    <xsl:if test="contains(substring-after($name,'}'), '{')">
      <xsl:text>
</xsl:text>
      <xsl:call-template name="data_struct">
        <xsl:with-param name="name" select="substring-after($name,'}')" />
      </xsl:call-template>
    </xsl:if>
  </xsl:if>
</xsl:template>



<!-- ================================ -->
<!-- include header files depends on  -->
<!-- different used fitlers           -->
<xsl:template name="filters_header_include">
<xsl:param name="used_filters"/>

extern "C" {
  <xsl:if test="contains($used_filters, 'ENCRYPT')">
    //&lt;block name="filter_ENCRYPT">
#include "encrypt.h"
extern EncryptParam encryptParam;
extern int KEYSIZE;
    //&lt;/block>
  </xsl:if>
  <xsl:if test="contains($used_filters, 'JPEG')">
    //&lt;block name="filter_JPEG">
#include "jpgcodec.h"
    //&lt;/block>
  </xsl:if>
  <xsl:if test="contains($used_filters, 'LZO')">
    //&lt;block name="filter_LZO">
int lzo_filter_init();
int lzo_compress(int options, char *in, int in_len, char *out, int *out_len);
int lzo_decompress(char *in, int in_len, char *out, int *out_len);
    //&lt;/block>
  </xsl:if>
  <xsl:if test="contains($used_filters, 'GZIP')">
    //&lt;block name="filter_GZIP">
#include "gzip.h"
    //&lt;/block>
  </xsl:if>
  <xsl:if test="contains($used_filters, 'SHRINK')">
    //&lt;block name="filter_SHRINK">
#include "imgFilters/filters.h"
    //&lt;/block>
  </xsl:if>
  <xsl:if test="contains($used_filters, 'GREY')">
    //&lt;block name="filter_GREY">
#include "imgFilters/filters.h"
    //&lt;/block>
  </xsl:if>
}    
</xsl:template>


<!-- ================================ -->
<!-- filters init  depends on         -->
<!-- different used fitlers           -->
<xsl:template name="filters_necessary_init">
<xsl:param name="used_filters"/>
  <xsl:if test="contains($used_filters, 'ENCRYPT')">
    //&lt;block name="filter_ENCRYPT">
  if (1) {
    encryptParam.keySize = KEYSIZE/8;
    encryptParam.ivSize = encryptParam.keySize;
    if (NULL == (encryptParam.key = (char *) malloc(encryptParam.keySize * sizeof(char))) ||
        NULL == (encryptParam.iv = (char *) malloc(encryptParam.ivSize * sizeof(char))))
      cout&lt;&lt;" Encryption key no mem\n", exit(-1);

    char algo[30];
    sprintf(algo, "%s-%d", "rijndael", encryptParam.keySize*8); //KEYSIZE in bytes.
    strncpy(encryptParam.key, "abcdefghijklmnopqrstuvwxyz123456", encryptParam.keySize);
    strncpy(encryptParam.iv, "abcdefghijklmnopqrstuvwxyz123456", encryptParam.keySize);

    if ( MCRYPT_FAILED == (encryptParam.cryptContext = CreateContext(algo, "cbc")))
      cout&lt;&lt;" Encryption error 1\n", exit(-1);
    if (0 > SetParams((void *) &amp;(encryptParam.cryptContext), encryptParam.key,
encryptParam.iv))
      cout&lt;&lt;" Encryption error 2\n", exit(-1);
  }
    //&lt;/block>
  </xsl:if>

  <xsl:if test="contains($used_filters, 'JPEG')">
    //&lt;block name="filter_JPEG">
  if (1) {
    int ret = init_jpeg_compress();
    ret = init_jpeg_decompress();
    if (ret &lt; 0)
      cout&lt;&lt;" Jpeg init error\n", exit(-1);
  }
    //&lt;/block>
  </xsl:if>

  <xsl:if test="contains($used_filters, 'LZO')">
    //&lt;block name="filter_LZO">
  if (1) 
    lzo_filter_init();
    //&lt;/block>
  </xsl:if>

</xsl:template>



<!-- ========================================== -->
<!-- The tags for pre/post processing using ISG -->
<!-- ========================================== -->
<!--
<xsl:template name="isg_open">
  <xsl:param name="value"/>
  <xsl:value-of select="$value"/>
</xsl:template>

<xsl:template name="output_file_open_tag">
<xsl:param name="value"/>
<xsl:value-of select="$value"/>
</xsl:template>

<xsl:template name="output_file_close_tag">
<xsl:param name="value"/>

</xsl:template>
-->



</xsl:stylesheet>
