
//////////////////////////////////////////////////////////////////////////////////////////
// Node001.cpp -- the implementation file for this node
/////////////////////////////////////////////////////////////////////////////
#include "../allNodes.hpp"
#include <sys/types.h>
#include <sys/stat.h>
#include <fstream.h>
#include <errno.h>

// definition of publish types
#define PUBLISH_FILE  0x0001

static  CManager cm;
static  EControlContext ec;






typedef struct Raw_data_2CCarrierStructDef
{
  
  Raw_data_2CStruct payload;
} Raw_data_2CCarrierStruct, *Raw_data_2CCarrierStruct_ptr;




static IOField Raw_data_2CFieldList[] = {
  {"tag", "integer",sizeof(int), IOOffset( Raw_data_2CStruct*, tag)},
  {"ppm1", "char",sizeof(char), IOOffset( Raw_data_2CStruct*, ppm1)},
  {"ppm2", "char",sizeof(char), IOOffset( Raw_data_2CStruct*, ppm2)},
  {"size", "integer",sizeof(int), IOOffset( Raw_data_2CStruct*, size)},
  {"width", "integer",sizeof(int), IOOffset( Raw_data_2CStruct*, width)},
  {"height", "integer",sizeof(int), IOOffset( Raw_data_2CStruct*, height)},
  {"maxval", "integer",sizeof(int), IOOffset( Raw_data_2CStruct*, maxval)},
  {"buff", "",sizeof(), IOOffset( Raw_data_2CStruct*, buff)},
  
  
  {NULL, NULL, 0, 0}
};

static CMFormatRec Raw_data_2CFormatList[] = 
{

  {"Raw_data_2CStruct",Raw_data_2CFieldList},
  { NULL, NULL}
};

static IOField Raw_data_2CCarrierFieldList[] = {
  
  {"payload","Raw_data_2CStruct", sizeof(Raw_data_2CStruct), IOOffset( Raw_data_2CCarrierStruct*, payload) },
  {NULL, NULL, 0, 0}
};


// variables to track



// channel vars for receive
static ECSinkHandle Raw_data2C_node_Node001ReceiveHandle;
static EChannel Raw_data2C_node_Node001ReceiveChannel;


// prototypes for the functions in the implementation file
void infopipeStartup( );
void infopipeShutdown( );

// functions publishing and looking up
static int publish(char *pipe, char *connType, char *connName, char *str, int type);
static int publishFile(char *pipe, char *connType, char *connName, char *str);
static int unpublish(char *pipe, char *connType, char *connName, int type);
static int unpublishFile(char *pipe, char *connType, char *connName);
static char *lookup(char *pipe, char *nodeOrBranch, char *connPoint, int type, char *str);
static char *lookupFile(char *pipe, char *connType, char *connName, char *str);



void Raw_data2C_node_Node001Handler( void *, void *, attr_list);






 



int makePathDir(char *path, mode_t mode = 0755)
{
  char *dir; 
  char *p;

  dir = new char[strlen(path) + 1];

  p = path;

  while(1)
  {
    p = strchr(p, '/');
    if (p == NULL) 
    {
      delete dir;
      return 0;
    }

    p++;
    // get first dir name including '/'
    memcpy(dir, path, p - path);
    dir[p - path] = 0;

    // make directory
    if (mkdir(dir, mode))
    {
      // if dir exists already, continue
      // otherwise(error occurs), return -1
      if (errno != EEXIST)
      {
        delete dir;
        return -1;
      }
    }
  }

  // cannot reach here
  return 0;
}

// publish
static int publish(char *pipe, char *connType, char *connName, char *str, int type)
{
  if (type & PUBLISH_FILE)
  {
    publishFile(pipe, connType, connName, str);
  }

  return 0;
}

static int publishFile(char *pipe, char *connType, char *connName, char *str)
{
  char *publishRoot;
  char *filename;

  if ((publishRoot = getenv("INFOPIPE_PUBLISH_ROOT")) == NULL)
  {
    // if no INFOPIPE_PUBLISH_ROOT is defined, use current directory
    publishRoot = getenv("PWD");
  }

  filename = new char[strlen(publishRoot) + strlen(pipe) + 
		      strlen(connType) + strlen(connName) + 10];

  // make filename = ""
  filename[0] = 0;
  strcat(filename, publishRoot);

  // if the path does not have the last '/', add '/'
  if (filename[strlen(publishRoot)-1] != '/')
    strcat(filename, "/");

  strcat(filename, pipe);
  strcat(filename, "/");
  strcat(filename, connType);
  strcat(filename, "/");
  strcat(filename, connName);

  makePathDir(filename);

  cout << "publishing file = " << filename << endl;

  ofstream ofile(filename);
  if (ofile.is_open())
  {
    ofile << str << endl;
  }
  else
    cout << "Error in publishing file " << filename << endl;

  return 0;
}


static int unpublish(char *pipe, char *connType, char *connName, int type)
{
  if (type & PUBLISH_FILE)
  {
    unpublishFile(pipe, connType, connName);
  }

  return 0;
}

static int unpublishFile(char *pipe, char *connType, char *connName)
{
  char *publishRoot;
  char *filename;

  if ((publishRoot = getenv("INFOPIPE_PUBLISH_ROOT")) == NULL)
  {
    // if no INFOPIPE_PUBLISH_ROOT is defined, use current directory
    publishRoot = getenv("PWD");
  }

  filename = new char[strlen(publishRoot) + strlen(pipe) + 
		      strlen(connType) + strlen(connName) + 10];

  // make filename = ""
  filename[0] = 0;
  strcat(filename, publishRoot);

  // if the path does not have the last '/', add '/'
  if (filename[strlen(publishRoot)-1] != '/')
    strcat(filename, "/");

  strcat(filename, pipe);
  strcat(filename, "/");
  strcat(filename, connType);
  strcat(filename, "/");
  strcat(filename, connName);

  return remove(filename);
}


static char *lookup(char *pipe, char *connType, char *connName, int type, char *str)
{
  if (type & PUBLISH_FILE)
  {
    return lookupFile(pipe, connType, connName, str);
  }

}


static char *lookupFile(char *pipe, char *connType, char *connName, char *str)
{
  char *publishRoot;
  char *filename;

  if ((publishRoot = getenv("INFOPIPE_PUBLISH_ROOT")) == NULL)
  {
    // if no INFOPIPE_PUBLISH_ROOT is defined, use current directory
    publishRoot = getenv("PWD");
  }

  filename = new char[strlen(publishRoot) + strlen(pipe) + 
		      strlen(connType) + strlen(connName) + 10];

  // make filename = ""
  filename[0] = 0;
  strcat(filename, publishRoot);

  // if the path does not have the last '/', add '/'
  if (filename[strlen(publishRoot)-1] != '/')
    strcat(filename, "/");

  strcat(filename, pipe);
  strcat(filename, "/");
  strcat(filename, connType);
  strcat(filename, "/");
  strcat(filename, connName);

  ifstream ifile(filename);
  if (ifile.is_open())
  {
    ifile >> str;
  }
  else
    cout << "Error in looking up file " << filename << endl;

  return str;
}


void infopipeStartup(  )
{
  char lookupStr[512];
  int forked;

  gen_pthread_init();

  // create the connection manager
  cm = CManager_create();

  // fork network manager thread
  forked = CMfork_comm_thread( cm );
  if( !forked )
    {
    cerr << "Fork of communications thread failed, exiting" << endl;
    exit( 1 );
    }

  // initialize the CM
  ec = ECho_CM_init( cm );






  /////////////////////////////////
  // INCOMING LINK -- open channel
  Raw_data2C_node_Node001ReceiveChannel = EChannel_open( ec, lookup("Raw_data2C","node","Node001", PUBLISH_FILE, lookupStr) );

  cout << "lookup string = " << lookupStr << endl;
  // subscribe as a sink on all incoming channels
  ECsink_typed_subscribe( Raw_data2C_node_Node001ReceiveChannel, Raw_data_2CCarrierFieldList, NULL, Raw_data2C_node_Node001Handler, NULL);



  // install signal handler
}

// shuts down the infopipe system and cleans out directory entries
void infopipeShutdown( )
{

// shutdown our outgoing pipes - the incoming pipes will
// be shutdown by the nodes upstream from us



  // closing down comm
  CManager_close(cm);


  // clean up threads?
}


/////////////////////////////////////////////////////////////////////////////
// handler function for an incoming pipe 
// This handler function accepts data coming from the pipe. It runs
// once for each set of data that comes into this machine from
// the pipe/channel.
void Raw_data2C_node_Node001Handler( void *event, void *client_data, attr_list as)
{
  // Typecast incoming data to workable type
  Raw_data_2CCarrierStruct *incomingData = (Raw_data_2CCarrierStruct *)event;

  
  
  // USER: Add code here to operate on incoming data
  // memory pointed to by incomingData (and event) may be reclaimed by calling 

}



// ENDING OF IMP FILE FOR A NODE

int main( void )
{
  // start the pipe
  infopipeStartup();
  
  while( 1 )
    sleep( 1 );

  // close down the pipes
  infopipeShutdown();

  // normal termination
  return 0;
}
