//
//$Id: display.h,v 1.1.1.1 2003/06/23 22:30:58 wyan Exp $
//
//File: display.h
//      Display the raw binary image
//

#ifndef _DISPLAY_H
#define _DISPLAY_H


#include "libvideogfx/graphics/basic/image.hh"
#include "libvideogfx/graphics/lowlevel/img2raw.hh"
#include "libvideogfx/x11/imgwin.hh"
#include "libvideogfx/types.hh" //For uint32 ... ToLittleEndian macro ...


extern "C" {
  int showPPM_wrapper(char* ppmFrame, int width, int height, int bytesPerPixel,
	      unsigned long PPMFrameNum,
	      char magic1, char magic2);
}


class RawImageData
{
public:
  RawImageData();
  virtual ~RawImageData() {};

  void Create(int width,int height);
  void CreateFromExistingMemory(Pixel* data,int width,int height);

  const Pixel*const* AskFrame_const() const;
  const Pixel* operator[](int y) const {return AskFrame_const()[y]; }

  Bitmap<Pixel> image;//Use AskWidth(), AskInternalWidth()... to 
                      //get the info from image

  //Info passed from source via struct PPM_frame_info
  int width, height; //frame width and height. 
  int  m_width,m_height; // Total size of frame picture. Same as width/height?
  int tag;//the frame # -- Not used 
  char ppm1; char ppm2; //'P''6'/'5'
  int size;//byte size of the image = width*height*bytes_per_pixel
  char* buff;//the pointer to byte array of the image -- Not used

  void FreePictureData()
  {
    if (!image.IsEmpty()) 
      image.Destroy();
  }
};


class VideoSink_X11
{
 public:
  VideoSink_X11();
  ~VideoSink_X11();

  int showPPM(char* ppmFrame, int width, int height, int bytesPerPixel,
	      uint32 PPMFrameNum,
	      char magic1, char magic2);
  void setImage(char* ppmFrame, int width, int height, int bytesPerPixel);
  void TransformPixmap(const Bitmap<Pixel>& dimg,
                 uint8* mem, int firstline, int lastline,
		 int width, RawImageSpec_RGB& spec);  
  void TransformGreymap(const Bitmap<Pixel>& dimg,
                 uint8* mem, int firstline, int lastline,
		 int width, RawImageSpec_RGB& spec); 

 private:
  class ImageWindow_X11* imgwin;
  class DisplayImage_X11* x11img;
  class RawImageData* decimg;

  int curr_img_width, curr_img_height;
  int curr_PPM_frame_num;
  int rpos, gpos, bpos;
  
  RawImageSpec_RGB imgspec;//a struct under basic/lowlevel
};



#endif
