/*
 * PPMP.cpp
 */

#include "PPMP.h"
extern "C" {
#include <stdlib.h>
#include <stdio.h>
#include <fcntl.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <sys/timeb.h>
#include <stdarg.h>
#include <time.h>
}

extern "C" int tprintf(const char* fmt,...);

#include <iostream.h>

#undef DEBUG_WHERE
#undef DEBUG_WHAT
#define DEBUG_WHERE 1
#define DEBUG_WHAT 1



PPMP_Fragment::PPMP_Fragment(char *frag_buf, size_t frag_len)
  : frag_buffer(frag_buf), frag_length(frag_len)
{
}

PPMP_Fragment::PPMP_Fragment(unsigned long ppmp_header,
			     unsigned long frame_no ,
			     const char *msg_buf, size_t len,
			     char *frag_buf)
  : frag_buffer(frag_buf),
    frag_length(len + PPMP_FRAME_INFO_OFFSET)
{
  memcpy(frag_buffer + PPMP_PICT_HEADER_OFFSET, &ppmp_header, 
	 sizeof(unsigned long));
  memcpy(frag_buffer + PPMP_FRAME_NO_OFFSET, &frame_no,
	 sizeof(unsigned long));

  memcpy(frag_buffer + PPMP_FRAME_INFO_OFFSET, msg_buf, frag_length - 
	 PPMP_FRAME_INFO_OFFSET);
}

//PPMP_Sender::PPMP_Sender()
//{
//}

PPMP_Sender::PPMP_Sender(int mus, int pixelsize, int maxpixelperline
			 /*,
			   PPM_frame_info& ppmfi*/)
  /* :pfi(ppmfi)*/
{
  //default value
  bpp = 3;
  max_buf_size = (mus + PPMP_FRAME_INFO_OFFSET + sizeof(unsigned long)
		  + sizeof(PPM_frame_info));
  lines_send_per_time = (int)(((double)mus)/(maxpixelperline) + .9999);
  max_buf_size = max_buf_size < lines_send_per_time*maxpixelperline ?
                 lines_send_per_time*maxpixelperline + PPMP_FRAME_INFO_OFFSET +
                 sizeof(unsigned long) + sizeof(PPM_frame_info) 
                 :
                 max_buf_size;

  num_times =(int)(((double)480/(double)lines_send_per_time)+.9999);

  /*
  bpp = 8*pfi.size/pfi.height/pfi.width;

  max_buf_size = (mus + PPMP_FRAME_INFO_OFFSET + sizeof(unsigned long)
		  + sizeof(PPM_frame_info));

  //lines_send_per_time = (int)(((double)mus)/((double)pfi.width*pixelsize) 
  //		      + .9999); //.01% = impossible
  lines_send_per_time = (int)(((double)mus)/((double)pfi.size/pfi.height)
			      + .9999);


  //Be careful if size/height modulo mus is not zero
  max_buf_size = max_buf_size < lines_send_per_time*pfi.width*3 ?
                 lines_send_per_time*pfi.width*3 + PPMP_FRAME_INFO_OFFSET +
                 sizeof(unsigned long) + sizeof(PPM_frame_info) 
                 :
                 max_buf_size;

  num_times =(int)(((double)pfi.height/(double)lines_send_per_time)+.9999);
  */
}


 
int PPMP_Sender::send_frame(char *msg_buf, size_t len, 
			    unsigned long PPMFrame_num)
{
  curr_frame_num = PPMFrame_num;
  int bytes_send_per_time = lines_send_per_time * pfi.width * bpp/8;

  if (!(fragment_buffer = (char*)malloc(max_buf_size*sizeof(char)))) {
    cerr<<" [PPMP_Sender::send_frame(): mem alloc error"<<endl;
    return (-1);
  }

  //send PPM frame info first
  //0x1ff is distinguished from Mpeg header in Dvdview
  //Just use it indicating it's PPM image.
  //buf[0]=buf[1]=0;
  //buf[2]=1; buf[3]=0xff;
  unsigned long tmp = PPMP_FRAME_HEADER;
  memcpy(fragment_buffer, &tmp, sizeof(unsigned long));

  //fill in frame #
  memcpy(&fragment_buffer[4],&curr_frame_num,sizeof(unsigned long));
  //send pre-frame info
  int sendresult=send_preframe(fragment_buffer);
  usleep(200);

  //send PPM pixel data
  int bytes_read = 0;
  int actualsend = 0;
  sendresult=0;

  for (int time=0; time<num_times; time++) {
    //Now the PPM pixel data
    int tmp = pfi.size - bytes_read;
    int stream_length = (bytes_send_per_time > tmp? tmp: bytes_send_per_time);

    PPMP_Fragment frag(PPMP_PICT_HEADER, curr_frame_num,
		       msg_buf + bytes_read, 
		       stream_length, fragment_buffer);

    bytes_read += stream_length;
    sendresult = this->send_fragment(fragment_buffer, frag.get_frag_length());
    actualsend += sendresult;
    usleep(400);
  }

  if (DEBUG_WHAT)
    tprintf(" [PPMP_Sender::send_frame(): %d bytes actually sent, bpp=%d, lines_send_per_time=%d, num_times=%d\n",actualsend,bpp,lines_send_per_time,num_times);

  if (fragment_buffer) free(fragment_buffer);

  return actualsend;
}

int PPMP_Sender::send_preframe(char *PPMHeader)
{
  char* buf;
  int sendresult;
  int PPMHeader_length = PPMP_FRAME_INFO_OFFSET + sizeof(PPM_frame_info);
 
  if (!(buf = (char*)malloc(PPMHeader_length*sizeof(char)))) {
    cerr<<" [PPMP_Sender::send_preframe(): 0ut of mem"<<endl;
    return (-1);
  }
 
  memcpy(buf,PPMHeader,PPMP_FRAME_INFO_OFFSET);
  memcpy(buf+PPMP_FRAME_INFO_OFFSET, &pfi, sizeof(PPM_frame_info));
 
  sendresult = this->send_fragment(buf, PPMHeader_length); 
 
  if (sendresult<0) {
    cerr<<" [PPMP_Sender::send_preframe(): cannot send preframe info"<<endl;
    return (-1);
  }

  if (buf) free(buf);
  return sendresult;
}

////////////////////////////////////////////////////////////////////

Flag_Array::Flag_Array(int first, int last) 
  :start(first),end(last),current(0)
{
  available = (bool*)malloc((end-start+1)*sizeof(bool));
  if (DEBUG_WHERE)
    tprintf(" [Flag_Array::Cstr(int, int)\n");

  //available = new bool[end-start+1];
  if (!available)
    cerr<<" [Flag_Array::Cstr(): mem alloc error"<<endl;
  
}


Flag_Array::Flag_Array()
  :start(0),end(0),current(0),available(NULL)
{
}
/*  
Flag_Array::Flag_Array(const Flag_Array& fa)
{
  start = fa.start;
  end = fa.end;
  current = fa.current;
  available = fa.available;
}
*/

void Flag_Array::init_available() {
  for (int i=start; i<=end; i++) {
    available[i] = false;
  }
}

///////////////////////////////////////////////////////////////////

PPMP_Receiver::PPMP_Receiver(int mus)
  :MUS(mus),first(true),BytesReceived(0)
{
  if (DEBUG_WHERE)
    tprintf(" [PPMP_Receiver::Cstr()\n");
  /* For debug */
  fileidx =0;
  img_buffer = NULL;
}

/*
PPMP_Receiver::~PPMP_Receiver()
{
  if (DEBUG_WHERE)
    tprintf(" [PPMP_Receiver::Dstr()\n");

}
*/

int PPMP_Receiver::process_ppm(const char* msg, size_t len,
  unsigned long PPMFrameNum, PPM_frame_info& pfi)
{
  if (first)
    {
      if (DEBUG_WHERE) tprintf(" [PPMP_Receiver(): if (first)\n");

      curr_PPM_frame_num = PPMFrameNum;
 
      //P1/P4 : bitmap
      //P2/P5 : greymap
      //P3/P6 : pixmap
      //Update later for X11 display
      bits_per_pixel = (pfi.ppm2 == '6'? 24:8);
      bytes_per_line = pfi.width * bits_per_pixel / 8;
      little_endian = true; //hack

      frame_height = pfi.height; //480;
      frame_width = pfi.width;//640
      img_start_line = 0;

      /*
      if (DEBUG_WHAT) {
	char filename[20];
	char header[20];

	sprintf(filename,"received-%d.ppm",fileidx%2);
	if ((PPMfile[fileidx] = open(filename,O_CREAT|O_WRONLY,S_IRWXU|S_IRGRP|S_IROTH)) < 0 ) {
	  if (DEBUG_WHERE) tprintf(" file open error\n");
	  exit(-1);
	}

	sprintf(header, "%c%c\n%d %d\n%d\n", pfi.ppm1, pfi.ppm2,  
		pfi.width, pfi.height, 255);  
	write(PPMfile[fileidx%2], header, sizeof(header));
      }
      */

      //# of lines sent each time must be conformed to the sender side
      start_defragment(msg, 0, frame_height-1, 0,
                            (int)((double)MUS/(double)bytes_per_line+.9999)-1,
                            img_start_line);


      first=false;
    }

  bool image_flush = false;

  if (PPMFrameNum != curr_PPM_frame_num) {
    //frame number conflicts. Flush the current accumulated image
    image_flush = true;
    first = true;
    //return false;//Be careful
  }

  char* tmp = accumulate_fragment(msg, len, pfi, image_flush);

  //img_start_line += (int)((double)MUS/(double)bytes_per_line+.9999);

  if (!tmp)
    return 1;//Not displayed

  //Display the image
  first = true;

  free_img_buffer();

  //store the curr img info in case the ImageWindow needs to be recontrusted.
  // Init value = 0 for both. 
  //curr_img_width = pfi.width;
  //curr_img_height = pfi.height;

  if (image_flush)
    return 2;//incomplete but displayed
  else
    return 3;//complete img displayed
}

void PPMP_Receiver::free_img_buffer()
{
  if (img_buffer) {
    free(img_buffer);
    img_buffer = NULL;
  }
}

void PPMP_Receiver::start_defragment(const char* img, int first,int last,
				     int d_start_line,int d_end_line,
				     int i_start_line)
{
  if (DEBUG_WHERE)
    tprintf(" [PPMP_Receiver::start_defragment(): data_start_line=%d, data_end_line=%d, img_start_line=%d\n",d_start_line,d_end_line,i_start_line);


  //init the flag arrary
  line = Flag_Array(first,last);
  line.init_available();

  line_accumulated = 0;

  data_start_line = d_start_line;
  data_end_line = d_end_line;
  img_start_line = i_start_line;


  /* For debugging*/
  BytesReceived = 0;
}


char* PPMP_Receiver::accumulate_fragment(const char *frag_buf, size_t frag_len,
				      PPM_frame_info& pfi, bool flushOrNot)
{
  //UDFP_Fragment frag(frag_buf, frag_len);

  const int first = line.get_start_idx();
  const int last  = pfi.height-1 > line.get_end_idx()?
                              line.get_end_idx() : pfi.height-1;

  const int ys = img_start_line;               // first output line
  const int ye = ys + data_end_line - data_start_line;

  if (DEBUG_WHERE) tprintf(" [PPMP_Receiver::accumulate_fragment(DecodedImageData* dimg)\n");
  if (DEBUG_WHERE) tprintf("       [first=%d, last=%d, ys=%d, ye=%d, m_src_y_end=%d, m_src_y_start=%d\n", first, last, ys, ye, data_end_line, data_start_line);

  //int bytesPerPixel = pfi.size/pfi.width/pfi.height;

  if (NULL == img_buffer) {
    if (DEBUG_WHERE) tprintf(" [PPMP_Receiver::accumulate_fragment(): img_buffer is EMPTY\n");
    int tmp = pfi.size > frame_height*frame_width*bits_per_pixel/8?
      pfi.size : frame_height*frame_width*bits_per_pixel/8;

    img_buffer = (char*)malloc(tmp*sizeof(char));

    if (!img_buffer) {
      cerr<<" [PPMP_Receiver::accumulate_fragment(): mem alloc error"<<endl;
      exit(-1);
    }
  }

  //If frame # conflicts, flush the curr frame
  if (flushOrNot) {
    return img_buffer;
  }

  //May change each time, depends on MUS and img size ...
  int line_to_accumulate = data_end_line+1 > pfi.height-line_accumulated ?
    pfi.height-line_accumulated : data_end_line+1;
  line_to_accumulate = line_to_accumulate > 0 ? line_to_accumulate:0;

  for (int y=0; y<line_to_accumulate;y++)
    {
      memcpy((void*)(img_buffer+ (y+ys)*bytes_per_line), frag_buf +
	     PPMP_FRAME_INFO_OFFSET + y*bytes_per_line, 
	     bytes_per_line);
      line.set_available(y+ys);
      line.increase_idx(1);
      
      line_accumulated ++;
  
      // if (DEBUG_WHERE) tprintf(" [PPMP_Receiver::accumulate_fragment(): line[%d] true\n",y+ys);
      /*For debugging*/
      /**/
      //write(PPMfile[fileidx%2],frag_buf+PPMP_FRAME_INFO_OFFSET+
      //    y*bytes_per_line,bytes_per_line);
      /**/
      BytesReceived += bytes_per_line;
    }

  img_start_line += line_to_accumulate;

  /*For debugging*/
  /*
    for (int row = 0; row <= line_to_accumulate; ++row) {
      unsigned char* pP = (unsigned char*)dimg->AskFrame_const()[row];
      
      for (int col = 0; col < 640; ++col ) {
        unsigned char val1 = (unsigned char)*pP++;
        unsigned char val2 = (unsigned char)*pP++;
        unsigned char val3 = (unsigned char)*pP;

        char pixelline[3];
        sprintf(pixelline,"%c%c%c",val1,val3,val2);
        write(PPMfile,pixelline,3);
        BytesReceived += 3;
      }
    } 
  */

  if (DEBUG_WHERE) tprintf(" [PPMP_Receiver::accumulate_fragment(): line_to_accumulate=%d, BytesReceived=%d, line_accumulated=%d\n", line_to_accumulate,BytesReceived,line_accumulated);

  if (pfi.size <= BytesReceived) {
    if (DEBUG_WHERE) tprintf(" [PPMP_Receiver::accumulate_fragment(): BytesReceived=%d\n",BytesReceived);
    /*
      close (PPMfile[fileidx%2]);
      fileidx ++;
      //      if (fileidx <38 )
      // remove(PPMfilename[fileidx%2]);//remove file
      */
    BytesReceived = 0;
  }

  if (false == line.all_available()) 
    return NULL;

  if (DEBUG_WHERE) {
    tprintf(" [PPMP_Receiver::accumulate_fragment(): A whole frame accumulated -------------------------\n");
  }

  if (DEBUG_WHAT) {
    process_whole_frame(img_buffer,pfi.size);
  }
  /*
  if (DEBUG_WHAT) {
    write(PPMfile[fileidx%2],img_buffer,pfi.size);
    close(PPMfile[fileidx%2]);
    fileidx++;
  }
  */
  return img_buffer;
}



 
