
#include "PPMP.h"
#include "ppmudpreader.h"

extern "C" {
#include "connect.h"
}

//extern "C"   int tprintf(const char* fmt, ...);

PPM_UDP_Reader::PPM_UDP_Reader(int mus, int pixelsize, int maxpixelperline,
			       /*PPM_frame_info& ppmfi,*/ const char* host,
		     const char* receiver_host, int receiver_port)
  :PPMP_Sender(mus,pixelsize,maxpixelperline/*,ppmfi*/)
{
  sprintf(hostname,"%s",host);
  sprintf(receiver_hostname,"%s",receiver_host);

  portno = receiver_port;

  h = gethostbyname(receiver_hostname);
  if(h==NULL) {
    printf(" [PPM_UDP_Reader::Cstr(): unknown host '%s' \n", receiver_host);
    herror(" gethostbyname Error:\n");
    exit(1);
  }

  receiver.sin_family = h->h_addrtype;
  memcpy((char *) &receiver.sin_addr.s_addr, 
         h->h_addr_list[0], h->h_length);
  receiver.sin_port = htons(portno);

  sd = connect_udp(hostname,0);
  if (sd<0) {
    fprintf(stderr, "[PPM_UDP_Reader::Cstr(): can't create socket\n");
    exit(-1);
  }
}

PPM_UDP_Reader::~PPM_UDP_Reader() {
  close(sd);
}


void PPM_UDP_Reader::setPPMFrameInfo (int mus, PPM_frame_info& ppmfi)
{
  pfi = ppmfi;

  bpp = 8*pfi.size/pfi.height/pfi.width;

  max_buf_size = (mus + PPMP_FRAME_INFO_OFFSET + sizeof(unsigned long)
		  + sizeof(PPM_frame_info));

  //lines_send_per_time = (int)(((double)mus)/((double)pfi.width*pixelsize) 
  //		      + .9999); //.01% = impossible
  lines_send_per_time = (int)(((double)mus)/((double)pfi.size/pfi.height)
			      + .9999);


  //Be careful if size/height modulo mus is not zero
  max_buf_size = max_buf_size < lines_send_per_time*pfi.width*3 ?
                 lines_send_per_time*pfi.width*3 + PPMP_FRAME_INFO_OFFSET +
                 sizeof(unsigned long) + sizeof(PPM_frame_info) 
                 :
                 max_buf_size;

  num_times =(int)(((double)pfi.height/(double)lines_send_per_time)+.9999);
}


int PPM_UDP_Reader::send_fragment(const char *frag_buf, size_t len) 
{
  int sendresult = sendto(sd, frag_buf, len, 0,
			(struct sockaddr*)&receiver,sizeof(receiver));

  if (sendresult<0) {
    printf(" [PPM_UDP_Reader::send_fragment(): cannot send \n");
    perror(" PERROR: \n");
    return 0;
  }

  return sendresult;
}
