/* -*-C-*-
   @author: Yan, Wenchang
*/

extern "C" {
#include <stdlib.h>
#include <stdio.h>
#include <time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>      
#include <getopt.h>

#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <string.h>
#include <signal.h>
/*for tprintf */
#include <sys/timeb.h>
#include <stdarg.h>

}

extern "C" int tprintf(const char* fmt, ...);


int tprintf(const char* fmt,...)
{
  va_list args;
  struct tm *tstruct;
  struct timeb tb;/* for fine granulity of millisec */
 
  time_t tsec;
 
  tsec = time(NULL);
  tstruct = localtime(&tsec);
 
  ftime(&tb);
 
  printf("[%02d:%02d.%d %5d] ",tstruct->tm_min,
         tstruct->tm_sec,
         tb.millitm,
         getpid());
  va_start(args, fmt);
  return vprintf(fmt, args);
}

#include "connect.h"
#include "PPMP.h"
#include "ppmudpreader.h"

#undef DEBUG_WHERE
#undef DEBUG_WHAT
#define DEBUG_WHERE 1
#define DEBUG_WHAT 1



int verbose = 1;
int ctrlced = 0;/*^C*/

/* Command line arguments */
static struct option longopts[] =
{
  {"verbose", no_argument, NULL, 'v'},
  {"help", no_argument, NULL, 'h'},
  {"nframes", required_argument, NULL, 'n'},
  {"format", required_argument, NULL, 'f'},
  {"portno",required_argument,NULL,'p'},
  {NULL, no_argument, NULL, 0}
};

/*
typedef unsigned long long uint64;
typedef   signed long long  int64;
typedef unsigned long      uint32;
typedef   signed long       int32;
typedef unsigned short     uint16;
typedef   signed short      int16;
typedef unsigned char      uint8;
typedef   signed char       int8;

inline uint16 ChangeEndian16(uint16 v) { return (v>>8) | (v<<8); }
inline uint32 ChangeEndian32(uint32 v) { return (v>>24) | ((v>>8)&0xFF00) | ((v&0xFF00)<<8) | (v<<
24); }


#ifdef WORDS_BIGENDIAN
inline uint16 ToLittleEndian16(uint16 v) { return ChangeEndian16(v); }
inline uint32 ToLittleEndian32(uint32 v) { return ChangeEndian32(v); }
inline uint16 ToBigEndian16(uint16 v)    { return v; }
inline uint32 ToBigEndian32(uint32 v)    { return v; }
#else
inline uint16 ToLittleEndian16(uint16 v) { return v; }
inline uint32 ToLittleEndian32(uint32 v) { return v; }
inline uint16 ToBigEndian16(uint16 v)    { return ChangeEndian16(v); }
inline uint32 ToBigEndian32(uint32 v)    { return ChangeEndian32(v); }
#endif

*/

/* 
 * PPM stuff
 *
 */

typedef unsigned char pixval;


#define RECEIVER_HOST "127.0.0.1" //"localhost"
#define SENDER_HOST "127.0.0.1" //"localhost"
#define RECEIVER_PORT 9999
#define IMAGE_SIZE  921600 /* 3*640*480 */
/*
#define MUS 57600 
*//* bytes in each UDP packet sent */
#define MUS 15360
#define MAXPIXELSIZE 3
#define MAXLINESIZE 640

#define PPM_OVERALLMAXVAL 65535
#define PPM_MAXMAXVAL 255

#define PPM_MAGIC1 'P'
#define PPM_MAGIC2 '3'
#define RPPM_MAGIC2 '6'
#define PPM_FORMAT (PPM_MAGIC1 * 256 + PPM_MAGIC2)
#define RPPM_FORMAT (PPM_MAGIC1 * 256 + RPPM_MAGIC2)
#define PPM_TYPE PPM_FORMAT

#define PPM_ASSIGN(p,red,grn,blu) \
   do { (p).r = (red); (p).g = (grn); (p).b = (blu); } while ( 0 )

/*
 * usage: Prints out 'usage' string
 *
 * Parameters: target 'stream' and 'status' to quit with
 *
 * Returns: -
 *
 */
 
void
usage (FILE* stream, int status)
{
  fprintf(stream, "Usage: %s [-vh] -n num-frames [-M MTU] [-P maxpixel] [-L maxlinesize] [-f format] [-H receiver host] [-p portno]\n"
	          "\t -v --verbose            \t set verbose mode\n"
	          "\t -h --help               \t give this help\n"
	          "\t -n --nframes num-frames \t number of images. Please use 40\n"
                  "\t -M --MTU value          \t the approx. package size\n"
                  "\t -P --Pixel maxpixel     \t 3 is default\n"
                  "\t -L --Line maxlinesize   \t 640 pixels is default\n"
	          "\t -f --format  format     \t RGB format (dflt 2, RGB)\n" 
         	  "\t -H --Host               \t the receiver host name\n"
	          "\t -p --port port_no       \t receiver port number\n",
	  "reader");
  exit(status);
} /* end usage */

void ctrlc(int signal)
{
    static char text[] = "^C - one moment please\n";
    fprintf(stdout,text,strlen(text));
    ctrlced=1;
}

int
main (int argc, char* argv[])
{
  char snapshotFile[20];/* 640*480 color file 1-30
			   320*240 color file 31-40*/
  char duplicateFile[20];

  int shotfd,duplicate; 
  char header[20];//header[20] for 640x480 sample, header[15] for 352x240
  char pixelline[3];
  int rawsize;
  struct tm *now;
  time_t nowt;
  char* image;
  char val1, val2, val3;
  int i, j, idx, row, col;
  int numFrames = 0;

  int optc;

  int mus=MUS;
  int maxpixelsize=MAXPIXELSIZE;
  int maxlinesize=MAXLINESIZE;

  int sendresult,readresult;
  char receiver_host[40];
  int portno = RECEIVER_PORT;
  char* buf;
  int format = 2;
  int header_idx;
  int actual_size;/*actual size of the packet sent each time*/

  int ii;
  //uint32 icopy;
  /* For PPM frame information*/
  PPM_frame_info pfi;
  int line_size,num_lines, pixel_size, lines_send_per_time, num_times;
  int switch_file=60;/* now have only 30 640x480 color PPM files,
			10 640x480 grey PPM files,
			10 352x240 color PPM files,
		        10 352x240 grey PPM files*/
  int partial_name;
  int DEBUG;

  sprintf(receiver_host,"%s",RECEIVER_HOST);

  while ((optc = getopt_long(argc, argv, "+vhn:M:P:L:f:H:p:", 
			     longopts, (int*) 0)) != EOF) {
    switch (optc) {
    case 'v':
      verbose = 1;
      break;
    case 'h':
      usage(stdout, 0);
      break;
    case 'n':
      numFrames = atoi(optarg);
      if (numFrames < 0) usage (stderr, 1);
      break; 
    case 'M':
      mus = atoi(optarg);
      break;
    case 'P':
      maxpixelsize = atoi(optarg);
      break;
    case 'L':
      maxlinesize = atoi(optarg);
      break;

    case 'f':
      format = atoi(optarg);
      if (format < 1) usage(stderr, 1);
      break;
    case 'H':
      sprintf(receiver_host,"%s",optarg);
      break;
    case 'p':
      portno = atoi(optarg);
      if (portno<0) usage(stderr, 1);
      break;
    default:
      usage(stderr, 1);
    }
  }

  if (numFrames < 1) usage(stderr, 1);

  /* no extra args */
  if (optind > argc) usage(stderr, 1);

  //Try to negotiate an appropriate buffer size first.
  //Here is the math:
  // lines_send_per_time = ceil {MUS/(line_size*pixel_size)}
  // num_times = num_lines / lines_send_per_time
  // Thus in order to find a buffer large enough, the data block size 
  // will be:
  //     pixel_size*line_size* lines_send_per_time
  //     <= pixel_size*line_size * {MUS/(line_size*pixel_size) +1}
  //     <= MUS + pixel_size*line_size
  //     <= MUS + max(pixel_size)* max(line_size)
  //
  // e.g <= MUS + 3*640 = MUS + 1920. 

  /*
  if (!(buf = (char*)malloc((MUS+MAXPIXELSIZE*MAXLINESIZE
			       + PPMP_FRAME_INFO_OFFSET+4
			     sizeof(PPM_frame_info))*sizeof(char))))
  */
  if (!(buf = (char*)malloc(IMAGE_SIZE*sizeof(char))))
  {
    printf(" [imagereader::main(): 0ut of mem\n");
    exit(-1);
  }

  signal(SIGINT,ctrlc);

  DEBUG=1;//1 == forever


  PPM_UDP_Reader ppmudpreader(mus, maxpixelsize, maxlinesize,
			      SENDER_HOST,RECEIVER_HOST,portno);

  ///////////////////////////////////////////////////
  do {
    for (i = 0; i < numFrames && !ctrlced; i++) {
      //sleep(2);
      time(&nowt);
      now = localtime(&nowt);
      
      /*choose between 2 kinds of PPM files*/
      partial_name = i % switch_file ;

      if (partial_name <30 && partial_name >5) {
	printf(" partial_name: %d, continue\n", partial_name);
        continue;
      }
	
      if (0<=partial_name && partial_name<30) {//640x480 color
	line_size = 640;
	num_lines = 480;
	pixel_size = 3;/* 3 bytes */
	sprintf(snapshotFile,"/usr/local/mware/quo_5_2/UAV/uav/dvdview/src/My/sample%d.ppm",partial_name+1);
      }
      else if (30<=partial_name && partial_name<40) {//352x240 color
	line_size = 352;
	num_lines = 240;
	pixel_size = 3;
	sprintf(snapshotFile,"/usr/local/mware/quo_5_2/UAV/uav/dvdview/src/My/img0000%d.ppm",partial_name-30);
      }
      /*
      else if (40<=partial_name && partial_name<50) {//640x480 grey
	line_size = 640;
	num_lines = 480;
	pixel_size = 1;
	sprintf(snapshotFile,"../My/grey%d.ppm",partial_name-39);
      }
      else if (50<=partial_name && partial_name<60) {//352x240 grey
	line_size = 352;
	num_lines = 240;
	pixel_size = 1;
	sprintf(snapshotFile,"../My/imgGrey%d.ppm",partial_name-50);
      }
      */
      if ((shotfd = open(snapshotFile, O_RDONLY )) < 0) {
	perror("Problems opening snapshot file");
	exit(-1);
      }
      
      pfi.tag = i;//reserved for future extension
      pfi.ppm1= 'P';
      pfi.ppm2=pixel_size < 3?'5':'6';
      pfi.size=line_size*num_lines*pixel_size;
      pfi.width=line_size;
      pfi.height=num_lines;
      pfi.buff=NULL;/* not used for udp between dvdview client/server*/
 
      j = 0;

      /* send the image */
      /* Forget the first 20 bytes */
      if (0<=partial_name && partial_name<30) {//640x480 color
	read(shotfd,buf,15);
      }
      else if (30<=partial_name && partial_name<40) {//352x240 color
	read(shotfd,buf,15);/*img0000*.ppm has 15 bytes as header*/
      }
      /*
      else if (40<=partial_name && partial_name<50) {//640x480 grey
	read(shotfd,buf,60);
      }
      else if (50<=partial_name && partial_name<60) {//352x240 grey
	read(shotfd,buf,60);
      }
      */

      /*
      PPM_UDP_Reader ppmudpreader(mus, maxpixelsize, maxlinesize, pfi,
			  SENDER_HOST,RECEIVER_HOST,portno);
      */
      ppmudpreader.setPPMFrameInfo (mus, pfi);

      readresult=read(shotfd,buf,pfi.size);   
      
      printf(" %d bytes read from file\n",readresult);

      sendresult = ppmudpreader.send_frame(buf,readresult,i);
      
      if (DEBUG_WHAT)
	printf("   The %dth file\n",i);

      if (sendresult<0) {
	printf(" [reader::main(): this frame is NOT sent \n");
	exit(-1);
      }
      
      j += sendresult;
      //usleep(1000);
    }

    if (DEBUG_WHERE)
      printf(" [reader::main(): %d bytes sent \n",j);
    
    close(shotfd);
  } while (DEBUG && !ctrlced);


  if (buf) free(buf);
  //close(sd);  

  return 0;
} /* end main */
