/* -*-C-*-
   @author: Yan, Wenchang
   @time: Mar, 2002
   ppmreceiver.cpp : testing program of PPMP
*/

extern "C" { 
#include <stdlib.h>
#include <stdio.h>
#include <time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <getopt.h>
 
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <string.h>
#include "connect.h"

#include <signal.h>

}

#include "PPMP.h"
#include "ppmdecoder.h"

#undef DEBUG_WHERE
#define DEBUG_WHERE 0

typedef enum {FALSE, TRUE} boolean;
 
static boolean verbose = TRUE;
 
/* Command line arguments */
static struct option longopts[] =
{
  {"verbose", no_argument, NULL, 'v'},
  {"help", no_argument, NULL, 'h'},
  {"nframes", required_argument, NULL, 'n'},
  {"format", required_argument, NULL, 'f'},
  {"portno",required_argument,NULL,'p'},
  {NULL, no_argument, NULL, 0}
};

/* for debug */
extern "C" int tprintf(const char* fmt, ...);
 
#define RECEIVER_HOST "localhost"
#define SENDER_HOST "localhost"
#define RECEIVER_PORT 9999

#define MUS 15360
#define MAXPIXELSIZE 3
#define MAXLINESIZE 640

static void  sig_int(int);

static int sd;/*socket descriptor */
 
#define PPM_MAGIC1 'P'
#define PPM_MAGIC2 '3'
#define RPPM_MAGIC2 '6'
 
/*
 * usage: Prints out 'usage' string
 *
 * Parameters: target 'stream' and 'status' to quit with
 *
 * Returns: -
 *
 */
static char* local_progname;
//static PPMDecoder ppmdecoder;
char* buf;

int shotfd;
 
static void
usage (FILE* stream, int status)
{
  fprintf(stream, "Usage: %s [-vh] [-M MTU] [-P maxpixel] [-L maxlinesize] [-f format] [-p portno]\n"
                  "\t -v --verbose            \t set verbose mode\n"
                  "\t -h --help               \t give this help\n"
	          "\t -M --MTU value          \t the approx. package size\n"
	          "\t -P --Pixel maxpixel     \t 3 is default\n"
	          "\t -L --Line maxlinesize   \t 640 pixels is default\n"
                  "\t -f --format  format     \t RGB format (dflt 2, RGB)\n"
                  "\t -p --port port_no       \t port number\n",
          "receiver");
  exit(status);
} /* end usage */

static int ctrlced = 0;
static void
sig_int(int signo)
{
  if (buf) free(buf);
  close(shotfd);
  ctrlced = 1;
}

/* PPM decoder stuff */

int
main (int argc, char* argv[])
{
  char filename[20];
  char header[20];
  char pixelline[3];
  int rawsize;
  struct tm *now;
  time_t nowt;

  char val1, val2, val3;
  int i, j, idx, row, col;
  int numFrames = 0;
  int format;
  int optc;
 

  int mus = MUS;
  int maxpixelsize = MAXPIXELSIZE;
  int maxlinesize = MAXLINESIZE;

  struct hostent* h;
  struct sockaddr_in sender;
  int result;
  int portno = RECEIVER_PORT;
  int bytes_recv = 0;
  int max_buf_size;

  struct sigaction intaction;




  /* Installs a handler for SIGINT so closedown() can be called when the
       server is killed by the ^C interrupt */
  intaction.sa_handler = sig_int;
  if (sigaction( SIGINT, &intaction, NULL ) < 0) {
    tprintf(" [receiver::main(): signal action error\n");
    exit(-1);
  }

  while ((optc = getopt_long(argc, argv, "+vhn:f:p:M:P:L:",
                             longopts, (int*) 0)) != EOF) {
    switch (optc) {
    case 'v':
      verbose = TRUE;
      break;
    case 'M':
      mus = atoi(optarg);
      break;
    case 'P':
      maxpixelsize = atoi(optarg);
      break;
    case 'L':
      maxlinesize = atoi(optarg);
      break;
    case 'h':
      usage(stdout, 0);
      break;
    case 'n':
      numFrames = atoi(optarg);
      if (numFrames < 0) usage (stderr, 1);
      break;
    case 'f':
      format = atoi(optarg);
      if (format < 1) usage(stderr, 1);
      break;
    case 'p':
      portno = atoi(optarg);
      if (portno<0) usage(stderr, 1);
      break;
    default:
      usage(stderr, 1);
    }
  }
 
  //if (numFrames < 1) usage(stderr, 1);
 
  /* no extra args */
  if (optind > argc) usage(stderr, 1);



  PPMDecoder ppmdecoder(mus,maxpixelsize,maxlinesize);
  /* Set up the udp connection */
  sd = connect_udp(RECEIVER_HOST,portno);

  if (sd<0) {
    fprintf(stderr, "[imagesender::main(): can't create socket\n");
    exit(-1);
  }

  /* allocate mem */
  //max_buf_size = MUS + PPMP_FRAME_INFO_OFFSET + sizeof(unsigned long) +
  //             sizeof(PPM_frame_info);
  max_buf_size = MUS + MAXPIXELSIZE*MAXLINESIZE;
    
  if (!(buf = (char*)malloc(max_buf_size)))
    {
      if (DEBUG_WHERE)
	tprintf(" [receiver::main(): 0ut of mem\n");
      exit(-1);
    }
 
  socklen_t sklt = sizeof(sender);

  while (!ctrlced) {
    
    result = recvfrom(sd, buf, max_buf_size, 0,
                        (struct sockaddr*)&sender,&sklt);
    if (result<0) {
        tprintf(" [receiver::main(): cannot receive \n");
        exit(-1);
    }

    if (DEBUG_WHERE)
      tprintf(" [receiver::main(): %d received from the sender\n",result);

    //decode the frame
    ppmdecoder.decode_ppm(buf, result);
     
    bytes_recv += result;
  }

  close(shotfd);

  return 0;
}

