/* -*-C-*-
   @author: Yan, Wenchang
   @time: Dec, 2001
*/

#include <src/config.h>
#include <stdlib.h>
#include <stdio.h>
#include <time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>      
#include <getopt.h>

#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <string.h>
#include <signal.h>
#include "connect.h"


#undef DEBUG_WHERE
#undef DEBUG_WHAT
#define DEBUG_WHERE 1
#define DEBUG_WHAT 1

/*for tprintf */
#include <sys/timeb.h>
#include <stdarg.h>


typedef enum {FALSE, TRUE} boolean;

static char* local_progname;
/*static boolean verbose = FALSE;*/
static boolean verbose = TRUE;
static int ctrlced = 0;/*^C*/

/* Command line arguments */
static struct option longopts[] =
{
  {"verbose", no_argument, NULL, 'v'},
  {"help", no_argument, NULL, 'h'},
  {"nframes", required_argument, NULL, 'n'},
  {"format", required_argument, NULL, 'f'},
  {"portno",required_argument,NULL,'p'},
  {NULL, no_argument, NULL, 0}
};

/* for debug */
extern int tprintf(const char* fmt, ...);

typedef unsigned long long uint64;
typedef   signed long long  int64;
typedef unsigned long      uint32;
typedef   signed long       int32;
typedef unsigned short     uint16;
typedef   signed short      int16;
typedef unsigned char      uint8;
typedef   signed char       int8;

inline uint16 ChangeEndian16(uint16 v) { return (v>>8) | (v<<8); }
inline uint32 ChangeEndian32(uint32 v) { return (v>>24) | ((v>>8)&0xFF00) | ((v&0xFF00)<<8) | (v<<
24); }


#ifdef WORDS_BIGENDIAN
inline uint16 ToLittleEndian16(uint16 v) { return ChangeEndian16(v); }
inline uint32 ToLittleEndian32(uint32 v) { return ChangeEndian32(v); }
inline uint16 ToBigEndian16(uint16 v)    { return v; }
inline uint32 ToBigEndian32(uint32 v)    { return v; }
#else
inline uint16 ToLittleEndian16(uint16 v) { return v; }
inline uint32 ToLittleEndian32(uint32 v) { return v; }
inline uint16 ToBigEndian16(uint16 v)    { return ChangeEndian16(v); }
inline uint32 ToBigEndian32(uint32 v)    { return ChangeEndian32(v); }
#endif



/* 
 * PPM stuff
 *
 */

typedef unsigned char pixval;

typedef struct {
  pixval r, g, b;
} pixel;


#define RECEIVER_HOST "localhost"
#define SENDER_HOST "localhost"
#define RECEIVER_PORT 12345
#define IMAGE_SIZE  921600 /* 3*640*480 */
/*
#define MUS 57600 
*//* bytes in each UDP packet sent */
#define MUS 15360 /*15360*/
#define MAXPIXELSIZE 3
#define MAXLINESIZE 640

#define PPM_OVERALLMAXVAL 65535
#define PPM_MAXMAXVAL 255

#define PPM_MAGIC1 'P'
#define PPM_MAGIC2 '3'
#define RPPM_MAGIC2 '6'
#define PPM_FORMAT (PPM_MAGIC1 * 256 + PPM_MAGIC2)
#define RPPM_FORMAT (PPM_MAGIC1 * 256 + RPPM_MAGIC2)
#define PPM_TYPE PPM_FORMAT

#define PPM_ASSIGN(p,red,grn,blu) \
   do { (p).r = (red); (p).g = (grn); (p).b = (blu); } while ( 0 )

/* from avs_raw.h */
typedef struct {
  int tag;
  char ppm1;
  char ppm2;
  int size;
  int width;
  int height;
  char* buff;
} PPM_frame_info, *PPM_frame_info_ptr;

/*
 * tprintf
 */

int tprintf(const char *fmt, ...) {
  va_list args;
  struct tm *tstruct;
  struct timeb tb;/* for fine granulity of millisec */
 
  time_t tsec;
 
  tsec = time(NULL);
  tstruct = localtime(&tsec);
 
  ftime(&tb);
 
  printf("[%02d:%02d.%d %5d] ",tstruct->tm_min,
         tstruct->tm_sec,
         tb.millitm,
         getpid());
  va_start(args, fmt);
  return vprintf(fmt, args);
}

/*
 * usage: Prints out 'usage' string
 *
 * Parameters: target 'stream' and 'status' to quit with
 *
 * Returns: -
 *
 */
 
static void
usage (FILE* stream, int status)
{
  fprintf(stream, "Usage: %s [-vh] -n num-frames [-M MTU] [-P maxpixel] [-L maxlinesize] [-f format] [-p portno]\n"
	          "\t -v --verbose            \t set verbose mode\n"
	          "\t -h --help               \t give this help\n"
	          "\t -n --nframes num-frames \t number of images\n"
                  "\t -M --MTU value          \t the approx. package size\n"
                  "\t -P --Pixel maxpixel     \t 3 is default\n"
                  "\t -L --Line maxlinesize   \t 640 pixels is default\n"
	          "\t -f --format  format     \t RGB format (dflt 2, RGB)\n" 
	          "\t -p --port port_no       \t port number\n",
	  local_progname);
  exit(status);
} /* end usage */

static void ctrlc(int signal)
{
    static char text[] = "^C - one moment please\n";
    fprintf(stdout,text,strlen(text));
    ctrlced=1;
}

int send_preframe(u_char* PPMHeader, PPM_frame_info* pfi,
		  int socket, struct sockaddr* receiver)
{
  u_char* buf;
  int sendresult;
  int PPMHeader_length = 4 + 4 + sizeof(PPM_frame_info);
  
  if (!(buf = (u_char*)malloc(PPMHeader_length*sizeof(u_char)))) {
    tprintf(" [imagereader::send_preframe(): 0ut of mem\n");
    exit(-1);
  }

  if (DEBUG_WHERE)
    tprintf(" [imagereader::send_preframe(): tag=%d,ppm1=%c,ppm2=%c,size=%d,width=%d,height=%d,*buff=%u\n",pfi->tag,pfi->ppm1,pfi->ppm2,pfi->size,pfi->width,pfi->height,pfi->buff);

  memcpy(buf,PPMHeader,4+4);
  memcpy(buf+4+4,pfi,sizeof(PPM_frame_info));

  if (DEBUG_WHERE)
    tprintf(" [imagereader::send_preframe(): PPMHeader_length=%d, buf[0]=%x, buf[1]=%x, buf[2]=%x, buf[3]=%x\n",PPMHeader_length,buf[0],buf[1],buf[2],buf[3]);

  sendresult = sendto(socket, buf, PPMHeader_length/*actual_size*/, 0,
  		receiver,sizeof(*receiver));

  //sendresult = sendto(socket,buf,4,0,receiver,sizeof(*receiver));

  if (sendresult<0) {
    tprintf(" [imagereader::send_preframe(): cannot send \n");
    exit(-1);
  }

  if (buf) free(buf);

  return 0;
}
  
  

int
main (int argc, char* argv[])
{
  /**/
  char snapshotFile[20];/* 640*480 color file 1-30
			   320*240 color file 31-40*/
  char duplicateFile[20];
  /**/
  /*
  char snapshotFile[]="../../../f14_short.mpg";
  */

  int shotfd,duplicate; 
  char header[20];//header[20] for 640x480 sample, header[15] for 352x240
  char pixelline[3];
  int rawsize;
  struct tm *now;
  time_t nowt;
  pixel *pixrow, *pP;
  u_char* image;
  u_char val1, val2, val3;
  int i, j, idx, row, col;
  int numFrames = 0;

  int optc;

  struct hostent* h;
  struct sockaddr_in receiver;
  int sd;/*socket descriptor */
  int sendresult,readresult;
  int portno = RECEIVER_PORT;
  u_char* buf;
  int format = 2;
  int header_idx;
  int actual_size;/*actual size of the packet sent each time*/

  int ii;
  uint32 icopy;
  /* For PPM frame information*/
  int mus = MUS;
  int maxpixelsize = MAXPIXELSIZE;
  int maxlinesize = MAXLINESIZE;

  PPM_frame_info pfi;
  int line_size,num_lines, pixel_size, lines_send_per_time, num_times;
  int switch_file=40;/* now have only 30 640x480 PPM files and 10 352x240
		        PPM files */
  int partial_name;
  int DEBUG;


  local_progname = argv[0];
  while ((optc = getopt_long(argc, argv, "+vhn:f::p::M::P::L::", 
			     longopts, (int*) 0)) != EOF) {
    switch (optc) {
    case 'v':
      verbose = TRUE;
      break;
    case 'h':
      usage(stdout, 0);
      break;
    case 'n':
      numFrames = atoi(optarg);
      if (numFrames < 0) usage (stderr, 1);
      break; 
    case 'M':
      mus = atoi(optarg);
      break;
    case 'P':
      maxpixelsize = atoi(optarg);
      break;
    case 'L':
      maxlinesize = atoi(optarg);
      break;

    case 'f':
      format = atoi(optarg);
      if (format < 1) usage(stderr, 1);
      break;
    case 'p':
      portno = atoi(optarg);
      if (portno<0) usage(stderr, 1);
      break;
    default:
      usage(stderr, 1);
    }
  }

  h = gethostbyname(RECEIVER_HOST);
  if(h==NULL) {
    printf(" [imagesender::main(): unknown host '%s' \n", RECEIVER_HOST);
    exit(1);
  }

  receiver.sin_family = h->h_addrtype;
  memcpy((char *) &receiver.sin_addr.s_addr, 
         h->h_addr_list[0], h->h_length);
  receiver.sin_port = htons(portno);

  /*set_server(&receiver,h->h_addr_list[0],portno);*/
  
  if (numFrames < 1) usage(stderr, 1);

  /* no extra args */
  if (optind > argc) usage(stderr, 1);

  /* Set up the udp connection */
  sd = connect_udp(SENDER_HOST,0);
  if (sd<0) {
    fprintf(stderr, "[imagesender::main(): can't create socket\n");
    exit(-1);
  }


  /* allocate mem */
  /* packet format (Specific fragmentation of PPM, various length):
 
     PPMHeader 4bytes + Frame# 4bytes + PPM_frame_info
     PictHeader 4bytes + Frame# 4bytes + Data MUS bytes
     PictHeader 4bytes + Frame# 4bytes + Data MUS bytes
     PictHeader 4bytes + Frame# 4bytes + Data MUS bytes
     ...

     Note: Need to REPEAT 15 times to send the whole 480*640*3 bytes 
     bitmap data for 640x480 color PPM image.
  */
  //Try to negotiate an appropriate buffer size first.
  //Here is the math:
  // lines_send_per_time = ceil {MUS/(line_size*pixel_size)}
  // num_times = num_lines / lines_send_per_time
  // Thus in order to find a buffer large enough, the data block size 
  // will be:
  //     pixel_size*line_size* lines_send_per_time
  //     <= pixel_size*line_size * {MUS/(line_size*pixel_size) +1}
  //     <= MUS + pixel_size*line_size
  //     <= MUS + max(pixel_size)* max(line_size)
  //
  // e.g <= MUS + 3*640 = MUS + 1920. 

  tprintf(" [imagereader::main(): mus=%d, pixelsize=%d, linesize=%d\n",
	  mus,maxpixelsize,maxlinesize);

  if (!(buf = (u_char*)malloc((mus + maxpixelsize * maxlinesize
			       +12+sizeof(PPM_frame_info))*sizeof(u_char))))
  {
    tprintf(" [imagereader::main(): 0ut of mem\n");
    exit(-1);
  }

  signal(SIGINT,ctrlc);

  DEBUG=1;//1 == forever
     
  do {
    for (i = 0; i < numFrames && !ctrlced; i++) {
      sleep(2);
      /*
	tprintf(" [imagesender::main(): Ready for another picture? [ENTER]\n");
	getchar();  
      */
      time(&nowt);
      now = localtime(&nowt);
      
      /*choose between 2 kinds of PPM files*/
      partial_name = i % switch_file ;

      if (0<=partial_name && partial_name<10) {//640x480 color
	line_size = 640;
	num_lines = 480;
	pixel_size = 3;/* 3 bytes */
	sprintf(snapshotFile,"/usr/local/mware/quo_5_2/UAV/uav/dvdview/src/My/sample%d.ppm",partial_name+1);
      }
      else if (30<=partial_name && partial_name<40) {//352x240 color
	line_size = 352;
	num_lines = 240;
	pixel_size = 3;
	sprintf(snapshotFile,"/usr/local/mware/quo_5_2/UAV/uav/dvdview/src/My/img0000%d.ppm",partial_name-30);
      }
      else
	continue;
      
      if ((shotfd = open(snapshotFile, O_RDONLY )) < 0) {
	perror("Problems opening snapshot file");
	exit(-1);
      }
      
      /*fill up pfi and other info needed in udp trans*/
      lines_send_per_time = (int)(((double)mus/((double)line_size*pixel_size))
				   + .9999);
      num_times =(int)(((double)num_lines/(double)lines_send_per_time)+.9999);

      pfi.tag = 0;/* not used now */
      pfi.ppm1= 'P';
      pfi.ppm2=pixel_size < 3?'5':'6';
      pfi.size=line_size*num_lines*pixel_size;
      pfi.width=line_size;
      pfi.height=num_lines;
      pfi.buff=NULL;/* not used for udp between dvdview client/server*/
 
      /*
      sprintf(duplicateFile,"duplicate%d.ppm",i);
      if ((duplicate = open(duplicateFile, O_CREAT|O_WRONLY, 
			    S_IRWXU|S_IRGRP|S_IROTH)) < 0) {
	perror("Problems opening duplicated file");
	exit(1);
      }

      sprintf(header, "%c%c\n%d %d\n%d\n", PPM_MAGIC1, RPPM_MAGIC2,
	      pfi.width, pfi.height, 255);
      write(duplicate, header, sizeof(header));
	*/
      //End

      j = 0;

      if (DEBUG_WHERE)
	tprintf(" [imagesender::main(): before sent, lines_send_per_time=%d, num_times=%d\n",lines_send_per_time,num_times);

      /* send the image */
      /* Forget the first 20 bytes */
      if (0<=partial_name && partial_name<30) {//640x480 color
	read(shotfd,buf,15);
      }
      else if (30<=partial_name && partial_name<40) {//352x240 color
	read(shotfd,buf,15);/*img0000*.ppm has 15 bytes as header*/
      }
      
      for (row=0;row<num_times;row++) {
	header_idx =0;

	//curr frame #
	icopy = ToBigEndian32(i);       
       /* if it's the first time reading from the PPM file */
       /* Send first the PPMHeader  */
	if (row == 0) {
	  buf[0]=buf[1]=0;
	  buf[2]=1; buf[3]=0xff;

	  //fill in frame #
	  memcpy(&buf[4],&i,4);
	  sendresult=send_preframe(buf, &pfi,
				  sd, (struct sockaddr*)&receiver);

	  usleep(100);
         //sendresult = sendto(sd, buf, 4, 0,
	 //			(struct sockaddr*)&receiver,sizeof(receiver));
	}
	else {
	 //actual_size = MUS+8;
	}

       /* Now append the PictHeader*/
	buf[header_idx]=buf[header_idx+1]=0;
	buf[header_idx+2]=1;
	buf[header_idx+3]=0xf0;
	header_idx+=4;
       /*Now the frame number*/
       memcpy(&buf[header_idx],&icopy,4);
       //tprintf(" [Imagereader: buf[]=%x, %x, %x, %x\n",buf[header_idx],
       //buf[header_idx+1],buf[header_idx+2],buf[header_idx+3]);

       header_idx+=4;
       /*Now the bitmap data*/
       buf += header_idx;
       readresult=read(shotfd,buf,lines_send_per_time*line_size*pixel_size);   
       
       buf -= header_idx;
       sendresult = sendto(sd, buf, readresult+8/*actual_size*/, 0,
			(struct sockaddr*)&receiver,sizeof(receiver));

       //debugging
       /* 
      for (ii=0;ii<16;ii++) {
	 printf("%x:",buf[ii]);
       }
       */
       if (DEBUG_WHAT)
	 tprintf("   The %dth file\n",i);


       if (sendresult<0) {
	 tprintf(" [imagesender::main(): cannot send \n");
	 exit(-1);
       }

       /*
       write(duplicate,buf+header_idx,readresult);
       */

       j += sendresult;
       usleep(1000);
      }

      if (DEBUG_WHERE)
	tprintf(" [imagesender::main(): %d bytes sent to %s:%d\n",j,
		inet_ntoa(receiver.sin_addr),ntohs(receiver.sin_port));
      /*      
      close(duplicate);
      */
      close(shotfd);
    }
  } while (DEBUG && !ctrlced);


  if (buf) free(buf);
  close(sd);  

  return 0;
} /* end main */
