/* -*-C-*-
   @author: Yan, Wenchang
   @time: Dec, 2001
*/
 
#include <src/config.h>
#include <stdlib.h>
#include <stdio.h>
#include <time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <getopt.h>
#include <vdev.h>
 
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <string.h>
#include "connect.h"

#include <signal.h>

/*for tprintf */
#include <sys/timeb.h>
#include <stdarg.h>
 
typedef enum {FALSE, TRUE} boolean;
 
static char* local_progname;
/*static boolean verbose = FALSE;*/
static boolean verbose = TRUE;
 
/* Command line arguments */
static struct option longopts[] =
{
  {"verbose", no_argument, NULL, 'v'},
  {"help", no_argument, NULL, 'h'},
  {"nframes", required_argument, NULL, 'n'},
  {"format", required_argument, NULL, 'f'},
  {"portno",required_argument,NULL,'p'},
  {NULL, no_argument, NULL, 0}
};

/* for debug */
extern int tprintf(const char* fmt, ...);
 
 
/*
 * PPM stuff
 *
 */
 
typedef unsigned char pixval;
 
typedef struct {
  pixval r, g, b;
} pixel;
 
 
#define RECEIVER_HOST "localhost"
#define SENDER_HOST "localhost"
#define RECEIVER_PORT 32333
#define IMAGE_SIZE  921600 /* 3*640*480 */
#define MUS 57600 /* bytes in each UDP sent */

static void  sig_int(int);

int sd;/*socket descriptor */
 
#define PPM_OVERALLMAXVAL 65535
#define PPM_MAXMAXVAL 255
 
#define PPM_MAGIC1 'P'
#define PPM_MAGIC2 '3'
#define RPPM_MAGIC2 '6'
#define PPM_FORMAT (PPM_MAGIC1 * 256 + PPM_MAGIC2)
#define RPPM_FORMAT (PPM_MAGIC1 * 256 + RPPM_MAGIC2)
#define PPM_TYPE PPM_FORMAT
 
#define PPM_ASSIGN(p,red,grn,blu) \
   do { (p).r = (red); (p).g = (grn); (p).b = (blu); } while ( 0 )

/*
 * tprintf
 */
 
int tprintf(const char *fmt, ...) {
  va_list args;
  struct tm *tstruct;
  struct timeb tb;/* for fine granulity of millisec */
 
  time_t tsec;
 
  tsec = time(NULL);
  tstruct = localtime(&tsec);
  /*
  printf("[%02d:%02d:%02d %5d] ",
         tstruct->tm_hour,
         tstruct->tm_min,
         tstruct->tm_sec,
         getpid());
  */
 
  ftime(&tb);
 
  printf("[%02d:%02d.%d %5d] ",tstruct->tm_min,
         tstruct->tm_sec,
         tb.millitm,
         getpid());
  va_start(args, fmt);
  return vprintf(fmt, args);
}

/*
 * usage: Prints out 'usage' string
 *
 * Parameters: target 'stream' and 'status' to quit with
 *
 * Returns: -
 *
 */
 
static void
usage (FILE* stream, int status)
{
  fprintf(stream, "Usage: %s [-vh] -n num-frames [-f format] [-p portno]\n"
                  "\t -v --verbose            \t set verbose mode\n"
                  "\t -h --help               \t give this help\n"
                  "\t -n --nframes num-frames \t number of images\n"
                  "\t -f --format  format     \t RGB format (dflt 2, RGB)\n"
                  "\t -p --port port_no       \t port number\n",
          local_progname);
  exit(status);
} /* end usage */

static void
sig_int(int signo)
{
  close(sd);
  exit(0);
}

int
main (int argc, char* argv[])
{
  char snapshotFile[40];
  int shotfd;
  char header[20];
  char pixelline[3];
  int rawsize;
  struct tm *now;
  time_t nowt;
  pixel *pixrow, *pP;
  u_char* buf;
  u_char val1, val2, val3;
  int i, j, idx, row, col;
  int numFrames = 0;
  int format = RBG;

  int optc;
 
  struct hostent* h;
  struct sockaddr_in sender;
  int result;
  int portno = RECEIVER_PORT;
  int bytes_recv = 0;

  struct sigaction intaction;

  /* Installs a handler for SIGINT so closedown() can be called when the
       server is killed by the ^C interrupt */
  intaction.sa_handler = sig_int;
  if (sigaction( SIGINT, &intaction, NULL ) < 0) {
    tprintf(" [imagereceiver::main(): signal action error\n");
    exit(-1);
  }

  /*
  h = gethostbyname(SENDER_HOST);
  if(h==NULL) {
    printf(" [imagereceiver.c::main(): unknown host '%s' \n", RECEIVER_HOST);
    exit(1);
  }
 
  set_server(&sender,*(struct in_addr*)(h->h_addr_list[0]),portno);
  */

  local_progname = argv[0];
  while ((optc = getopt_long(argc, argv, "+vhn:f:p:",
                             longopts, (int*) 0)) != EOF) {
    switch (optc) {
    case 'v':
      verbose = TRUE;
      break;
    case 'h':
      usage(stdout, 0);
      break;
    case 'n':
      numFrames = atoi(optarg);
      if (numFrames < 0) usage (stderr, 1);
      break;
    case 'f':
      format = atoi(optarg);
      if (format < 1) usage(stderr, 1);
      break;
    case 'p':
      portno = atoi(optarg);
      if (portno<0) usage(stderr, 1);
      break;
    default:
      usage(stderr, 1);
    }
  }
 
  if (numFrames < 1) usage(stderr, 1);
 
  /* no extra args */
  if (optind > argc) usage(stderr, 1);

  /* Set up the udp connection */
  sd = connect_udp(RECEIVER_HOST,RECEIVER_PORT);
  if (sd<0) {
    fprintf(stderr, "[imagesender::main(): can't create socket\n");
    exit(-1);
  }

  /* allocate mem */
  if (!(buf = (u_char*)malloc(MUS))) {
    tprintf(" [imagereceiver::main(): 0ut of mem\n");
    exit(-1);
  }
 
  i=0;
    /* write the ppm header */
      time(&nowt);
      now = localtime(&nowt);


      sprintf(snapshotFile, "snapshot-received%d-%d-%d-%d.ppm",
            i+1, now->tm_hour, now->tm_min, now->tm_sec);
 
      if ((shotfd = open(snapshotFile, O_CREAT|O_WRONLY,
                       S_IRWXU|S_IRGRP|S_IROTH)) < 0) {
        perror("Problems opening PPM-derived snapshot file");
        exit(1);
      }
 
      sprintf(header, "%c%c\n%d %d\n%d\n", PPM_MAGIC1, RPPM_MAGIC2,
            640, 480, 255);
      write(shotfd, header, sizeof(header));

  while (1) {

    if (bytes_recv >= IMAGE_SIZE) {
      i++;
      close(shotfd);
      time(&nowt);
      now = localtime(&nowt);

    /* write the ppm header */
      sprintf(snapshotFile, "snapshot-received%d-%d-%d-%d.ppm",
            i+1, now->tm_hour, now->tm_min, now->tm_sec);
 
      if ((shotfd = open(snapshotFile, O_CREAT|O_WRONLY,
                       S_IRWXU|S_IRGRP|S_IROTH)) < 0) {
        perror("Problems opening PPM-derived snapshot file");
        exit(1);
      }
 
      sprintf(header, "%c%c\n%d %d\n%d\n", PPM_MAGIC1, RPPM_MAGIC2,
            640, 480, 255);
      write(shotfd, header, sizeof(header));
      bytes_recv = 0;

    }
    
    j = sizeof(sender);
    result = recvfrom(sd, buf, MUS, 0,
                        (struct sockaddr*)&sender,&j);
    if (result<0) {
        tprintf(" [imagereceiver::main(): cannot receive \n");
        exit(-1);
    }

    tprintf(" [imagereceiver::main(): %d bytes receiverd\n",result);

    bytes_recv += result;
    write(shotfd,buf,result);
  }

  close(shotfd);
  return 0;
}

