/* -*-C-*-
*******************************************************************************
*
* File:         vdevCaminfo.c
* RCS:          $Id: vdevCaminfo.c,v 1.1.1.1 2001/07/03 20:30:44 fabianb Exp $
* Description:  Small tool to dump information on your video devices
* Author:       Fabian Ernesto Bustamante
*               Systems Research Group
*               College of Computing
*               Georgia Institute of Technology
* Created:      Wed May 09, 2001 at 16:53:32
* Modified:     Mon Jun 11, 2001 at 13:13:33 fabianb@cc.gatech.edu
* Language:     C
* Package:      N/A
* Status:       Experimental (Do Not Distribute)
*
* (C) Copyright 2001, Georgia Institute of Technology, all rights reserved.
*
*******************************************************************************
*/

#if HAVE_CONFIG_H
#  include <src/config.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <vdev.h>

int
main(int argc, char *argv[])
{
  VDev_T vdev;
  int i, numVDev;
  struct VDev_imageSize size;

  printf("-------------------------------------\n");
  printf(" INFORMATION ON AVAILABLE VIDEO CAMS \n");
  printf("-------------------------------------\n");

  VDev_init();
  
  numVDev = VDev_numVDevices();
  printf("Num devices: %d\n", numVDev);

  for (i = 0; i < numVDev; i++) {
    vdev = VDev_getVDevice(i);
    if (vdev == NULL) fprintf(stderr, "Wrong device?\n");
    printf("Valid? %s\n", VDev_isValid(vdev) ? "True":"False");
    
    printf("V4L name:    %s\n", VDev_getIntfName(vdev));
    printf("Device name: %s\n", VDev_getNodeName(vdev));

    VDev_getMaxSize(vdev, &size);
    printf("Max size (width, height): (%d, %d)\n", size.width, size.height);

    VDev_getMinSize(vdev, &size);
    printf("Min size (width, height): (%d, %d)\n", size.width, size.height); 

    VDev_getSize(vdev, &size);
    printf("Size (width, heigth): (%d, %d)\n", size.width, size.height);
    
    VDev_open(vdev, 1);
    printf("Frame rate: %d\n", VDev_getFramerate(vdev));
    printf("Brightness: %u\n", VDev_getBrightness(vdev));
    printf("Contrast: %u\n", VDev_getContrast(vdev));
    printf("Hue: %u\n", VDev_getHue(vdev));
    printf("Color: %u\n", VDev_getColor(vdev));
    printf("Whiteness: %u\n", VDev_getWhiteness(vdev));
    VDev_close(vdev);
    printf("-------------------------------------\n");
  }

  VDev_finalize();
  exit(0);
} /* end main */

