
// <block name="pipe_uavReceiverNormal">
/////////////////////////////////////////////////////////////////////////////
// receiver.c -- the implementation file for this node
/////////////////////////////////////////////////////////////////////////////
#include "receiver.h"
#include <sys/types.h>
#include <sys/stat.h>
#include <errno.h>


static EControlContext ec;
static CManager cm;

// this probably shouldn't go here (bad form), but we'll cope for now
char *incomingGlobalID;

// definition of publish types
#define PUBLISH_FILE  0x0001


// functions publishing and looking up
static int publish(char *pipe, char *connType, char *connName, char *str, int type);
static int publishFile(char *pipe, char *connType, char *connName, char *str);
static int unpublish(char *pipe, char *connType, char *connName, int type);
static int unpublishFile(char *pipe, char *connType, char *connName);
static char *lookup(char *pipe, char *nodeOrBranch, char *connPoint, int type, char *str);
static char *lookupFile(char *pipe, char *connType, char *connName, char *str);


// <block name="datatype_Raw_data">

static IOField Raw_dataFieldList[] = {
  
  {"tag", "integer ",sizeof(int), IOOffset( Raw_dataStruct*, tag)},
  
  {"ppm1", "char ",sizeof(char), IOOffset( Raw_dataStruct*, ppm1)},
  
  {"ppm2", "char ",sizeof(char), IOOffset( Raw_dataStruct*, ppm2)},
  
  {"size", "integer ",sizeof(int), IOOffset( Raw_dataStruct*, size)},
  
  {"width", "integer ",sizeof(int), IOOffset( Raw_dataStruct*, width)},
  
  {"height", "integer ",sizeof(int), IOOffset( Raw_dataStruct*, height)},
  
  {"maxval", "integer ",sizeof(int), IOOffset( Raw_dataStruct*, maxval)},
  
  {"buff", "char[size]",sizeof(char), IOOffset( Raw_dataStruct*, buff)},
  
  {NULL, NULL, 0, 0}
};

static CMFormatRec Raw_dataFormatList[] = 
{
  {"Raw_dataStruct",Raw_dataFieldList},
  { NULL, NULL}
};


// </block>

// <block name="datatype_Raw_data_1C">

static IOField Raw_data_1CFieldList[] = {
  
  {"tag", "integer ",sizeof(int), IOOffset( Raw_data_1CStruct*, tag)},
  
  {"ppm1", "char ",sizeof(char), IOOffset( Raw_data_1CStruct*, ppm1)},
  
  {"ppm2", "char ",sizeof(char), IOOffset( Raw_data_1CStruct*, ppm2)},
  
  {"size", "integer ",sizeof(int), IOOffset( Raw_data_1CStruct*, size)},
  
  {"width", "integer ",sizeof(int), IOOffset( Raw_data_1CStruct*, width)},
  
  {"height", "integer ",sizeof(int), IOOffset( Raw_data_1CStruct*, height)},
  
  {"maxval", "integer ",sizeof(int), IOOffset( Raw_data_1CStruct*, maxval)},
  
  {"buff", "char[921600]",sizeof(char), IOOffset( Raw_data_1CStruct*, buff)},
  
  {NULL, NULL, 0, 0}
};

static CMFormatRec Raw_data_1CFormatList[] = 
{
  {"Raw_data_1CStruct",Raw_data_1CFieldList},
  { NULL, NULL}
};


// </block>

// <block name="datatype_Raw_data_1G">

static IOField Raw_data_1GFieldList[] = {
  
  {"tag", "integer ",sizeof(int), IOOffset( Raw_data_1GStruct*, tag)},
  
  {"ppm1", "char ",sizeof(char), IOOffset( Raw_data_1GStruct*, ppm1)},
  
  {"ppm2", "char ",sizeof(char), IOOffset( Raw_data_1GStruct*, ppm2)},
  
  {"size", "integer ",sizeof(int), IOOffset( Raw_data_1GStruct*, size)},
  
  {"width", "integer ",sizeof(int), IOOffset( Raw_data_1GStruct*, width)},
  
  {"height", "integer ",sizeof(int), IOOffset( Raw_data_1GStruct*, height)},
  
  {"maxval", "integer ",sizeof(int), IOOffset( Raw_data_1GStruct*, maxval)},
  
  {"buff", "char[307200]",sizeof(char), IOOffset( Raw_data_1GStruct*, buff)},
  
  {NULL, NULL, 0, 0}
};

static CMFormatRec Raw_data_1GFormatList[] = 
{
  {"Raw_data_1GStruct",Raw_data_1GFieldList},
  { NULL, NULL}
};


// </block>

// <block name="datatype_Raw_data_1DC">

static IOField Raw_data_1DCFieldList[] = {
  
  {"tag", "integer ",sizeof(int), IOOffset( Raw_data_1DCStruct*, tag)},
  
  {"ppm1", "char ",sizeof(char), IOOffset( Raw_data_1DCStruct*, ppm1)},
  
  {"ppm2", "char ",sizeof(char), IOOffset( Raw_data_1DCStruct*, ppm2)},
  
  {"size", "integer ",sizeof(int), IOOffset( Raw_data_1DCStruct*, size)},
  
  {"width", "integer ",sizeof(int), IOOffset( Raw_data_1DCStruct*, width)},
  
  {"height", "integer ",sizeof(int), IOOffset( Raw_data_1DCStruct*, height)},
  
  {"maxval", "integer ",sizeof(int), IOOffset( Raw_data_1DCStruct*, maxval)},
  
  {"buff", "char[1843200]",sizeof(char), IOOffset( Raw_data_1DCStruct*, buff)},
  
  {NULL, NULL, 0, 0}
};

static CMFormatRec Raw_data_1DCFormatList[] = 
{
  {"Raw_data_1DCStruct",Raw_data_1DCFieldList},
  { NULL, NULL}
};


// </block>

// <block name="datatype_Raw_data_1DG">

static IOField Raw_data_1DGFieldList[] = {
  
  {"tag", "integer ",sizeof(int), IOOffset( Raw_data_1DGStruct*, tag)},
  
  {"ppm1", "char ",sizeof(char), IOOffset( Raw_data_1DGStruct*, ppm1)},
  
  {"ppm2", "char ",sizeof(char), IOOffset( Raw_data_1DGStruct*, ppm2)},
  
  {"size", "integer ",sizeof(int), IOOffset( Raw_data_1DGStruct*, size)},
  
  {"width", "integer ",sizeof(int), IOOffset( Raw_data_1DGStruct*, width)},
  
  {"height", "integer ",sizeof(int), IOOffset( Raw_data_1DGStruct*, height)},
  
  {"maxval", "integer ",sizeof(int), IOOffset( Raw_data_1DGStruct*, maxval)},
  
  {"buff", "char[614400]",sizeof(char), IOOffset( Raw_data_1DGStruct*, buff)},
  
  {NULL, NULL, 0, 0}
};

static CMFormatRec Raw_data_1DGFormatList[] = 
{
  {"Raw_data_1DGStruct",Raw_data_1DGFieldList},
  { NULL, NULL}
};


// </block>

// <block name="datatype_Raw_data_2C">

static IOField Raw_data_2CFieldList[] = {
  
  {"tag", "integer ",sizeof(int), IOOffset( Raw_data_2CStruct*, tag)},
  
  {"ppm1", "char ",sizeof(char), IOOffset( Raw_data_2CStruct*, ppm1)},
  
  {"ppm2", "char ",sizeof(char), IOOffset( Raw_data_2CStruct*, ppm2)},
  
  {"size", "integer ",sizeof(int), IOOffset( Raw_data_2CStruct*, size)},
  
  {"width", "integer ",sizeof(int), IOOffset( Raw_data_2CStruct*, width)},
  
  {"height", "integer ",sizeof(int), IOOffset( Raw_data_2CStruct*, height)},
  
  {"maxval", "integer ",sizeof(int), IOOffset( Raw_data_2CStruct*, maxval)},
  
  {"buff", "char[230400]",sizeof(char), IOOffset( Raw_data_2CStruct*, buff)},
  
  {NULL, NULL, 0, 0}
};

static CMFormatRec Raw_data_2CFormatList[] = 
{
  {"Raw_data_2CStruct",Raw_data_2CFieldList},
  { NULL, NULL}
};


// </block>

// <block name="datatype_Raw_data_2G">

static IOField Raw_data_2GFieldList[] = {
  
  {"tag", "integer ",sizeof(int), IOOffset( Raw_data_2GStruct*, tag)},
  
  {"ppm1", "char ",sizeof(char), IOOffset( Raw_data_2GStruct*, ppm1)},
  
  {"ppm2", "char ",sizeof(char), IOOffset( Raw_data_2GStruct*, ppm2)},
  
  {"size", "integer ",sizeof(int), IOOffset( Raw_data_2GStruct*, size)},
  
  {"width", "integer ",sizeof(int), IOOffset( Raw_data_2GStruct*, width)},
  
  {"height", "integer ",sizeof(int), IOOffset( Raw_data_2GStruct*, height)},
  
  {"maxval", "integer ",sizeof(int), IOOffset( Raw_data_2GStruct*, maxval)},
  
  {"buff", "char[76800]",sizeof(char), IOOffset( Raw_data_2GStruct*, buff)},
  
  {NULL, NULL, 0, 0}
};

static CMFormatRec Raw_data_2GFormatList[] = 
{
  {"Raw_data_2GStruct",Raw_data_2GFieldList},
  { NULL, NULL}
};


// </block>

// <block name="datatype_Raw_data_2DC">

static IOField Raw_data_2DCFieldList[] = {
  
  {"tag", "integer ",sizeof(int), IOOffset( Raw_data_2DCStruct*, tag)},
  
  {"ppm1", "char ",sizeof(char), IOOffset( Raw_data_2DCStruct*, ppm1)},
  
  {"ppm2", "char ",sizeof(char), IOOffset( Raw_data_2DCStruct*, ppm2)},
  
  {"size", "integer ",sizeof(int), IOOffset( Raw_data_2DCStruct*, size)},
  
  {"width", "integer ",sizeof(int), IOOffset( Raw_data_2DCStruct*, width)},
  
  {"height", "integer ",sizeof(int), IOOffset( Raw_data_2DCStruct*, height)},
  
  {"maxval", "integer ",sizeof(int), IOOffset( Raw_data_2DCStruct*, maxval)},
  
  {"buff", "char[1843200]",sizeof(char), IOOffset( Raw_data_2DCStruct*, buff)},
  
  {NULL, NULL, 0, 0}
};

static CMFormatRec Raw_data_2DCFormatList[] = 
{
  {"Raw_data_2DCStruct",Raw_data_2DCFieldList},
  { NULL, NULL}
};


// </block>

// <block name="datatype_Raw_data_2DG">

static IOField Raw_data_2DGFieldList[] = {
  
  {"tag", "integer ",sizeof(int), IOOffset( Raw_data_2DGStruct*, tag)},
  
  {"ppm1", "char ",sizeof(char), IOOffset( Raw_data_2DGStruct*, ppm1)},
  
  {"ppm2", "char ",sizeof(char), IOOffset( Raw_data_2DGStruct*, ppm2)},
  
  {"size", "integer ",sizeof(int), IOOffset( Raw_data_2DGStruct*, size)},
  
  {"width", "integer ",sizeof(int), IOOffset( Raw_data_2DGStruct*, width)},
  
  {"height", "integer ",sizeof(int), IOOffset( Raw_data_2DGStruct*, height)},
  
  {"maxval", "integer ",sizeof(int), IOOffset( Raw_data_2DGStruct*, maxval)},
  
  {"buff", "char[614400]",sizeof(char), IOOffset( Raw_data_2DGStruct*, buff)},
  
  {NULL, NULL, 0, 0}
};

static CMFormatRec Raw_data_2DGFormatList[] = 
{
  {"Raw_data_2DGStruct",Raw_data_2DGFieldList},
  { NULL, NULL}
};


// </block>

// <block name="datatype_Raw_data_3C">

static IOField Raw_data_3CFieldList[] = {
  
  {"tag", "integer ",sizeof(int), IOOffset( Raw_data_3CStruct*, tag)},
  
  {"ppm1", "char ",sizeof(char), IOOffset( Raw_data_3CStruct*, ppm1)},
  
  {"ppm2", "char ",sizeof(char), IOOffset( Raw_data_3CStruct*, ppm2)},
  
  {"size", "integer ",sizeof(int), IOOffset( Raw_data_3CStruct*, size)},
  
  {"width", "integer ",sizeof(int), IOOffset( Raw_data_3CStruct*, width)},
  
  {"height", "integer ",sizeof(int), IOOffset( Raw_data_3CStruct*, height)},
  
  {"maxval", "integer ",sizeof(int), IOOffset( Raw_data_3CStruct*, maxval)},
  
  {"buff", "char[57600]",sizeof(char), IOOffset( Raw_data_3CStruct*, buff)},
  
  {NULL, NULL, 0, 0}
};

static CMFormatRec Raw_data_3CFormatList[] = 
{
  {"Raw_data_3CStruct",Raw_data_3CFieldList},
  { NULL, NULL}
};


// </block>

// <block name="datatype_Raw_data_3G">

static IOField Raw_data_3GFieldList[] = {
  
  {"tag", "integer ",sizeof(int), IOOffset( Raw_data_3GStruct*, tag)},
  
  {"ppm1", "char ",sizeof(char), IOOffset( Raw_data_3GStruct*, ppm1)},
  
  {"ppm2", "char ",sizeof(char), IOOffset( Raw_data_3GStruct*, ppm2)},
  
  {"size", "integer ",sizeof(int), IOOffset( Raw_data_3GStruct*, size)},
  
  {"width", "integer ",sizeof(int), IOOffset( Raw_data_3GStruct*, width)},
  
  {"height", "integer ",sizeof(int), IOOffset( Raw_data_3GStruct*, height)},
  
  {"maxval", "integer ",sizeof(int), IOOffset( Raw_data_3GStruct*, maxval)},
  
  {"buff", "char[19200]",sizeof(char), IOOffset( Raw_data_3GStruct*, buff)},
  
  {NULL, NULL, 0, 0}
};

static CMFormatRec Raw_data_3GFormatList[] = 
{
  {"Raw_data_3GStruct",Raw_data_3GFieldList},
  { NULL, NULL}
};


// </block>

// <block name="datatype_Raw_data_3DC">

static IOField Raw_data_3DCFieldList[] = {
  
  {"tag", "integer ",sizeof(int), IOOffset( Raw_data_3DCStruct*, tag)},
  
  {"ppm1", "char ",sizeof(char), IOOffset( Raw_data_3DCStruct*, ppm1)},
  
  {"ppm2", "char ",sizeof(char), IOOffset( Raw_data_3DCStruct*, ppm2)},
  
  {"size", "integer ",sizeof(int), IOOffset( Raw_data_3DCStruct*, size)},
  
  {"width", "integer ",sizeof(int), IOOffset( Raw_data_3DCStruct*, width)},
  
  {"height", "integer ",sizeof(int), IOOffset( Raw_data_3DCStruct*, height)},
  
  {"maxval", "integer ",sizeof(int), IOOffset( Raw_data_3DCStruct*, maxval)},
  
  {"buff", "char[1843200]",sizeof(char), IOOffset( Raw_data_3DCStruct*, buff)},
  
  {NULL, NULL, 0, 0}
};

static CMFormatRec Raw_data_3DCFormatList[] = 
{
  {"Raw_data_3DCStruct",Raw_data_3DCFieldList},
  { NULL, NULL}
};


// </block>

// <block name="datatype_Raw_data_3DG">

static IOField Raw_data_3DGFieldList[] = {
  
  {"tag", "integer ",sizeof(int), IOOffset( Raw_data_3DGStruct*, tag)},
  
  {"ppm1", "char ",sizeof(char), IOOffset( Raw_data_3DGStruct*, ppm1)},
  
  {"ppm2", "char ",sizeof(char), IOOffset( Raw_data_3DGStruct*, ppm2)},
  
  {"size", "integer ",sizeof(int), IOOffset( Raw_data_3DGStruct*, size)},
  
  {"width", "integer ",sizeof(int), IOOffset( Raw_data_3DGStruct*, width)},
  
  {"height", "integer ",sizeof(int), IOOffset( Raw_data_3DGStruct*, height)},
  
  {"maxval", "integer ",sizeof(int), IOOffset( Raw_data_3DGStruct*, maxval)},
  
  {"buff", "char[614400]",sizeof(char), IOOffset( Raw_data_3DGStruct*, buff)},
  
  {NULL, NULL, 0, 0}
};

static CMFormatRec Raw_data_3DGFormatList[] = 
{
  {"Raw_data_3DGStruct",Raw_data_3DGFieldList},
  { NULL, NULL}
};


// </block>

// <block name="datatype_Raw_data_4C">

static IOField Raw_data_4CFieldList[] = {
  
  {"tag", "integer ",sizeof(int), IOOffset( Raw_data_4CStruct*, tag)},
  
  {"ppm1", "char ",sizeof(char), IOOffset( Raw_data_4CStruct*, ppm1)},
  
  {"ppm2", "char ",sizeof(char), IOOffset( Raw_data_4CStruct*, ppm2)},
  
  {"size", "integer ",sizeof(int), IOOffset( Raw_data_4CStruct*, size)},
  
  {"width", "integer ",sizeof(int), IOOffset( Raw_data_4CStruct*, width)},
  
  {"height", "integer ",sizeof(int), IOOffset( Raw_data_4CStruct*, height)},
  
  {"maxval", "integer ",sizeof(int), IOOffset( Raw_data_4CStruct*, maxval)},
  
  {"buff", "char[14400]",sizeof(char), IOOffset( Raw_data_4CStruct*, buff)},
  
  {NULL, NULL, 0, 0}
};

static CMFormatRec Raw_data_4CFormatList[] = 
{
  {"Raw_data_4CStruct",Raw_data_4CFieldList},
  { NULL, NULL}
};


// </block>

// <block name="datatype_Raw_data_4G">

static IOField Raw_data_4GFieldList[] = {
  
  {"tag", "integer ",sizeof(int), IOOffset( Raw_data_4GStruct*, tag)},
  
  {"ppm1", "char ",sizeof(char), IOOffset( Raw_data_4GStruct*, ppm1)},
  
  {"ppm2", "char ",sizeof(char), IOOffset( Raw_data_4GStruct*, ppm2)},
  
  {"size", "integer ",sizeof(int), IOOffset( Raw_data_4GStruct*, size)},
  
  {"width", "integer ",sizeof(int), IOOffset( Raw_data_4GStruct*, width)},
  
  {"height", "integer ",sizeof(int), IOOffset( Raw_data_4GStruct*, height)},
  
  {"maxval", "integer ",sizeof(int), IOOffset( Raw_data_4GStruct*, maxval)},
  
  {"buff", "char[4800]",sizeof(char), IOOffset( Raw_data_4GStruct*, buff)},
  
  {NULL, NULL, 0, 0}
};

static CMFormatRec Raw_data_4GFormatList[] = 
{
  {"Raw_data_4GStruct",Raw_data_4GFieldList},
  { NULL, NULL}
};


// </block>

// <block name="datatype_Raw_data_4DC">

static IOField Raw_data_4DCFieldList[] = {
  
  {"tag", "integer ",sizeof(int), IOOffset( Raw_data_4DCStruct*, tag)},
  
  {"ppm1", "char ",sizeof(char), IOOffset( Raw_data_4DCStruct*, ppm1)},
  
  {"ppm2", "char ",sizeof(char), IOOffset( Raw_data_4DCStruct*, ppm2)},
  
  {"size", "integer ",sizeof(int), IOOffset( Raw_data_4DCStruct*, size)},
  
  {"width", "integer ",sizeof(int), IOOffset( Raw_data_4DCStruct*, width)},
  
  {"height", "integer ",sizeof(int), IOOffset( Raw_data_4DCStruct*, height)},
  
  {"maxval", "integer ",sizeof(int), IOOffset( Raw_data_4DCStruct*, maxval)},
  
  {"buff", "char[1843200]",sizeof(char), IOOffset( Raw_data_4DCStruct*, buff)},
  
  {NULL, NULL, 0, 0}
};

static CMFormatRec Raw_data_4DCFormatList[] = 
{
  {"Raw_data_4DCStruct",Raw_data_4DCFieldList},
  { NULL, NULL}
};


// </block>

// <block name="datatype_Raw_data_4DG">

static IOField Raw_data_4DGFieldList[] = {
  
  {"tag", "integer ",sizeof(int), IOOffset( Raw_data_4DGStruct*, tag)},
  
  {"ppm1", "char ",sizeof(char), IOOffset( Raw_data_4DGStruct*, ppm1)},
  
  {"ppm2", "char ",sizeof(char), IOOffset( Raw_data_4DGStruct*, ppm2)},
  
  {"size", "integer ",sizeof(int), IOOffset( Raw_data_4DGStruct*, size)},
  
  {"width", "integer ",sizeof(int), IOOffset( Raw_data_4DGStruct*, width)},
  
  {"height", "integer ",sizeof(int), IOOffset( Raw_data_4DGStruct*, height)},
  
  {"maxval", "integer ",sizeof(int), IOOffset( Raw_data_4DGStruct*, maxval)},
  
  {"buff", "char[614400]",sizeof(char), IOOffset( Raw_data_4DGStruct*, buff)},
  
  {NULL, NULL, 0, 0}
};

static CMFormatRec Raw_data_4DGFormatList[] = 
{
  {"Raw_data_4DGStruct",Raw_data_4DGFieldList},
  { NULL, NULL}
};


// </block>

// <block name="datatype_rectangle">

static IOField rectangleFieldList[] = {
  
  {"x", "integer ",sizeof(int), IOOffset( rectangleStruct*, x)},
  
  {"y", "integer ",sizeof(int), IOOffset( rectangleStruct*, y)},
  
  {"width", "integer ",sizeof(int), IOOffset( rectangleStruct*, width)},
  
  {"height", "integer ",sizeof(int), IOOffset( rectangleStruct*, height)},
  
  {NULL, NULL, 0, 0}
};

static CMFormatRec rectangleFormatList[] = 
{
  {"rectangleStruct",rectangleFieldList},
  { NULL, NULL}
};


// </block>

// <block name="datatype_factor">

static IOField factorFieldList[] = {
  
  {"factor", "integer ",sizeof(int), IOOffset( factorStruct*, factor)},
  
  {NULL, NULL, 0, 0}
};

static CMFormatRec factorFormatList[] = 
{
  {"factorStruct",factorFieldList},
  { NULL, NULL}
};


// </block>
 


int makePathDir(char *path, mode_t mode)
{
  char *dir; 
  char *p;
  mode = 0777;

  dir = (char *)malloc(sizeof(char) * (strlen(path) + 1));

  p = path;

  while(1)
  {
    p = strchr(p, '/');
    if (p == NULL) 
    {
      free( dir );
      return 0;
    }

    p++;
    // get first dir name including '/'
    memcpy(dir, path, p - path);
    dir[p - path] = 0;

    // make directory
    if (mkdir(dir, mode))
    {
      // if dir exists already, continue
      // otherwise(error occurs), return -1
      if (errno != EEXIST)
      {
        free(dir);
        return -1;
      }
    }
  }

  // cannot reach here
  return 0;
}

// publish
static int publish(char *pipe, char *connType, char *connName, char *str, int type)
{
  if (type & PUBLISH_FILE)
  {
    publishFile(pipe, connType, connName, str);
  }

  return 0;
}

static int publishFile(char *pipe, char *connType, char *connName, char *str)
{
  char *publishRoot;
  char *filename;
  FILE *ofile;

  if ((publishRoot = getenv("INFOPIPE_PUBLISH_ROOT")) == NULL)
  {
    // if no INFOPIPE_PUBLISH_ROOT is defined, use current directory
    publishRoot = getenv("PWD");
  }

  filename = (char *)malloc( sizeof(char) * (strlen(publishRoot) + strlen(pipe) 
                                             + strlen(connType) + strlen(connName)
                                             + 10));

  // make filename = ""
  filename[0] = 0;
  strcat(filename, publishRoot);

  // if the path does not have the last '/', add '/'
  if (filename[strlen(publishRoot)-1] != '/')
    strcat(filename, "/");

  strcat(filename, pipe);
  strcat(filename, "/");
  strcat(filename, connType);
  strcat(filename, "/");
  strcat(filename, connName);

  makePathDir(filename,0);

  printf("publishing file = %s\n",  filename);

  ofile = fopen(filename,"w");
  if (ofile)
  {
    fprintf(ofile,"%s\n",str);
  }
  else
    printf("Error in publishing file %s\n", filename);
 
  // close and write info
  fclose(ofile);
  return 0;
}


static int unpublish(char *pipe, char *connType, char *connName, int type)
{
  if (type & PUBLISH_FILE)
  {
    unpublishFile(pipe, connType, connName);
  }

  return 0;
}

static int unpublishFile(char *pipe, char *connType, char *connName)
{
  char *publishRoot;
  char *filename;

  if ((publishRoot = getenv("INFOPIPE_PUBLISH_ROOT")) == NULL)
  {
    // if no INFOPIPE_PUBLISH_ROOT is defined, use current directory
    publishRoot = getenv("PWD");
  }

  filename = (char *) malloc(sizeof(char) * (strlen(publishRoot) + strlen(pipe) + 
                                             strlen(connType) + strlen(connName) + 10));

  // make filename = ""
  filename[0] = 0;
  strcat(filename, publishRoot);

  // if the path does not have the last '/', add '/'
  if (filename[strlen(publishRoot)-1] != '/')
    strcat(filename, "/");

  strcat(filename, pipe);
  strcat(filename, "/");
  strcat(filename, connType);
  strcat(filename, "/");
  strcat(filename, connName);

  return remove(filename);
}


static char *lookup(char *pipe, char *connType, char *connName, int type, char *str)
{
  if (type & PUBLISH_FILE)
  {
    return lookupFile(pipe, connType, connName, str);
  }

}


static char *lookupFile(char *pipe, char *connType, char *connName, char *str)
{
  char *publishRoot;
  char *filename;
  FILE *ifile;

  if ((publishRoot = getenv("INFOPIPE_PUBLISH_ROOT")) == NULL)
  {
    // if no INFOPIPE_PUBLISH_ROOT is defined, use current directory
    publishRoot = getenv("PWD");
  }

  filename = (char *)malloc(sizeof(char) * (strlen(publishRoot) + strlen(pipe) + 
                                  		      strlen(connType) + strlen(connName) + 10));

  // make filename = ""
  filename[0] = 0;
  strcat(filename, publishRoot);

  // if the path does not have the last '/', add '/'
  if (filename[strlen(publishRoot)-1] != '/')
    strcat(filename, "/");

  strcat(filename, pipe);
  strcat(filename, "/");
  strcat(filename, connType);
  strcat(filename, "/");
  strcat(filename, connName);

  ifile = fopen(filename, "r");
  if (ifile && !feof(ifile))
  {
    fgets(str,strlen(str),ifile);
  }
  else
    fprintf(stderr,"Error in looking up file %s\n",filename); 

  return str;
}




void receiver__receive1Handler( void *, void *, attr_list );

// variables to track




extern IOContext ioc;
static int doPipeInit(void)
{
/*
  IOFormat format;

*/
  gen_pthread_init();
  cm = CManager_create();
//  forked = CMfork_comm_thread(cm);
//  if( !forked ) {
//    fprintf(stderr, "Fork of comm thread failed. Exiting.\n");
//    exit(1);
//  }

  ec = ECho_CM_init(cm);
  return 0;
}


// incoming channel vars for receive
static ECSinkHandle receiver__receive1ReceiveHandle;
static EChannel receiver__receive1ReceiveChannel;

  // Filtering code for pipe "receiver" port "receive1"
  

// pipe receiver
// port receive1

#include <string.h>
// filterInfoStructDef -- defines a struct which holds data about
// -- a filter available for installation
// -- name - name by which a filter is refereneced
// -- inType - input type a filter expects, should matching ouput of pipe on which filter is placed
// -- outType - output type a filter expects, could be ""
// -- outTypePtr - output type a filter expects, could be NULL
// -- paramsType - the type (a char name) of params that a filter may use, could be NULL
struct filterInfoStructDef
{
  char *name;
  char *uri;
  char *inType;
  char *outType;
  IOFieldList outTypePtr;  
  int   takesParams;
  IOFieldList paramsTypeList;
};

// filterTableStructDef -- way to store filter info about
// -- available filters. Fields map what was named above.
typedef struct filterInfoStructDef filterInfo;
filterInfo filterTable[] = {


// <block name="filter_cropImage1Cto2C">
  { "cropImage1Cto2C",
      "cropImage.ecl",    
          "Raw_data_1C",     
              "Raw_data_2C",  
                  Raw_data_2CFieldList,  
                      1, 
                          rectangleFieldList },
// </block>
// <block name="filter_cropImage1Cto3C">
  { "cropImage1Cto3C",
      "cropImage.ecl",    
          "Raw_data_1C",     
              "Raw_data_3C",  
                  Raw_data_3CFieldList,  
                      1, 
                          rectangleFieldList },
// </block>
// <block name="filter_cropImage1Cto4C">
  { "cropImage1Cto4C",
      "cropImage.ecl",    
          "Raw_data_1C",     
              "Raw_data_4C",  
                  Raw_data_4CFieldList,  
                      1, 
                          rectangleFieldList },
// </block>
// <block name="filter_cropImage2Cto3C">
  { "cropImage2Cto3C",
      "cropImage.ecl",    
          "Raw_data_2C",     
              "Raw_data_3C",  
                  Raw_data_3CFieldList,  
                      1, 
                          rectangleFieldList },
// </block>
// <block name="filter_cropImage2Cto4C">
  { "cropImage2Cto4C",
      "cropImage.ecl",    
          "Raw_data_2C",     
              "Raw_data_4C",  
                  Raw_data_4CFieldList,  
                      1, 
                          rectangleFieldList },
// </block>
// <block name="filter_doubleImage2C">
  { "doubleImage2C",
      "enlargeImage.ecl",    
          "Raw_data_2C",     
              "Raw_data_1C",  
                  Raw_data_1CFieldList,  
                      1, 
                          factorFieldList },
// </block>
// <block name="filter_doubleImage3C">
  { "doubleImage3C",
      "enlargeImage.ecl",    
          "Raw_data_3C",     
              "Raw_data_2C",  
                  Raw_data_2CFieldList,  
                      1, 
                          factorFieldList },
// </block>
// <block name="filter_doubleImage4C">
  { "doubleImage4C",
      "enlargeImage.ecl",    
          "Raw_data_4C",     
              "Raw_data_3C",  
                  Raw_data_3CFieldList,  
                      1, 
                          factorFieldList },
// </block>
// <block name="filter_edgeImage1C">
  { "edgeImage1C",
      "edgeImage.ecl",    
          "Raw_data_1C",     
              "Raw_data_1G",  
                  Raw_data_1GFieldList,  
                      0, 
                          NULL },
// </block>
// <block name="filter_edgeImage2C">
  { "edgeImage2C",
      "edgeImage.ecl",    
          "Raw_data_2C",     
              "Raw_data_2G",  
                  Raw_data_2GFieldList,  
                      0, 
                          NULL },
// </block>
// <block name="filter_edgeImage2G">
  { "edgeImage2G",
      "edgeImage.ecl",    
          "Raw_data_2G",     
              "Raw_data_2G",  
                  Raw_data_2GFieldList,  
                      0, 
                          NULL },
// </block>
// <block name="filter_edgeImage3C">
  { "edgeImage3C",
      "edgeImage.ecl",    
          "Raw_data_3C",     
              "Raw_data_3G",  
                  Raw_data_3GFieldList,  
                      0, 
                          NULL },
// </block>
// <block name="filter_edgeImage3G">
  { "edgeImage3G",
      "edgeImage.ecl",    
          "Raw_data_3G",     
              "Raw_data_3G",  
                  Raw_data_3GFieldList,  
                      0, 
                          NULL },
// </block>
// <block name="filter_edgeImage4C">
  { "edgeImage4C",
      "edgeImage.ecl",    
          "Raw_data_4C",     
              "Raw_data_4G",  
                  Raw_data_4GFieldList,  
                      0, 
                          NULL },
// </block>
// <block name="filter_edgeImage4G">
  { "edgeImage4G",
      "edgeImage.ecl",    
          "Raw_data_4G",     
              "Raw_data_4G",  
                  Raw_data_4GFieldList,  
                      0, 
                          NULL },
// </block>
// <block name="filter_greyImage1C">
  { "greyImage1C",
      "greyImage.ecl",    
          "Raw_data_1C",     
              "Raw_data_1G",  
                  Raw_data_1GFieldList,  
                      0, 
                          NULL },
// </block>
// <block name="filter_greyImage2C">
  { "greyImage2C",
      "greyImage.ecl",    
          "Raw_data_2C",     
              "Raw_data_2G",  
                  Raw_data_2GFieldList,  
                      0, 
                          NULL },
// </block>
// <block name="filter_greyImage3C">
  { "greyImage3C",
      "greyImage.ecl",    
          "Raw_data_3C",     
              "Raw_data_3G",  
                  Raw_data_3GFieldList,  
                      0, 
                          NULL },
// </block>
// <block name="filter_greyImage4C">
  { "greyImage4C",
      "greyImage.ecl",    
          "Raw_data_4C",     
              "Raw_data_4G",  
                  Raw_data_4GFieldList,  
                      0, 
                          NULL },
// </block>
// <block name="filter_halfImage1C">
  { "halfImage1C",
      "reduceImage.ecl",    
          "Raw_data_1C",     
              "Raw_data_2C",  
                  Raw_data_2CFieldList,  
                      1, 
                          factorFieldList },
// </block>
// <block name="filter_halfImage2C">
  { "halfImage2C",
      "reduceImage.ecl",    
          "Raw_data_2C",     
              "Raw_data_3C",  
                  Raw_data_3CFieldList,  
                      1, 
                          factorFieldList },
// </block>
// <block name="filter_halfImage3C">
  { "halfImage3C",
      "reduceImage.ecl",    
          "Raw_data_3C",     
              "Raw_data_4C",  
                  Raw_data_4CFieldList,  
                      1, 
                          factorFieldList },
// </block>
// <block name="filter_halfRate">
  { "halfRate",
      "halfRate.ecl",    
          "*",     
              "Raw_data_2C",  
                  Raw_data_2CFieldList,  
                      0, 
                          NULL },
// </block>
// <block name="filter_identity">
  { "identity",
      "identity.ecl",    
          "*",     
              "Raw_data_1C",  
                  Raw_data_1CFieldList,  
                      0, 
                          NULL },
// </block>
// <block name="filter_shrinkAvgImage2Cto3C">
  { "shrinkAvgImage2Cto3C",
      "shrinkAvgImage.ecl",    
          "Raw_data_2C",     
              "Raw_data_3C",  
                  Raw_data_3CFieldList,  
                      0, 
                          NULL },
// </block>
// <block name="filter_shrinkAvgImage3Cto4C">
  { "shrinkAvgImage3Cto4C",
      "shrinkAvgImage.ecl",    
          "Raw_data_3C",     
              "Raw_data_4C",  
                  Raw_data_4CFieldList,  
                      0, 
                          NULL },
// </block>
// <block name="filter_shrinkImage2Cto3C">
  { "shrinkImage2Cto3C",
      "shrinkImage.ecl",    
          "Raw_data_2C",     
              "Raw_data_3C",  
                  Raw_data_3CFieldList,  
                      0, 
                          NULL },
// </block>
// <block name="filter_shrinkImage3Cto4C">
  { "shrinkImage3Cto4C",
      "shrinkImage.ecl",    
          "Raw_data_3C",     
              "Raw_data_4C",  
                  Raw_data_4CFieldList,  
                      0, 
                          NULL },
// </block>
  { NULL, NULL, NULL, NULL, NULL, 0, NULL }  };


// getFilterCode() -- pull the code that is the source for a filter. Currently,
// only works for retrieving code from a file. Returns NULL if no source found.
// Also, do NOT free returned buffer. That buffer is owned by this functions.
static char *getFilterCode( char *filterName )
{
  filterInfo *curr;
  char *location = NULL;

  // conserve our buffer allocation
  // create a "grow only" strategy for get larger enough buffer
  static char *filterSourceBuffer;
  // remember buffer size forever
  static int bufferSize = 0;

  // read new each time
  int totalRead = 0, lastRead = 0;

  // FILE ptr we read from
  FILE *srcFile;

  // first, get URI of buffer location, assume file for now
  while( curr->name != NULL )
  {
    if( !strcmp( curr->name, filterName ) )
      location = curr->uri;
  }

  // if can't find URI, return NULL
  if( location == NULL )
    return NULL;

  // open for read ing the sourcefile
  srcFile = fopen( location, "r" );
  if( srcFile == NULL )
  {
    // if can't get file, dump out
    fprintf(stderr, "Can't open filter file.");
    return NULL;
  }


  // read to end of file
  while( !feof( srcFile ) )
  {
    // check to see how much we've read.
    // if total read this time is size of buffer, we need more buffer space,
    // so allocate another 4K (one page)
    if( bufferSize == totalRead )
    {
      bufferSize += 4096;
      filterSourceBuffer = (char *)realloc( filterSourceBuffer, bufferSize );
      if( !filterSourceBuffer )
      {
        fprintf(stderr, "Can't allocate space for buffer source." );
        return NULL;
      }
    }
    lastRead = fread( filterSourceBuffer + totalRead, bufferSize - totalRead, (size_t) 1, srcFile);
    totalRead += lastRead;
  }

  if( totalRead == bufferSize )
  {
    bufferSize += 4096;
    filterSourceBuffer = (char *)realloc( filterSourceBuffer, bufferSize );
    if( !filterSourceBuffer )
    {
      fprintf(stderr, "Can't allocate space for buffer source -- 2." );
      return NULL;
    }
  }

  // null terminate the source
  filterSourceBuffer[ totalRead ] = 0;
  printf( "Source:\n%s\n", filterSourceBuffer );

  // close the file
  fclose(srcFile);

  // done
  return filterSourceBuffer;
}

// getFilterOutType() -- returns the string name of the output type of the 
// -- filter named filterName, NULL if not found in
// -- the list
static char *getFilterOutType( char *filterName )
{
  filterInfo *curr;
  while( curr->name != NULL )
    {
    if( !strcmp( curr->name, filterName ) )
      return curr->outType;
    }
  return NULL;
}

// getFilterOutTypePtr() -- returns a pointer to the output type field list of the 
// -- filter named filterName, NULL if not found in
// -- the list
static IOFieldList getFilterOutTypePtr( char *filterName )
{
  filterInfo *curr;
  while( curr->name != NULL )
    {
    if( !strcmp( curr->name, filterName ) )
      return curr->outTypePtr;
    }
  return NULL;
}


// getFilterInType() -- returns the string name of the input type of the 
// -- filter named filterName, NULL if not found in
// -- the list
static char *getFilterInType( char *filterName )
{
  filterInfo *curr;
  while( curr->name != NULL )
    {
    if( !strcmp( curr->name, filterName ) )
      return curr->inType;
    }
  return NULL;
}

// getFilterParamsTypeList() -- returns the name IOFieldList for the params of the 
// -- filter named filterName, NULL if not found in
// -- the list
static IOFieldList getFilterParamsTypeList( char *filterName )
{
  filterInfo *curr;
  while( curr->name != NULL )
    {
    if( !strcmp( curr->name, filterName ) )
      return curr->paramsTypeList;
    }
  return NULL;
}

// getFilterTakesParams() -- this function looks up whether a filter 
// -- requires parameters be supplied to function correctly. If so,
// -- then the installFilter_receiver() function expect that 
// -- params be provided
static int getFilterTakesParams( char *filterName )
{
  filterInfo *curr;
  while( curr->name != NULL )
    {
    if( !strcmp( curr->name, filterName ) )
      return curr->takesParams;
    }
  return -1;
}


// filterCheck( ) -- checks to see if a filter will work
// -- on the current incoming data stream. Primarily, this
// -- is a type check. I.e. a filter is a typed object
// -- and its input type must match the type coming from
// -- our source.
static int filterCheck( char *filterName ) { return 1; }

// we need to keep track of all the filters installed on this channel
// -- In doing so, we need to know the name of a filter, the ID under
// -- which it publishes its data, and any filters on top of which
// -- it may be stacked.
struct filterStackElementStruct
{
  char *filterName;
  char *connToChanId;
  struct filterStackElementStruct *next;
};
typedef struct filterStackElementStruct filterStackElement;
typedef struct filterStackElementStruct *filterStackElementPtr;

// We need to keep track of all filters installed so we
// -- keep a list of them on a stack. This lets us un-install
// -- them later one at a time and we will still know what
// -- type to expect.
static filterStackElementPtr filterStackTop = NULL; 
    
// getFilterStackString() -- returns a string which lists the filters
// -- in the current filter stack. Each is listed by filterName
// -- and filterName's are separated by ':'
char *getFilterStackString(){
  char *buf; // holds result 
  int bufsize;
  int totalLen = 0;
  int partialLen;
  filterStackElementPtr current = filterStackTop;

  // allocate a space for string and term to len 0
  buf = (char *)malloc( sizeof(char) * 100 );
  if( !buf ) {
    fprintf(stderr, "getFilterStackString(): out-of-mem err[2].");
  }
  buf[0] = 0;
  bufsize = 0;
  totalLen = 0;

  while( current ){
    // be sure we have space in buffer
    partialLen = strlen( current->filterName );
    while( bufsize <= (totalLen + partialLen + 2) ) { 
                                       // plus 2 for NULL term and ':'
      buf = (char *)realloc( buf, 100 );
      if( !buf ) {
        fprintf(stderr, "getFilterStackString(): out-of-mem err[2].");
      }
    }

    // append separator (if not first)
    if( totalLen > 0 )
      strcat( buf, ":" );

    // append string
    strcat( buf, current->filterName );
  }
  
  return buf;
}




// installFilter_receiver() -- adds a filter to the current
// -- incoming data stream. If a filter is already installed, this
// -- will install a filter "stacked" on top of the older filter.
// -- you can query the stack with other functions.
// filterName - name of filter to install
// filterParams - params that may be needed by the filter for it
//              - to function. If no params to be passed then
//              - send null.
// -- returns 0 if successful, non-zero otherwise
int installFilter_receiver( char *filterName, void *filterParams )
{
  char lookupStr[512];
  char *filterCode;
  filterStackElementPtr fse;
  EChannel filteredChannel;
  ECdata_struct paramSpec;
  

  // check to be sure the filter will work on this stream
  // (not implemented)
  if(!filterCheck( filterName )) { 
    fprintf(stderr, "Filter incompatible with existing stream type." ); 
    return -1; 
  }


  // add filter to stack of filters
  // allocate for stack
  fse = (filterStackElementPtr) malloc( sizeof( filterStackElement) );
  if( fse == NULL ) { 
    fprintf(stderr, "Error no mem."); 
    return -1; 
  }

  // save the name
  fse->filterName = (char *)malloc( strlen( filterName ) + 1 );
  if( fse->filterName == NULL ) {
    fprintf(stderr, "Error no mem."); free( fse ); 
    return -1; 
  }
  strcpy( fse->filterName, filterName );

  // save the old channelId
  fse->connToChanId = (char *)malloc( strlen( incomingGlobalID ) + 1 );
  strcpy( fse->connToChanId, incomingGlobalID );
  
  // stack it up
  fse->next = filterStackTop;
  filterStackTop = fse;

  // load filter code from URI
  filterCode = getFilterCode(filterName);


  // 3 cases:
  // 1) filter is boolean - output type is input type
  // 2) filter has new output type
  // 3) filter has new output type AND is paramterized


  // *****CASE 1***** boolean filter
  if( !getFilterOutTypePtr( filterName ) ) {
    filteredChannel = EChannel_typed_derive( ec, fse->connToChanId, filterCode, getFilterOutTypePtr( filterName ), NULL );

  }
  else if( !getFilterTakesParams(filterName) ){
    filteredChannel = EChannel_typed_derive_data( ec, fse->connToChanId, filterCode, getFilterOutTypePtr( filterName ), NULL, filterParams );
     
  }
  // is it a filter that changes data or is simply boolean?
  else {

    // check to be sure params there
    if( filterParams == NULL )
    {
      fprintf(stderr, "No params supplied. Params needed.\n");
      return 1;
    }
/*
    // allocate param struct
    if( !paramStruct ) {
      paramStruct = (ECdata_spec) malloc( sizeof( ECdata_struct ) );
      if( !paramStruct ) {
        fprintf( stderr, "Unable to allocate mem needed for parameterized filtering.\n");
        return 1;       
      }
    }
*/
    // set up our params struct
    paramSpec.data_field_list = getFilterParamsTypeList( filterName );
    paramSpec.data_subformat_list = NULL;
    paramSpec.initial_value = filterParams;
    // derive
    filteredChannel = EChannel_typed_derive_data( ec, fse->connToChanId, filterCode, getFilterOutTypePtr( filterName ), NULL, &paramSpec );

  }
  
  ECsink_typed_subscribe( filteredChannel, 
                           getFilterOutTypePtr(filterName),
                            NULL, receiver__receive1Handler, NULL);


  // 0 for success
  return 0;
}

int uninstallLastFilter_receiver( void )
{

  // uninstalls the last filter installed
  filterStackElementPtr fse;

  // do nothing if no filter installed at all
  if( filterStackTop == NULL ) return 0;

  // pop from stack
  fse = filterStackTop;
  filterStackTop = filterStackTop->next;

  // release space
  free( fse->filterName );
  free( fse->connToChanId );
  free( fse );

  return 0;
}


  // end Filtering code

void receiver__receive1Handler( void *, void *, attr_list);


int firstTime = 1;
/* start this sub-pipe */
void infopipeStart_receiver(  )
{
  char lookupStr[512];

  if( firstTime )
  {
    firstTime = 0;
    doPipeInit();
  }



  /////////////////////////////////
  // INCOMING LINK -- open channel
  incomingGlobalID = lookup("receiver","","receive1", PUBLISH_FILE, lookupStr);
  receiver__receive1ReceiveChannel = EChannel_open( ec,  incomingGlobalID);

  printf("lookup string = %s\n", lookupStr);
  // subscribe as a sink on all incoming channels
  ECsink_typed_subscribe( receiver__receive1ReceiveChannel, Raw_dataFieldList, NULL, receiver__receive1Handler, NULL);


  //ec = ECho_CM_init();
  CMrun_network(cm);
}

// shuts down the infopipe system and cleans out directory entries
void infopipeStop_receiver( )
{

// shutdown our outgoing pipes - the incoming pipes will
// be shutdown by the nodes upstream from us


}


/////////////////////////////////////////////////////////////////////////////
// handler function for an incoming pipe 
// This handler function accepts data coming from the pipe. It runs
// once for each set of data that comes into this machine from
// the pipe/channel.
void receiver__receive1Handler( void *event, void *client_data, attr_list as)
{
  int li;
  int lnlr;
  // Typecast incoming data to workable type
  //Raw_dataCarrierStruct *incomingData = (Raw_dataCarrierStruct *)event;
  Raw_dataStruct_ptr incomingData = (Raw_dataStruct *)event;

  

  
  // USER: Add code here to operate on incoming data
  // memory pointed to by incomingData (and event) may be reclaimed by calling 

}

// ENDING OF IMP FILE FOR A NODE

int test_main( int argc, char *argv )
{
  //infopipeEnvironStartup();
  // start the pipe
  infopipeStart_receiver();
  
  while( 1 )
    sleep( 1 );

  // close down the pipes
  infopipeStop_receiver();

  // normal termination
  return 0;
}

// </block>
