# ---
# sip2xip.py - a lex file for SIP-II
#

import lex

#get file ready
#lexFile = open('./siptest.sip')
#lexin = lexFile.read()

import xipDoc
treeRoot = xipDoc.xipDoc(None,None,None,None) 

#tokens we accept
tokens = ( #'NL', # a newline
           'END', #semi-colon an end of a statement
           'DATATYPE', #datatype declaration
           'ID', #identifier for a type or a pipe
           'LPAREN', 'RPAREN',
           'LBRACE', 'RBRACE',
           'COLON', 
           'COMMA', 
           'PERIODMARK', 
           'RARROW', 
           'FLOAT', 'INTEGER', 'BYTE', 'CHAR', 'STRING',
           'PIPE', 'PIPES',
           'COMPOSEDPIPE',
           'INPORTS', 'OUTPORTS',
           'CONNECTIONS',
           'COMPOSE') 

t_LPAREN = r'\('
t_RPAREN = r'\)'
t_LBRACE = r'\{'
t_RBRACE = r'\}'
t_COLON  = r':'
t_COMMA = r','
t_PERIODMARK = r'.'
t_END = r';'
t_RARROW = r'->'

reserved = { 'datatype' : 'DATATYPE', 
             'float' : 'FLOAT',
             'integer' : 'INTEGER',
             'char' : 'CHAR',
             'string' : 'STRING',
             'byte' : 'BYTE',
             'pipe' : 'PIPE',
             'pipes' : 'PIPES',
             'composedPipe' : 'COMPOSEDPIPE',
             'inports' : 'INPORTS',
             'outports' : 'OUTPORTS',
             'connections' : 'CONNECTIONS',
             'compose' : 'COMPOSE' }
             

def t_ID(t):
  r'[a-zA-Z0-9_]+'
  t.type = reserved.get(t.value, 'ID')
  return t

def t_NL(t):
  r'\n+'
  t.lineno += len(t.value)
  print 'newline, line no.', t.lineno, 't.value:',t.value
  return t

t_ignore = ' \t\n'

def t_error(t):
  print "Illegal Character '%s'" % t.value
  t.skip(1)


siplexer = lex.lex()

#siplexer.input(lexin)

#while 1:
#  tok = lex.token()
#  if not tok: 
#    print 'not tok',tok
#    break
#  print tok

#print
#print 'Doing yacc now'

#print
# build our yacc rules

# GRAMMAR
# sip :  types filters pipes sip
# datatypes : empty | datatype datatypes
# datatype : 'datatype' ID '{' varlist '}'
# varlist : onevar varlist | onevar
# onevar : ID ':' ptype ';'
# ptype : 'float' | 'integer' | 'byte' | 'string'
# ID : [a-zA-Z_][a-zA-Z0-9_]*


def p_sip(t):
  'sip : datatypes pipes composed_pipe'
  #print 'sip'
  treeRoot.datatypes = t[1] 
  treeRoot.pipes = t[2] 
  treeRoot.composedPipe = t[3]
  t[0] = treeRoot

def p_datatypes(t):
  '''datatypes : datatype datatypes 
               | empty
               '''
  #print 'p_datatypes'
  if t[1] != None:
    t[0] = xipDoc.objectList(treeRoot, 'datatype', t[1], t[2])

def p_datatype(t):
  'datatype : DATATYPE ID LBRACE args RBRACE'
  #print 'p_datatype'
  t[0] = xipDoc.datatypeNode(treeRoot, t[2], t[4])

def p_args(t):
  '''args : arg args
          | empty
          '''
  #print 'p_args'
  if t[1] != None:
    t[0] = xipDoc.objectList(treeRoot, 'arg', t[1], t[2])

def p_arg(t):
  'arg : ID COLON ptype'
  #print 'p_arg'
  t[0] = xipDoc.argNode(treeRoot, t[1], t[3])

def p_ptype_integer(t):
  'ptype : INTEGER'
  #print 'p_ptype'
  t[0] = t[1]

def p_ptype_float(t):
  'ptype : FLOAT'
  t[0] = t[1]

def p_ptype_string(t):
  'ptype : STRING'
  t[0] = t[1]

def p_ptype_char(t):
  'ptype : CHAR'
  t[0] = t[1]

def p_ptype_byte(t):
  'ptype : BYTE'
  t[0] = t[1]

# pipes : empty | pipe pipes
# pipe : 'pipe' ID '{' inports outports '}'
# inports : empty | 'inports' ports
# ports : '{' portlist '}'
# portlist : empty | oneport portlist
# oneport : ID ':' ID (active | passive) ';'

def p_pipes(t):
  '''pipes : pipe pipes
           | empty
           '''
  if t[1] != None:
    t[0] = xipDoc.objectList(treeRoot, 'pipe', t[1],t[2])

def p_pipe(t):
  'pipe : PIPE ID LBRACE inportlist outportlist RBRACE'
  #print 'p_pipe'
  t[0] = xipDoc.pipeNode(treeRoot, t[2], t[4], t[5])

def p_inportlist(t):
  '''inportlist : INPORTS LBRACE inports RBRACE
                | empty
                '''
  #print 'p_inportlist'
  if t[1] != None:
    t[0] = t[3]

def p_inports(t):
  '''inports : inport inports
             | empty
             '''
  if t[1] != None:
    t[0] = xipDoc.objectList(treeRoot, 'port', t[1], t[2])

def p_inport(t):
  'inport : ID COLON ID'
  t[0] = xipDoc.portNode(treeRoot, t[1], 'in', t[3], 'active')

def p_outportlist(t):
  '''outportlist : OUTPORTS LBRACE outports RBRACE
                 | empty
                 '''
  #print 'p_outportlist'
  if t[1] != None:
    t[0] = t[3]

def p_outports(t):
  '''outports : outport outports
              | empty
              '''
  #print 'p_outports'
  if t[1] != None:
    t[0] = xipDoc.objectList(treeRoot, 'port', t[1], t[2])

def p_outport(t):
  'outport : ID COLON ID'
  t[0] = xipDoc.portNode(treeRoot, t[1], 'out', t[3], 'active')


# composedPipe : compsedPipe ID { pipe_decl_list inport_decl_list outport_decl_list conn_list aspect_decl_list }
# pipe_decl_list : pipes { pipe_decls }
# inport_decl_list : inports { inport_decls }
# outport_decl_list : outports { outport_decls }
# conn_list : connections { conn_decls }
# aspect_decl_list : aspects { aspect_decls } 
def p_composedPipe(t):
  'composed_pipe : COMPOSEDPIPE ID LBRACE pipe_decl_list inport_decl_list outport_decl_list conn_list RBRACE'
  #'composedPipe : COMPOSEDPIPE ID LBRACE pipe_decl_list inport_decl_list outport_decl_list conn_list aspect_decl_list RBRACE'
  #print 'composedPipe'
  t[0] = xipDoc.composedPipeNode(treeRoot, t[2], t[4], t[5], t[6], t[7])

def p_pipe_decl_list(t):
  '''pipe_decl_list : PIPES LBRACE pipe_decls RBRACE
                    | empty
                    '''
  if t[1] != None:
    t[0] = t[3]

def p_pipe_decls(t):
  '''pipe_decls : pipe_decl pipe_decls 
                | empty
                '''
  if t[1] != None:
    t[0] = xipDoc.objectList(treeRoot, 'pipe_decl', t[1], t[2])

def p_pipe_decl(t):
  'pipe_decl : ID COLON ID'
  t[0] = xipDoc.pipeDeclNode(treeRoot, t[1], t[3])

def p_inport_decl_list(t):
  '''inport_decl_list : INPORTS LBRACE inport_decls RBRACE
                      | empty
                      '''
  if t[1] != None:
    t[0] = t[3]

def p_inport_decls(t):
  '''inport_decls : inport_decl inport_decls
                  | empty
                  '''
  if t[1] != None:
    t[0] = xipDoc.objectList(treeRoot, 'port_decl', t[1], t[2])

def p_inport_decl(t):
  'inport_decl : ID COLON ID PERIODMARK ID'
  t[0] = xipDoc.portDeclNode(treeRoot, t[1], t[3], t[5], 'in')

def p_outport_decl_list(t):
  '''outport_decl_list : OUTPORTS LBRACE outport_decls RBRACE
                       | empty
                       '''
  if t[1] != None:
    t[0] = t[3]

def p_outport_decls(t):
  '''outport_decls : outport_decl outport_decls
                   | empty
                   '''
  if t[1] != None:
    t[0] = xipDoc.objectList(treeRoot, 'port_decl', t[1], t[2])

def p_outport_decl(t):
  'outport_decl : ID COLON ID PERIODMARK ID'
  t[0] = xipDoc.portDeclNode(treeRoot, t[1], t[3], t[5], 'out')

def p_conn_list(t):
  '''conn_list : CONNECTIONS LBRACE connections RBRACE
               | empty
               '''
  if t[1] != None:
    t[0] = t[3]

def p_connections(t):
  '''connections : connection connections
                 | empty
                 '''
  if t[1] != None:
    t[0] = xipDoc.objectList(treeRoot, 'connection', t[1], t[2])

def p_connection(t):
  'connection : ID PERIODMARK ID RARROW ID PERIODMARK ID'
  t[0] = xipDoc.connectionNode(treeRoot, t[1], t[3], t[5], t[7])

def p_empty(t):
  'empty :'

def p_error(t):
  print 'Syntax error in input! Choked on ', t


import yacc
sipparser = yacc.yacc()
#result = sipparser.parse(lexin)

#print result
#print 'Printing XIP'
#print result.Xml()
